unit ActiveShape;

interface

uses
  Windows, Types, Messages, SysUtils, Classes, Controls, Graphics;

type
  TActiveShape = class(TGraphicControl)
  private
    FAlignment: TAlignment;
    FTransparent: Boolean;
    FOverColor: TColor;
    FOverFont: TFont;
    FBorderWidth: Integer;
    FOverBorderWidth: Integer;
    FBorderColor: TColor;
    FOverBorderColor: TColor;
    FMouseOver: Boolean;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CMColorChanged(var Message: TMessage); message CM_COLORCHANGED;
    procedure SetAlignment(const Value: TAlignment);
    procedure SetBorderColor(const Value: TColor);
    procedure SetBorderWidth(const Value: Integer);
    procedure SetOverBorderColor(const Value: TColor);
    procedure SetOverBorderWidth(const Value: Integer);
    procedure SetOverColor(const Value: TColor);
    procedure SetOverFont(const Value: TFont);
    procedure SetTransparent(const Value: Boolean);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Alignment: TAlignment read FAlignment write SetAlignment;
    property Transparent: Boolean read FTransparent write SetTransparent;
    property Caption;
    property Color;
    property OverColor: TColor read FOverColor write SetOverColor;
    property Font;
    property OverFont: TFont read FOverFont write SetOverFont;
    property BorderWidth: Integer read FBorderWidth write SetBorderWidth;
    property OverBorderWidth: Integer read FOverBorderWidth write
      SetOverBorderWidth;
    property BorderColor: TColor read FBorderColor write SetBorderColor;
    property OverBorderColor: TColor read FOverBorderColor write
      SetOverBorderColor;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseWheel;
    property OnMouseWheelDown;
    property OnMouseWheelUp;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Vampyre Shared', [TActiveShape]);
end;

function Iff(const Condition: Boolean; const TruePart, FalsePart: Integer):
  Integer;
begin
  if Condition then
    Result := TruePart
  else
    Result := FalsePart;
end;

{ TActiveShape }

procedure TActiveShape.CMColorChanged(var Message: TMessage);
begin
  inherited;
  Invalidate;
end;

procedure TActiveShape.CMFontChanged(var Message: TMessage);
begin
  inherited;
  Invalidate;
end;

procedure TActiveShape.CMMouseEnter(var Message: TMessage);
begin
  inherited;
  FMouseOver := True;
  Invalidate;
end;

procedure TActiveShape.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  FMouseOver := False;
  Invalidate;
end;

procedure TActiveShape.CMTextChanged(var Message: TMessage);
begin
  inherited;
  Invalidate;
end;

constructor TActiveShape.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FOverColor := Color;
  FBorderWidth := 1;
  FOverBorderWidth := 1;
  FBorderColor := clBlack;
  FOverBorderColor := clBlack;
  FTransparent := False;
  Font.Name := 'Verdana';
  FOverFont := TFont.Create;
  FOverFont.Assign(Font);
  FAlignment := taCenter;
  FMouseOver := False;
end;

procedure TActiveShape.Paint;
const
  Alignments: array[TAlignment] of Longint = (DT_LEFT, DT_RIGHT, DT_CENTER);
var
  Rect: TRect;
  FontHeight, Flags: Integer;
begin
  Rect := GetClientRect;
  Canvas.Brush.Color := Iff(FMouseOver, FOverColor, Color);
  Canvas.Pen.Width := Iff(FMouseOver, FOverBorderWidth, FBorderWidth);
  if Canvas.Pen.Width = 0 then
    Canvas.Pen.Color := Canvas.Brush.Color
  else
    Canvas.Pen.Color := Iff(FMouseOver, FOverBorderColor, FBorderColor);
  if FTransparent then
    Canvas.Brush.Style := bsClear;
  if not Odd(Canvas.Pen.Width) then
    InflateRect(Rect, -1, -1);
  Canvas.Rectangle(Rect);
  if FMouseOver then
    Canvas.Font.Assign(FOverFont)
  else
    Canvas.Font.Assign(Font);
  FontHeight := Canvas.TextHeight('W');
  with Rect do
  begin
    Top := ((Bottom + Top) - FontHeight) div 2;
    Bottom := Top + FontHeight;
  end;
  Flags := DT_EXPANDTABS or DT_VCENTER or Alignments[FAlignment];
  DrawText(Canvas.Handle, PChar(Caption), -1, Rect, Flags);
end;

procedure TActiveShape.SetAlignment(const Value: TAlignment);
begin
  FAlignment := Value;
  Invalidate;
end;

procedure TActiveShape.SetBorderColor(const Value: TColor);
begin
  FBorderColor := Value;
  Invalidate;
end;

procedure TActiveShape.SetBorderWidth(const Value: Integer);
begin
  FBorderWidth := Value;
  Invalidate;
end;

procedure TActiveShape.SetOverBorderColor(const Value: TColor);
begin
  FOverBorderColor := Value;
  Invalidate;
end;

procedure TActiveShape.SetOverBorderWidth(const Value: Integer);
begin
  FOverBorderWidth := Value;
  Invalidate;
end;

procedure TActiveShape.SetOverColor(const Value: TColor);
begin
  FOverColor := Value;
  Invalidate;
end;

procedure TActiveShape.SetOverFont(const Value: TFont);
begin
  FOverFont.Assign(Value);
  Invalidate;
end;

procedure TActiveShape.SetTransparent(const Value: Boolean);
begin
  FTransparent := Value;
  Invalidate;
end;

end.

