unit VideVclActions;

interface

uses
  Windows, SysUtils, Classes,
  Controls, StdCtrls, Clipbrd, ActnList, StdActns, SynEdit;

type

  TVideTextAction = class(TAction)
  protected
    function GetControl(Target: TObject): TCustomEdit; virtual;
    function GetSynEdit(Target: TObject): TCustomSynEdit; virtual;
  public
    function HandlesTarget(Target: TObject): Boolean; override;
    procedure UpdateTarget(Target: TObject); override;
  end;

  { Edit Actions }

  TVideEditCut = class(TVideTextAction)
  public
    procedure ExecuteTarget(Target: TObject); override;
  end;

  TVideEditCopy = class(TVideTextAction)
  public
    procedure ExecuteTarget(Target: TObject); override;
  end;

  TVideEditDelete = class(TVideTextAction)
  public
    procedure ExecuteTarget(Target: TObject); override;
  end;

  TVideEditPaste = class(TVideTextAction)
  public
    procedure UpdateTarget(Target: TObject); override;
    procedure ExecuteTarget(Target: TObject); override;
  end;

  TVideEditSelectAll = class(TVideTextAction)
  public
    procedure UpdateTarget(Target: TObject); override;
    procedure ExecuteTarget(Target: TObject); override;
  end;

  TVideEditUndo = class(TVideTextAction)
  public
    procedure UpdateTarget(Target: TObject); override;
    procedure ExecuteTarget(Target: TObject); override;
  end;

  TVideEditRedo = class(TVideTextAction)
  public
    procedure UpdateTarget(Target: TObject); override;
    procedure ExecuteTarget(Target: TObject); override;
  end;

  { Formatting Actions }

  TVideFormatLowerCase = class(TVideTextAction)
  public
    procedure ExecuteTarget(Target: TObject); override;
  end;

  TVideFormatUpperCase = class(TVideTextAction)
  public
    procedure ExecuteTarget(Target: TObject); override;
  end;


procedure Register;

implementation

procedure Register;
begin
  RegisterActions('Edit', [TVideEditCut, TVideEditCopy, TVideEditDelete,
    TVideEditPaste, TVideEditSelectAll, TVideEditUndo, TVideEditRedo], nil);
  RegisterActions('Format', [TVideFormatLowerCase, TVideFormatUpperCase], nil);
end;

{ TVideTextAction }

function TVideTextAction.GetControl(Target: TObject): TCustomEdit;
begin
  Result := Target as TCustomEdit;
end;

function TVideTextAction.GetSynEdit(Target: TObject): TCustomSynEdit;
begin
  Result := Target as TCustomSynEdit;
end;

function TVideTextAction.HandlesTarget(Target: TObject): Boolean;
begin
  Result := ((Target is TCustomEdit) or (Target is TCustomSynEdit))
    and TCustomControl(Target).Focused;
end;

procedure TVideTextAction.UpdateTarget(Target: TObject);
begin
  if Target is TCustomEdit then
    Enabled := GetControl(Target).SelLength > 0
  else
    Enabled := GetSynEdit(Target).SelAvail;
end;

{ TVideEditCut }

procedure TVideEditCut.ExecuteTarget(Target: TObject);
begin
  if Target is TCustomEdit then
    GetControl(Target).CutToClipboard
  else
    GetSynEdit(Target).CutToClipboard;
end;

{ TVideEditCopy }

procedure TVideEditCopy.ExecuteTarget(Target: TObject);
begin
  if Target is TCustomEdit then
    GetControl(Target).CopyToClipboard
  else
    GetSynEdit(Target).CopyToClipboard;
end;

{ TVideEditDelete }

procedure TVideEditDelete.ExecuteTarget(Target: TObject);
begin
  if Target is TCustomEdit then
    GetControl(Target).ClearSelection
  else
    GetSynEdit(Target).ClearSelection;
end;

{ TVideEditPaste }

procedure TVideEditPaste.ExecuteTarget(Target: TObject);
begin
  if Target is TCustomEdit then
    GetControl(Target).PasteFromClipboard
  else
    GetSynEdit(Target).PasteFromClipboard;
end;

procedure TVideEditPaste.UpdateTarget(Target: TObject);
begin
  Enabled := Clipboard.HasFormat(CF_TEXT);
end;

{ TVideEditSelectAll }

procedure TVideEditSelectAll.ExecuteTarget(Target: TObject);
begin
  if Target is TCustomEdit then
    GetControl(Target).SelectAll
  else
    GetSynEdit(Target).SelectAll;
end;

procedure TVideEditSelectAll.UpdateTarget(Target: TObject);
begin
  if Target is TCustomEdit then
    Enabled := Length(GetControl(Target).Text) > 0
  else
    Enabled := True;
end;

{ TVideEditUndo }

procedure TVideEditUndo.ExecuteTarget(Target: TObject);
begin
  if Target is TCustomEdit then
    GetControl(Target).Undo
  else
    GetSynEdit(Target).Undo;
end;

procedure TVideEditUndo.UpdateTarget(Target: TObject);
begin
  if Target is TCustomEdit then
    Enabled := GetControl(Target).CanUndo
  else
    Enabled := GetSynEdit(Target).CanUndo;
end;

{ TVideEditRedo }

procedure TVideEditRedo.ExecuteTarget(Target: TObject);
begin
  if Target is TCustomSynEdit then
    GetSynEdit(Target).Redo;
end;

procedure TVideEditRedo.UpdateTarget(Target: TObject);
begin
  if Target is TCustomSynEdit then
    Enabled := GetSynEdit(Target).CanRedo;
end;

{ TVideFormatLowerCase }

procedure TVideFormatLowerCase.ExecuteTarget(Target: TObject);
begin
 if Target is TCustomEdit then
    GetControl(Target).SelText := AnsiLowerCase(GetControl(Target).SelText)
  else
    GetSynEdit(Target).SelText := AnsiLowerCase(GetSynEdit(Target).SelText);
end;

{ TVideFormatUpperCase }

procedure TVideFormatUpperCase.ExecuteTarget(Target: TObject);
begin
 if Target is TCustomEdit then
    GetControl(Target).SelText := AnsiUpperCase(GetControl(Target).SelText)
  else
    GetSynEdit(Target).SelText := AnsiUpperCase(GetSynEdit(Target).SelText);
end;


end.

