unit WizardHeader;

interface

uses
  Windows, Types, Messages, SysUtils, Classes, Controls, Graphics;

type
  TWizardHeader = class(TGraphicControl)
  private
    FComments: TStrings;
    FCaptions: TStrings;
    FActivePage: Integer;
    FGlyph: TPicture;
    FBottomBorderWidth: Integer;
    FBottomBorderColor: TColor;
    procedure SetActivePage(const Value: Integer);
    procedure SetCaptions(const Value: TStrings);
    procedure SetComments(const Value: TStrings);
    procedure SetGlyph(const Value: TPicture);
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CMColorChanged(var Message: TMessage); message CM_COLORCHANGED;
    procedure SetBottomBorderColor(const Value: TColor);
    procedure SetBottomBorderWidth(const Value: Integer);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Align default alTop;
    property Anchors;
    property Color;
    property Font;
    property ParentFont;
    property ActivePage: Integer read FActivePage write SetActivePage;
    property Captions: TStrings read FCaptions write SetCaptions;
    property Comments: TStrings read FComments write SetComments;
    property Glyph: TPicture read FGlyph write SetGlyph;
    property BottomBorderWidth: Integer read FBottomBorderWidth write
      SetBottomBorderWidth;
    property BottomBorderColor: TColor read FBottomBorderColor write
      SetBottomBorderColor;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Vampyre Shared', [TWizardHeader]);
end;

{ TWizardHeader }

procedure TWizardHeader.CMColorChanged(var Message: TMessage);
begin
  inherited;
  Invalidate;
end;

procedure TWizardHeader.CMFontChanged(var Message: TMessage);
begin
  inherited;
  Invalidate;
end;

constructor TWizardHeader.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csOpaque];
  ParentColor := False;
  Color := clWindow;
  Align := alTop;
  FActivePage := 0;
  Height := 60;
  FBottomBorderWidth := 1;
  FBottomBorderColor := clBtnShadow;
  FCaptions := TStringList.Create;
  FComments := TStringList.Create;
  FGlyph := TPicture.Create;
end;

destructor TWizardHeader.Destroy;
begin
  FGlyph.Free;
  FComments.Free;
  FCaptions.Free;
  inherited Destroy;
end;

procedure TWizardHeader.Paint;
var
  Caption: string;
  Comment: string;
  Rect: TRect;
  TextHeight: Integer;
begin
  Rect := ClientRect;
  Canvas.Brush.Color := Color;
  Canvas.Pen.Color := Color;
  Canvas.Rectangle(Rect);
  Inc(Rect.Left, 10);
  Inc(Rect.Top, 6);
  if FActivePage < FCaptions.Count then
    Caption := FCaptions[FActivePage]
  else
    Caption := 'Caption';
  if FActivePage < FComments.Count then
    Comment := FComments[FActivePage]
  else
    Comment := 'Comment';
  Canvas.Font.Assign(Font);
  Canvas.Font.Style := Canvas.Font.Style + [fsBold];
  TextHeight := Canvas.TextHeight('W');
  DrawText(Canvas.Handle, PChar(Caption), -1, Rect, 0);
  Inc(Rect.Left, 10);
  Inc(Rect.Top, TextHeight + 2);
  Dec(Rect.Right, FGlyph.Width + 8);
  Canvas.Font.Style := Canvas.Font.Style - [fsBold];
  DrawText(Canvas.Handle, PChar(Comment), -1, Rect, DT_WORDBREAK);
  if FGlyph.Graphic <> nil then
  begin
    Rect := ClientRect;
    Dec(Rect.Right, 2);
    Inc(Rect.Left, Width - FGlyph.Width - 4);
    Inc(Rect.Top, (Height - FGlyph.Height) div 2);
    Rect.Bottom := Rect.Top + FGlyph.Height;
    Canvas.Draw(Rect.Left, Rect.Top, FGlyph.Graphic);
  end;
  Rect := ClientRect;
  Canvas.Pen.Color := FBottomBorderColor;
  Canvas.Pen.Width := FBottomBorderWidth;
  Canvas.MoveTo(Rect.Left, Rect.Bottom - FBottomBorderWidth);
  Canvas.LineTo(Rect.Right, Rect.Bottom - FBottomBorderWidth);
end;

procedure TWizardHeader.SetActivePage(const Value: Integer);
begin
  if FActivePage <> Value then
  begin
    FActivePage := Value;
    Invalidate;
  end;
end;

procedure TWizardHeader.SetBottomBorderColor(const Value: TColor);
begin
  FBottomBorderColor := Value;
  Invalidate;
end;

procedure TWizardHeader.SetBottomBorderWidth(const Value: Integer);
begin
  FBottomBorderWidth := Value;
  Invalidate;
end;

procedure TWizardHeader.SetCaptions(const Value: TStrings);
begin
  FCaptions.Assign(Value);
  Invalidate;
end;

procedure TWizardHeader.SetComments(const Value: TStrings);
begin
  FComments.Assign(Value);
  Invalidate;
end;

procedure TWizardHeader.SetGlyph(const Value: TPicture);
begin
  FGlyph.Assign(Value);
  Invalidate;
end;

end.

