{
  Numalian Engine
  Demo Application: Compress

  Shows usage of TNCompression descendants.
}

program Compress;

{$APPTYPE CONSOLE}

{$I Compiler.inc}

uses
  {Base engine unit (also sets custom memory manager so it must be first).}
  Kernel,
  {File and data functions and classes.}
  Files,
  {Base classes.}
  Objects,
  {Platform class (created automatically) - acces to platform specific functions
   in platform independent manner.}
  Platforms,
  SysUtils,
{$IFDEF MSWINDOWS}
  CrtWin32;
{$ENDIF}
{$IFDEF LINUX}
CrtLinux;
{$ENDIF}

{Draws Numalian Logo from external file.}
procedure DrawLogo;
var
  {Our own string list.}
  List: Objects.TNStringList;
  I: Long;
begin
  List := TNStringList.Create;
  {Vfs transforms relative path to absolute path.}
  List.LoadFromFile(Vfs.GetFullFileName('Data/LogoASCII.con'));
  for I := 0 to List.Count - 1 do
  begin
    Write(List[I]);
    if Length(List[I]) < 80 then
      Write(sLineBreak);
  end;
  List.Free;
end;

const
  sIdent = #32#32;
  sTestFile = 'Tiles.tga';

var
  Data1, Data2, Data3: Pointer;
  Size1, Size2, Size3, I, Ratio: Long;
  T, TCom, TDecom: Int64;
  CMod: TNCompression;
  Comps: TNPointerList;
  F: TNBinaryFile;

begin
  {Load and register external modules in Bin\Modules\ directory.}
  ModuleRegister.LoadFromDirectory(Vfs.GetFullDirName('Modules', True));
  ClrScr;
  TextColor(LightCyan);
  DrawLogo;
  TextColor(LightGray);
  WriteLn('This demo application shows usage of TNCompression class descendants.');
  WriteLn;

  {Fills Comps list with all created and registered compression modules.}
  Comps := TNPointerList.Create;
  for I := 0 to Files.RegisteredCompressionsCount - 1 do
    Comps.Add(FindCompression(I));

  {Write list of registred compressions.}
  WriteLn('These compression modules are registered:');
  for I := 0 to Comps.Count - 1 do
    WriteLn(sIdent, I + 1, ') ', TNCompression(Comps[I]).FormatName);
  WriteLn;

  {Show testing prompt.}
  WriteLn('If you want to test speed and compress ratio of these modules, press RETURN');
  WriteLn('Press another key to exit application.');
  WriteLn('If you want to test compressions on another file, rename it to Tiles.tga');
  WriteLn('and copy it to ..\Data\ directory.');

  if ReadKey = #13 then
  begin
    {Load input data.}
    F := Vfs.Open(sTestFile);
    Size1 := F.Size;
    Data1 := F.ReadWholeFile;
    Vfs.Close(F);

    {Test each module's speed and compress ratio.}
    for I := 0 to Comps.Count - 1 do
    begin
      CMod := TNCompression(Comps[I]);
      WriteLn;
      WriteLn('Testing: ', CMod.FormatName);
      {Test compression speed.}
      T := Platform.GetTimeMicroseconds;
      CMod.Compress(Data1, Size1, Data2, Size2);
      TCom := Platform.GetTimeMicroseconds - T;
      {Test decompression speed.}
      T := Platform.GetTimeMicroseconds;
      CMod.DeCompress(Data2, Size2, Data3, Size3);
      TDecom := Platform.GetTimeMicroseconds - T;
      {Compute compress ratio.}
      Ratio := Round(Size2 / Size1 * 100);

      WriteLn(sIdent, 'Data compressed in:   ',
        FloatToStrF(TCom, ffNumber, 10, 0), ' us');
      WriteLn(sIdent, 'Data decompressed in: ',
        FloatToStrF(TDecom, ffNumber, 10, 0), ' us');
      WriteLn(sIdent, 'Compression ratio:    ', Ratio, '% (',
        Size2, '/', Size1, ' B)');

      FreeMemNil(Data2);
      FreeMemNil(Data3);

      if I < Comps.Count - 1 then
      begin
        WriteLn('Press any key to continue.');
        ReadKey;
      end;
    end;

    WriteLn;
    WriteLn('Testing completed. Press any key to exit.');
    ReadKey;

    FreeMemNil(Data1);
  end;

  Comps.Free;
end.

