{
  Numalian Engine
  Demo Application: Config

  Shows usage of TNConfigFile.
}

program Config;

{$APPTYPE CONSOLE}

{$I Compiler.inc}

uses
  {Base engine unit (also sets custom memory manager so it must be first).}
  Kernel,
  {File and data functions and classes.}
  Files,
  {Base classes.}
  Objects,
  {Platform class (created automatically) - acces to platform specific functions
   in platform independent manner.}
  Platforms,
  {Contains TNConfigFile for storage and management of configuration data.}
  Configs,
  SysUtils,
{$IFDEF MSWINDOWS}
  CrtWin32;
{$ENDIF}
{$IFDEF LINUX}
CrtLinux;
{$ENDIF}  

{Draws Numalian Logo from external file.}
procedure DrawLogo;
var
  {Our own string list.}
  List: Objects.TNStringList;
  I: Long;
begin
  List := TNStringList.Create;
  {Vfs transforms relative path to absolute path.}
  List.LoadFromFile(Vfs.GetFullFileName('Data/LogoASCII.con'));
  for I := 0 to List.Count - 1 do
  begin
    Write(List[I]);
    if Length(List[I]) < 80 then
      Write(sLineBreak);
  end;
  List.Free;
end;

var
  Cfg1, Cfg2: TNConfigFile;

begin
  ClrScr;
  TextColor(LightCyan);
  DrawLogo;
  TextColor(LightGray);
  WriteLn('This demo application shows usage of TNConfigFile for storage and management of configuration data.');
  WriteLn;

  {Create and load configs (note that ondisk .cfg files will not be updated
   when TNConfigFile is destroyed.).}
  Cfg1:=TNConfigFile.Create('Conf1.cfg', csText);
  Cfg2:=TNConfigFile.Create('Conf2.cfg', csText);
  WriteLn('Configs loaded.');
  {Write sample data to configs.}
  Cfg2.WriteInt('Items.Sword.Long.Damage',12);
  Cfg2.WriteFloat('Items.Sword.Long.Weight',3.5);
  Cfg2.WriteBool('CanCastSpells',True);
  WriteLn('Data written.');
  {Read sample data from configs.}
  WriteLn('Video.Renderer=', Cfg1.ReadString('Video.Renderer'));
  WriteLn('CanCastSpells=', Cfg2.ReadBool('CanCastSpells'));
  {This also works - all values can be read as strings.}
  WriteLn('CanCastSpells=', Cfg2.ReadString('CanCastSpells'));
  WriteLn('Data read.');
  {Add copy of config 2 to config 1 as sub node.
   Note that if you call Cfg1.AddConfig('Character', Cfg2) then
   Cfg2 becimes subnode of Cfg1 so you can not free it!}
  Cfg1.AddConfig('Character', Cfg2.Clone);
  {Save added config to Data\added.cfg}
  Cfg1.SaveToFile('Data\Added.cfg');
  WriteLn('Added config saved to ', Vfs.GetFullFileName('Data\Added.cfg'));
  {Merge both configs (Cfg1 has already clone of Cfg2 as one of subnodes).}
  Cfg2.CopyFromConfig(Cfg1);
  Cfg2.SaveToFile('Data\Merged.cfg');
  WriteLn('Merged config saved to ', Vfs.GetFullFileName('Data\Merged.cfg'));
  {Exit now.}
  WriteLn;
  WriteLn('Testing complete. Look at resulting files Added.cfg and Merged.cfg');
  WriteLn('Press any key to exit.');
  ReadKey;

  Cfg1.Free;
  Cfg2.Free;
end.
