{
  Numalian Engine
  Demo Application: Vfs

  Shows usage of various image manipulation functions and TNImageFileFormat
  descendants.
}

program ImageConvert;

{$APPTYPE CONSOLE}

{$I Compiler.inc}

uses
  {Base engine unit (also sets custom memory manager so it must be first).}
  Kernel,
  {File and data functions and classes.}
  Files,
  {Base classes.}
  Objects,
  {Platform class (created automatically) - acces to platform specific functions
   in platform independent manner.}
  Platforms,
  {Structures, classes and functions for working with images.}
  Images,
  SysUtils,
{$IFDEF MSWINDOWS}
  CrtWin32;
{$ENDIF}
{$IFDEF LINUX}
CrtLinux;
{$ENDIF}

const
  sIdent = #32#32;

{Draws Numalian Logo from external file.}
procedure DrawLogo;
var
  {Our own string list.}
  List: Objects.TNStringList;
  I: Long;
begin
  List := TNStringList.Create;
  {Vfs transforms relative path to absolute path.}
  List.LoadFromFile(Vfs.GetFullFileName('Data/LogoASCII.con'));
  for I := 0 to List.Count - 1 do
  begin
    Write(List[I]);
    if Length(List[I]) < 80 then
      Write(sLineBreak);
  end;
  List.Free;
end;

{Writes info about given image (resolution, format, ...)}
procedure ShowImageInfo(const Img: TImageData);
begin
  WriteLn(sIdent + 'Resolution:  ', Img.Levels[0].Width, 'x',
    Img.Levels[0].Height, ' Pixels');
  WriteLn(sIdent + 'BitCount:    ', Img.DataFormat.BitCount);
  WriteLn(sIdent + 'Data format: ', Img.DataFormat.Name);
  WriteLn(sIdent + 'Data size:   ', Img.Levels[0].Size, ' Bytes');
end;

var
  {Low-level image data structure.}
  Img: TImageData;
  {Pointer to structure which define data format of image (bitcount,
   pixel format, ...)}
  Dest: PImageDataFormat;
  {Class which loads/saves image data to some specific format (jpg, png, ...).}
  DestIFF: TNImageFileFormat;
  F: TNBinaryFile;
  I: Long;
  Formats: TNPointerList;
  S, Ext: string;

begin
  {Load and register external modules in Bin\Modules\ directory.}
  ModuleRegister.LoadFromDirectory(Vfs.GetFullDirName('Modules', True));
  ClrScr;
  TextColor(LightCyan);
  DrawLogo;
  TextColor(LightGray);
  WriteLn('This demo application shows usage of image manipulation functions');
  WriteLn('and TNImageFileFormat descendants.');
  WriteLn;

  {Load image Data\Sample.jpg.}
  F := Vfs.Open('Sample.jpg', fdRead);
  FindImageFileFormat('jpg').LoadImage(Img, F.Stream);
  Vfs.Close(F);

  {Display image info.}
  WriteLn('Image loaded.');
  WriteLn(sIdent + 'File:        Sample.jpg');
  ShowImageInfo(Img);
  WriteLn;
  WriteLn('Press any key to continue.');
  WriteLn;
  ReadKey;

  {Display image data format options.}
  WriteLn('Do you want to change image''s data format?');
  WriteLn(sIdent + 'RETURN) No. Leave data format as it is.');
  for I := 0 to RegisteredImageDataFormatsCount - 1 do
    WriteLn(sIdent, Char(I + $61), ') ', FindImageDataFormat(I).Name,
      ' (', FindImageDataFormat(I).BitCount, ' bits per pixel)');
  WriteLn;

  I := Ord(ReadKey);
  if I <> 13 then
  begin
    {Assign selected image format.}
    Dest := FindImageDataFormat(I - $61);
    if Dest <> nil then
    begin
      {Convert to new data format using current format's data format.}
      Img.DataFormat.ConvertFunc(Img, Dest);
      WriteLn('Image converted to new data format.');
      ShowImageInfo(Img);
    end
    else
      WriteLn('Invalid image data format selected - image not converted.');

    WriteLn;
    WriteLn('Press any key to continue.');
    WriteLn;
    ReadKey;
  end;

  {Display image file format options.}
  WriteLn('Do you want to save image in one of these file formats?');
  WriteLn('(Only image file formats which support saving are listed, ');
  WriteLn('note that selected file format should not support data format you');
  WriteLn('have selected. In this case image will be converted to some');
  WriteLn('data format compatible with given file format)');
  WriteLn(sIdent + 'RETURN) No. Discard this image.');

  Formats := TNPointerList.Create;
  {Build list of image file formats which support saving.}
  for I := 0 to RegisteredImageFileFormats.Modules.Count - 1 do
    if TNImageFileFormat(RegisteredImageFileFormats.Modules[I]).CanSave then
      Formats.Add(RegisteredImageFileFormats.Modules[I]);

  for I := 0 to Formats.Count - 1 do
    WriteLn(sIdent, Char(I + $61), ') ',
      TNImageFileFormat(Formats[I]).FormatName);
  WriteLn;

  I := Ord(ReadKey);
  if I <> 13 then
  begin
    {Assign selected image file format.}
    DestIFF := TNImageFileFormat(Formats[I - $61]);
    if DestIFF <> nil then
    begin
      Ext := DestIFF.Extensions;
      S := 'Data\Output.' + StrToken(Ext, [',', ';']);
      {Save image to Data\ directory in selected format.}
      F := Vfs.Open(S, fdCreate);
      DestIFF.SaveImage(Img, F.Stream);
      Vfs.Close(F);
      WriteLn('Image saved to file ', Vfs.GetFullFileName(S));
    end
    else
      WriteLn('Invalid image file format selected - image not saved.');

    WriteLn;
    WriteLn('Press any key to exit.');
    ReadKey;
  end;

  Formats.Free;
  FreeImage(Img);
end.

