{
  Numalian Engine
  Demo Application: VfsDemo

  Shows usage of Virtual File System.
}

program VfsDemo;

{$APPTYPE CONSOLE}

{$I Compiler.inc}

uses
  {Base engine unit (also sets custom memory manager so it must be first).}
  Kernel,
  {File and data functions and classes.}
  Files,
  {Base classes.}
  Objects,
  {Platform class (created automatically) - acces to platform specific functions
   in platform independent manner.}
  Platforms,
  SysUtils,
{$IFDEF MSWINDOWS}
  CrtWin32;
{$ENDIF}
{$IFDEF LINUX}
CrtLinux;
{$ENDIF}  

{Draws Numalian Logo from external file.}
procedure DrawLogo;
var
  {Our own string list.}
  List: Objects.TNStringList;
  I: Long;
begin
  List := TNStringList.Create;
  {Vfs transforms relative path to absolute path.}
  List.LoadFromFile(Vfs.GetFullFileName('Data/LogoASCII.con'));
  for I := 0 to List.Count - 1 do
  begin
    Write(List[I]);
    if Length(List[I]) < 80 then
      Write(sLineBreak);
  end;
  List.Free;
end;

var
  F, F2:TNBinaryFile;

begin
  ClrScr;
  TextColor(LightCyan);
  DrawLogo;
  TextColor(LightGray);
  WriteLn('This demo application shows usage of Virtual File System.');
  WriteLn;

  {You can mount multiple real directories or archives to one virtual directory.
   You can use variables in directory names. You can use path delimiters you like.
   Paths are case insensitive.
   Some standard dirs are mounted on initialization: Bin, Modules, Logs, Data, ...}
  Vfs.Mount('MyData','$(ROOT)/Data');
  Vfs.Mount('MyData','$(ROOT)\Data\Arkyf.zip');
  WriteLn('Virtual directory mounted: MyData (',
    Vfs.VirtualDirRealPathsStr['MyData'][0] ,')');
  WriteLn('Virtual directory mounted: MyData (',
    Vfs.VirtualDirRealPathsStr['MyData'][1] ,')');
  {Now if you open file in MyData directory it can be in Data\ directory
   or in Arkyf.zip archive. This behaviour can be used e.g. when you have
   some your game data on CD and some on HDD.}
  F:=Vfs.Open('MyData/Dir/Necron.png', fdRead);
  WriteLn('File opened: ', F.FileName, ' (', F.Size,' B)');
  {Create new file in Data\ directory.}
  F2:=Vfs.Open('Data/FromArchive.png', fdCreate);
  WriteLn('File created: ', F2.FileName);
  {Copy from F to F2.}
  F2.Stream.CopyFrom(F.Stream, F.Size);
  WriteLn(F.FileName, ' copied to ', F2.FileName);
  {Now F and F2 are the same.}
  Vfs.Close(F);
  Vfs.Close(F2);
  {You can also create new files in archives but TNArchiveFormat
   for given archive must support adding new files.}

  WriteLn('Press any key to exit.');
  ReadKey;
end.
