{*************************************************************************}
{*   ZLibFormat                                                          *}
{*   Copyright (C) 2003 Marek Mauder, Vampyre SoftWorks                  *}
{*   Engine\Files\Compressions\ZLib\ZLibFormat.pas                       *}
{*************************************************************************}
{                                                                         }
{ This program is free software; you can redistribute it and/or modify it }
{ under the terms of the GNU General Public License as published by the   }
{ Free Software Foundation; either version 2 of the License, or (at your  }
{ option) any later version. This program is distributed in the hope that }
{ it will be useful, but WITHOUT ANY WARRANTY; without even the implied   }
{ warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See   }
{ the GNU General Public License for more details.                        }
{                                                                         }
{ You should have received a copy of the GNU General Public License       }
{ along with this program; if not, write to the Free Software Foundation, }
{ Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.         }
{                                                                         }
{*************************************************************************}
{  Created On: 7th November 2003  (07.11.2003)                            }
{*************************************************************************}

{This unit contains class for compress/decompress data using ZLib library
 (used in PNG images, Zip, GZip, ...).}

unit ZLibFormat;

{$I 'Compiler.inc'}

interface

uses
  Kernel, Objects, Platforms, Files, ZLibEx;

type

  {This compression format provides ZLib compression/decompression.}
  TNZLibFormat = class(TNCompression)
  protected
    procedure DoCompress(InData: Pointer; InSize: Long; var OutData: Pointer; var
      OutSize: Long); override;
    procedure DoDecompress(InData: Pointer; InSize: Long; var OutData: Pointer;
      var OutSize: Long); override;
  public
    constructor Create; override;
  end;

const

  sZLibExtensions = 'z,zlib,tz';
  sZLibFormatName = 'ZLib compression/decompression';

var

  ZLibInstance: TNZLibFormat = nil;


{*************************************************************************}
{                         IMPLEMEMTATION                                  }
{*************************************************************************}


implementation


{*************************************************************************}
{                        TNZLibFormat methods                              }
{*************************************************************************}


constructor TNZLibFormat.Create;
begin
  inherited Create;
  FExtensions := sZLibExtensions;
  FFormatName := sZLibFormatName;
  FCanLoad := True;
  FCanSave := True;
end;

procedure TNZLibFormat.DoCompress(InData: Pointer; InSize: Long;
  var OutData: Pointer; var OutSize: Long);
begin
  ZLibEx.CompressBuffer(InData, InSize, OutData, OutSize);
end;

procedure TNZLibFormat.DoDecompress(InData: Pointer; InSize: Long;
  var OutData: Pointer; var OutSize: Long);
begin
  ZLibEx.DecompressBuffer(InData, InSize, OutData, OutSize);
end;


{*************************************************************************}
{                         Internal Stuff                                  }
{*************************************************************************}


initialization

  ZLibInstance := TNZLibFormat.Create;

finalization

  FreeAndNil(ZLibInstance);

end.

