{*************************************************************************}
{*   StdImageDataFormats                                                 *}
{*   Copyright (C) 2003 Marek Mauder, Vampyre SoftWorks                  *}
{*   Engine\Graphic\ImageDataFormats\StdImageDataFormats.pas             *}
{*************************************************************************}
{                                                                         }
{ This program is free software; you can redistribute it and/or modify it }
{ under the terms of the GNU General Public License as published by the   }
{ Free Software Foundation; either version 2 of the License, or (at your  }
{ option) any later version. This program is distributed in the hope that }
{ it will be useful, but WITHOUT ANY WARRANTY; without even the implied   }
{ warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See   }
{ the GNU General Public License for more details.                        }
{                                                                         }
{ You should have received a copy of the GNU General Public License       }
{ along with this program; if not, write to the Free Software Foundation, }
{ Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.         }
{                                                                         }
{*************************************************************************}
{  Created On: 11st September 2003  (11.09.2003)                          }
{*************************************************************************}

{This unit contains some standard image data formats.
Their definitions as well as converting functions.
All formats are registered in initialization section.}

unit StdImageDataFormats;

{DONE : Add: A1R5G5B5, A4R4G4B4, R3G3B2, Intensity32F, R16G16B16, A8I8}

{$I Compiler.inc}

interface

uses
  SysUtils, Kernel, Objects, Platforms, Graphic, Images;

{(*}

{These are ImageDataFormat convering functions.
 They first check if dest format resembles its own format
 (e.g. both are indexed or floating point ...). If they resemble
 each other conversion will proceed right into dest format.
 If they do not resemble, image is converted to default format
 >>idfA8R8G8B8<< and then is called converting procedure
 of destination format. This proc will convert A8R8G8B8 image to its own format.
 So conversions between related formats are faster and needs less memory
 (also some data special for given format are not lost during conversion).}
{Note on parameters: at least one of parameters must be in
 format for which this function is defined. Example:
 When calling ConvertR8G8B8() Image must be be in R8G8B8 format
 OR DestFormat must be R8G8B8.}
function ConvertA8R8G8B8(var Image: TImageData; DestFormat: PImageDataFormat): Boolean; forward;
function ConvertR8G8B8(var Image: TImageData; DestFormat: PImageDataFormat): Boolean; forward;
function ConvertIndex8(var Image: TImageData; DestFormat: PImageDataFormat): Boolean; forward;
function ConvertIntensity8(var Image: TImageData; DestFormat: PImageDataFormat): Boolean; forward;
function ConvertIntensity16(var Image: TImageData; DestFormat: PImageDataFormat): Boolean; forward;
function ConvertIntensity32(var Image: TImageData; DestFormat: PImageDataFormat): Boolean; forward;
function ConvertIntensity32F(var Image: TImageData; DestFormat: PImageDataFormat): Boolean; forward;
function ConvertA8Intensity8(var Image: TImageData; DestFormat: PImageDataFormat): Boolean; forward;
function ConvertR5G5B5(var Image: TImageData; DestFormat: PImageDataFormat): Boolean; forward;
function ConvertA1R5G5B5(var Image: TImageData; DestFormat: PImageDataFormat): Boolean; forward;
function ConvertA4R4G4B4(var Image: TImageData; DestFormat: PImageDataFormat): Boolean; forward;
function ConvertR3G3B2(var Image: TImageData; DestFormat: PImageDataFormat): Boolean; forward;
function ConvertR1G1B1(var Image: TImageData; DestFormat: PImageDataFormat): Boolean; forward;
function ConvertA16R16G16B16(var Image: TImageData; DestFormat: PImageDataFormat): Boolean; forward;
function ConvertR16G16B16(var Image: TImageData; DestFormat: PImageDataFormat): Boolean; forward;
function ConvertA32R32G32B32F(var Image: TImageData; DestFormat: PImageDataFormat): Boolean; forward;

function TransARGB_R8G8B8(Format: PImageDataFormat; ARGB: TARGB; Pal:PPalArray = nil): ULong; forward;
function TransColor_R8G8B8(Format: PImageDataFormat; Color: ULong; Pal:PPalArray = nil): TARGB; forward;
function TransARGB_Index(Format: PImageDataFormat; ARGB: TARGB; Pal:PPalArray = nil): ULong; forward;
function TransColor_Index(Format: PImageDataFormat; Color: ULong; Pal:PPalArray = nil): TARGB; forward;
function TransARGB_Intensity(Format: PImageDataFormat; ARGB: TARGB; Pal:PPalArray = nil): ULong; forward;
function TransColor_Intensity(Format: PImageDataFormat; Color: ULong; Pal:PPalArray = nil): TARGB; forward;
function TransARGB_PF(Format: PImageDataFormat; ARGB: TARGB; Pal:PPalArray = nil): ULong; forward;
function TransColor_PF(Format: PImageDataFormat; Color: ULong; Pal:PPalArray = nil): TARGB; forward;

{*)}

var

  {DataFormat record variables.}
  IDFA8R8G8B8Rec: TImageDataFormat;
  IDFR8G8B8Rec: TImageDataFormat;
  IDFIndex8Rec: TImageDataFormat;
  IDFIntensity8Rec: TImageDataFormat;
  IDFIntensity16Rec: TImageDataFormat;
  IDFIntensity32Rec: TImageDataFormat;
  IDFIntensity32FRec: TImageDataFormat;
  IDFA8Intensity8Rec: TImageDataFormat;
  IDFR5G5B5Rec: TImageDataFormat;
  IDFA1R5G5B5Rec: TImageDataFormat;
  IDFA4R4G4B4Rec: TImageDataFormat;
  IDFR3G3B2Rec: TImageDataFormat;
  IDFR1G1B1Rec: TImageDataFormat;
  IDFA16R16G16B16Rec: TImageDataFormat;
  IDFR16G16B16Rec: TImageDataFormat;
  IDFA32R32G32B32FRec: TImageDataFormat;

  {Default ImageData format used when converting non-related images.}
  DefFormat: TImageDataFormat absolute IDFA8R8G8B8Rec;


{*************************************************************************}
{                             IMPLEMEMTATION                              }
{*************************************************************************}


implementation

uses
  ColorQuantization;

procedure InitFormats;
begin
  with IDFA8R8G8B8Rec do
  begin
    ZeroMemory(@IDFA8R8G8B8Rec, SizeOf(TImageDataFormat));
    Name := idfA8R8G8B8;
    BitCount := 32;
    PalSize := 0;
    Indexed := False;
    HasAlpha := True;
    FixedPixelFormat := True;
    FloatFormat := False;
    OneChannelFormat := False;
    OneChannelSize := 8;
    SupportsDrawing := True;
    PixelFormat := Graphic.PixelFormat(8, 8, 8, 8);
    ConvertFunc := @ConvertA8R8G8B8;
    TransARGBFunc := nil;
    TransColorFunc := nil;
  end;
  with IDFR8G8B8Rec do
  begin
    ZeroMemory(@IDFR8G8B8Rec, SizeOf(TImageDataFormat));
    Name := idfR8G8B8;
    BitCount := 24;
    PalSize := 0;
    Indexed := False;
    HasAlpha := False;
    FixedPixelFormat := True;
    FloatFormat := False;
    OneChannelFormat := False;
    OneChannelSize := 8;
    SupportsDrawing := True;
    PixelFormat := Graphic.PixelFormat(0, 8, 8, 8);
    ConvertFunc := @ConvertR8G8B8;
    TransARGBFunc := @TransARGB_R8G8B8;
    TransColorFunc := @TransColor_R8G8B8;
  end;
  with IDFIndex8Rec do
  begin
    ZeroMemory(@IDFIndex8Rec, SizeOf(TImageDataFormat));
    Name := idfIndex8;
    BitCount := 8;
    PalSize := 256;
    Indexed := True;
    HasAlpha := True;
    FixedPixelFormat := True;
    FloatFormat := False;
    OneChannelFormat := False;
    OneChannelSize := 8;
    SupportsDrawing := True;
    PixelFormat := Graphic.PixelFormat(8, 8, 8, 8);
    ConvertFunc := @ConvertIndex8;
    TransARGBFunc := @TransARGB_Index;
    TransColorFunc := @TransColor_Index;
  end;
  with IDFIntensity8Rec do
  begin
    ZeroMemory(@IDFIntensity8Rec, SizeOf(TImageDataFormat));
    Name := idfIntensity8;
    BitCount := 8;
    PalSize := 0;
    Indexed := False;
    HasAlpha := False;
    FixedPixelFormat := True;
    FloatFormat := False;
    OneChannelFormat := True;
    OneChannelSize := 8;
    SupportsDrawing := True; //?
    PixelFormat := Graphic.PixelFormat(0, 8, 0, 0);
    ConvertFunc := @ConvertIntensity8;
    TransARGBFunc := @TransARGB_Intensity;
    TransColorFunc := @TransColor_Intensity;
  end;
  with IDFIntensity16Rec do
  begin
    ZeroMemory(@IDFIntensity16Rec, SizeOf(TImageDataFormat));
    Name := idfIntensity16;
    BitCount := 16;
    PalSize := 0;
    Indexed := False;
    HasAlpha := False;
    FixedPixelFormat := True;
    FloatFormat := False;
    OneChannelFormat := True;
    OneChannelSize := 16;
    SupportsDrawing := True; //?
    PixelFormat := Graphic.PixelFormat(0, 8, 8, 0);
    ConvertFunc := @ConvertIntensity16;
    TransARGBFunc := @TransARGB_Intensity;
    TransColorFunc := @TransColor_Intensity;
  end;
  with IDFIntensity32Rec do
  begin
    ZeroMemory(@IDFIntensity32Rec, SizeOf(TImageDataFormat));
    Name := idfIntensity32;
    BitCount := 32;
    PalSize := 0;
    Indexed := False;
    HasAlpha := False;
    FixedPixelFormat := True;
    FloatFormat := False;
    OneChannelFormat := True;
    OneChannelSize := 32;
    SupportsDrawing := True;
    PixelFormat := Graphic.PixelFormat(0, 32, 0, 0);
    ConvertFunc := @ConvertIntensity32;
    TransARGBFunc := @TransARGB_Intensity;
    TransColorFunc := @TransColor_Intensity;
  end;
  with IDFIntensity32FRec do
  begin
    ZeroMemory(@IDFIntensity32FRec, SizeOf(TImageDataFormat));
    Name := idfIntensity32F;
    BitCount := 32;
    PalSize := 0;
    Indexed := False;
    HasAlpha := False;
    FixedPixelFormat := True;
    FloatFormat := True;
    OneChannelFormat := True;
    OneChannelSize := 32;
    SupportsDrawing := False;
    PixelFormat := Graphic.PixelFormat(0, 32, 0, 0);
    ConvertFunc := @ConvertIntensity32F;
    TransARGBFunc := nil;
    TransColorFunc := nil;
  end;
  with IDFA8Intensity8Rec do
  begin
    ZeroMemory(@IDFA8Intensity8Rec, SizeOf(TImageDataFormat));
    Name := idfA8Intensity8;
    BitCount := 16;
    PalSize := 0;
    Indexed := False;
    HasAlpha := True;
    FixedPixelFormat := True;
    FloatFormat := False;
    OneChannelFormat := True;
    OneChannelSize := 8;
    SupportsDrawing := True;
    PixelFormat := Graphic.PixelFormat(8, 8, 0, 0);
    ConvertFunc := @ConvertA8Intensity8;
    TransARGBFunc := @TransARGB_Intensity;
    TransColorFunc := @TransColor_Intensity;
  end;
  with IDFR5G5B5Rec do
  begin
    ZeroMemory(@IDFR5G5B5Rec, SizeOf(TImageDataFormat));
    Name := idfR5G5B5;
    BitCount := 16;
    PalSize := 0;
    Indexed := False;
    HasAlpha := False;
    FixedPixelFormat := False;
    FloatFormat := False;
    OneChannelFormat := False;
    OneChannelSize := 5;
    SupportsDrawing := True;
    PixelFormat := Graphic.PixelFormat(0, 5, 5, 5);
    ConvertFunc := @ConvertR5G5B5;
    TransARGBFunc := @TransARGB_PF;
    TransColorFunc := @TransColor_PF;
  end;
  with IDFA1R5G5B5Rec do
  begin
    ZeroMemory(@IDFA1R5G5B5Rec, SizeOf(TImageDataFormat));
    Name := idfA1R5G5B5;
    BitCount := 16;
    PalSize := 0;
    Indexed := False;
    HasAlpha := True;
    FixedPixelFormat := False;
    FloatFormat := False;
    OneChannelFormat := False;
    OneChannelSize := 5;
    SupportsDrawing := True;
    PixelFormat := Graphic.PixelFormat(1, 5, 5, 5);
    ConvertFunc := @ConvertA1R5G5B5;
    TransARGBFunc := @TransARGB_PF;
    TransColorFunc := @TransColor_PF;
  end;
  with IDFA4R4G4B4Rec do
  begin
    ZeroMemory(@IDFA4R4G4B4Rec, SizeOf(TImageDataFormat));
    Name := idfA4R4G4B4;
    BitCount := 16;
    PalSize := 0;
    Indexed := False;
    HasAlpha := True;
    FixedPixelFormat := False;
    FloatFormat := False;
    OneChannelFormat := False;
    OneChannelSize := 4;
    SupportsDrawing := True;
    PixelFormat := Graphic.PixelFormat(4, 4, 4, 4);
    ConvertFunc := @ConvertA4R4G4B4;
    TransARGBFunc := @TransARGB_PF;
    TransColorFunc := @TransColor_PF;
  end;
  with IDFR3G3B2Rec do
  begin
    ZeroMemory(@IDFR3G3B2Rec, SizeOf(TImageDataFormat));
    Name := idfR3G3B2;
    BitCount := 8;
    PalSize := 0;
    Indexed := False;
    HasAlpha := False;
    FixedPixelFormat := False;
    FloatFormat := False;
    OneChannelFormat := False;
    OneChannelSize := 3;
    SupportsDrawing := True;
    PixelFormat := Graphic.PixelFormat(0, 3, 3, 2);
    ConvertFunc := @ConvertR3G3B2;
    TransARGBFunc := @TransARGB_PF;
    TransColorFunc := @TransColor_PF;
  end;
  with IDFR1G1B1Rec do
  begin
    ZeroMemory(@IDFR1G1B1Rec, SizeOf(TImageDataFormat));
    Name := idfR1G1B1;
    BitCount := 8;
    PalSize := 0;
    Indexed := False;
    HasAlpha := False;
    FixedPixelFormat := False;
    FloatFormat := False;
    OneChannelFormat := False;
    OneChannelSize := 1;
    SupportsDrawing := True;
    PixelFormat := Graphic.PixelFormat(0, 1, 1, 1);
    ConvertFunc := @ConvertR1G1B1;
    TransARGBFunc := @TransARGB_PF;
    TransColorFunc := @TransColor_PF;
  end;
  with IDFA16R16G16B16Rec do
  begin
    ZeroMemory(@IDFA16R16G16B16Rec, SizeOf(TImageDataFormat));
    Name := idfA16R16G16B16;
    BitCount := 64;
    PalSize := 0;
    Indexed := False;
    HasAlpha := True;
    FixedPixelFormat := True;
    FloatFormat := False;
    OneChannelFormat := False;
    OneChannelSize := 16;
    SupportsDrawing := False;
    PixelFormat := Graphic.PixelFormat(16, 16, 16, 16);
    ConvertFunc := @ConvertA16R16G16B16;
    TransARGBFunc := nil;
    TransColorFunc := nil;
  end;
  with IDFR16G16B16Rec do
  begin
    ZeroMemory(@IDFR16G16B16Rec, SizeOf(TImageDataFormat));
    Name := idfR16G16B16;
    BitCount := 48;
    PalSize := 0;
    Indexed := False;
    HasAlpha := False;
    FixedPixelFormat := True;
    FloatFormat := False;
    OneChannelFormat := False;
    OneChannelSize := 16;
    SupportsDrawing := False;
    PixelFormat := Graphic.PixelFormat(0, 16, 16, 16);
    ConvertFunc := @ConvertR16G16B16;
    TransARGBFunc := nil;
    TransColorFunc := nil;
  end;
  with IDFA32R32G32B32FRec do
  begin
    ZeroMemory(@IDFA32R32G32B32FRec, SizeOf(TImageDataFormat));
    Name := idfA32R32G32B32F;
    BitCount := 128;
    PalSize := 0;
    Indexed := False;
    HasAlpha := True;
    FixedPixelFormat := True;
    FloatFormat := True;
    OneChannelFormat := False;
    OneChannelSize := 32;
    SupportsDrawing := False;
    PixelFormat := Graphic.PixelFormat(32, 32, 32, 32);
    ConvertFunc := @ConvertA32R32G32B32F;
    TransARGBFunc := nil;
    TransColorFunc := nil;
  end;
end;


{*************************************************************************}
{                             A8R8G8B8                                    }
{*************************************************************************}


function ConvertA8R8G8B8(var Image: TImageData; DestFormat: PImageDataFormat):
  Boolean;
var
  OrigFmt: PImageDataFormat;
begin
  Result := False;
  OrigFmt := Image.DataFormat;
  if not TestImage(Image) then Exit;
  if Image.DataFormat = DestFormat then
  begin
    Result := True;
    Exit;
  end;
  try
    if DestFormat = @IDFA8R8G8B8Rec then
      // conversion:  Another -> ARGB
      // formats are not related -> convert to ARGB
      // by Image.DataFormat's ConvertProc
      Result := Image.DataFormat^.ConvertFunc(Image, @IDFA8R8G8B8Rec);
    if Image.DataFormat = @IDFA8R8G8B8Rec then
      // conversion:  ARGB -> Another
      // formats are not related -> convert to ARGB and convert
      // by DestFormat's ConvertProc
      Result := DestFormat^.ConvertFunc(Image, DestFormat);
  except
    on E: Exception do
    begin
      ShowError(ErrorStr(nil, 'StdImageDataFormats.ConvertA8R8G8B8',
        Format('Error when converting image from [%s] format to [%s].' +
        sLineBreak + sExceptMsg + E.Message, [OrigFmt^.Name,
        DestFormat^.Name])), E.ClassName);
    end;
  end;
end;


{*************************************************************************}
{                             R8G8B8                                      }
{*************************************************************************}


function ConvertR8G8B8(var Image: TImageData; DestFormat: PImageDataFormat):
  Boolean;
var
  I: Long;
  OrigFmt: PImageDataFormat;

  procedure ConvertLevelToARGB(var Level: TImageLevel);
  var
    I, X, Y: Long;
    CA: PARGBRec;
    C: PRGBRec;
    Data: Pointer;
  begin
    with Level do
    begin
      Size := Width * Height * DefFormat.BitCount shr 3;
      GetMem(Data, Size);
      CA := Data;
      C := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          CA.A := $FF;
          CA.R := C.R;
          CA.G := C.G;
          CA.B := C.B;
          Inc(CA);
          Inc(C);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Count - 1 do
        ConvertLevelToARGB(Levels[I]);
    end;
  end;

  procedure ConvertLevelFromARGB(var Level: TImageLevel);
  var
    I, X, Y: Long;
    CA: PARGBRec;
    C: PRGBRec;
    Data: Pointer;
  begin
    with Level do
    begin
      Size := Width * Height * 3;
      GetMem(Data, Size);
      C := Data;
      CA := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          C.R := CA.R;
          C.G := CA.G;
          C.B := CA.B;
          Inc(CA);
          Inc(C);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Count - 1 do
        ConvertLevelFromARGB(Levels[I]);
    end;
  end;

begin
  Result := False;
  OrigFmt := Image.DataFormat;
  if not TestImage(Image) then Exit;
  if Image.DataFormat = DestFormat then
  begin
    Result := True;
    Exit;
  end;
  try
    if DestFormat = @IDFR8G8B8Rec then
    begin
      // conversion:  Another -> R8G8B8
      // convert to ARGB by DestFormat's ConvertProc and then
      // convert to own format
      if Image.DataFormat <> @DefFormat then
        Image.DataFormat^.ConvertFunc(Image, @DefFormat);
      for I := 0 to Image.Count - 1 do
        ConvertLevelFromARGB(Image.Levels[I]);
      Image.DataFormat := @IDFR8G8B8Rec;
      Result := True;
    end
    else
      if Image.DataFormat = @IDFR8G8B8Rec then
      begin
        // conversion:  R8G8B8 -> Another
        // convert to ARGB and convert by DestFormat's ConvertProc
        Image.DataFormat := @DefFormat;
        for I := 0 to Image.Count - 1 do
          ConvertLevelToARGB(Image.Levels[I]);
        DestFormat^.ConvertFunc(Image, DestFormat);
        Result := True;
      end;
  except
    on E: Exception do
    begin
      ShowError(ErrorStr(nil, 'StdImageDataFormats.ConvertR8G8B8',
        Format('Error when converting image from [%s] format to [%s].' +
        sLineBreak + sExceptMsg + E.Message, [OrigFmt^.Name,
        DestFormat^.Name])), E.ClassName);
    end;
  end;
end;

function TransARGB_R8G8B8(Format: PImageDataFormat; ARGB: TARGB; Pal:
  PPalArray): ULong;
begin
  Result := ARGB and $00FFFFFF;
end;

function TransColor_R8G8B8(Format: PImageDataFormat; Color: ULong; Pal:
  PPalArray): TARGB;
begin
  Result := $FF000000 or Color;
end;


{*************************************************************************}
{                           Indexed functions                             }
{*************************************************************************}


procedure ConvertIndexed(var Image: TImageData; DestFormat: PImageDataFormat);
begin
{TODO : Add code here.When some related formats will be added.}
end;

function TransARGB_Index(Format: PImageDataFormat; ARGB: TARGB; Pal: PPalArray):
  ULong;
begin
  Result := ClosestColorInPal(Pal, Format^.PalSize, ARGB);
end;

function TransColor_Index(Format: PImageDataFormat; Color: ULong; Pal:
  PPalArray): TARGB;
begin
  if (Color < Format^.PalSize) and (Pal <> nil) then
    Result := Pal[Color].Color
  else
    Result := $000000;
end;


{*************************************************************************}
{                             Index8                                      }
{*************************************************************************}


function ConvertIndex8(var Image: TImageData; DestFormat: PImageDataFormat):
  Boolean;
var
  I: Long;
  OrigFmt: PImageDataFormat;
  Quant: TNColorQuantizer;

  procedure ConvertLevelToARGB(var Level: TImageLevel);
  var
    Data: Pointer;
    PB: PByte;
    PU: PULong;
    I, X, Y: Long;
  begin
    with Level do
    begin
      Size := Width * Height * DefFormat.BitCount shr 3;
      GetMem(Data, Size);
      PU := Data;
      PB := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          PU^ := Image.Palette[PB^].Color;
          Inc(PU);
          Inc(PB);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Count - 1 do
        ConvertLevelToARGB(Levels[I]);
    end;
  end;

  procedure ConvertLevelFromARGB(var Level: TImageLevel);

    procedure BuildOctree(var Level: TImageLevel);
    var
      I: Long;
    begin
      Quant.UpdateOctree(Level.Width, Level.Height, DefFormat.BitCount,
        Level.Bits);
      for I := 0 to Level.Count - 1 do
        BuildOctree(Level.Levels[I]);
    end;

    procedure Convert(var Level: TImageLevel);
    var
      I: Long;
      Data: Pointer;
    begin
      with Level do
      begin
        Size := Width * Height;
        Quant.ConvertImage(Width, Height, DefFormat.BitCount, Bits, Data);
        FreeMem(Bits);
        Bits := Data;
        for I := 0 to Count - 1 do
          Convert(Levels[I]);
      end;
    end;

  begin
    BuildOctree(Level);
    Convert(Level);
  end;

begin
  Result := False;
  OrigFmt := Image.DataFormat;
  if not TestImage(Image) then Exit;
  if Image.DataFormat = DestFormat then
  begin
    Result := True;
    Exit;
  end;
  try
    if DestFormat = @IDFIndex8Rec then
    begin
      // conversion:  Another -> Index8
      if Image.DataFormat^.Indexed then
        // formats are related
        ConvertIndexed(Image, @IDFIndex8Rec)
      else
      begin
        // formats are not related -> convert to ARGB by
        //DestFormat's ConvertProc and then convert to own format
        if Image.DataFormat <> @DefFormat then
          Image.DataFormat^.ConvertFunc(Image, @DefFormat);
        Quant := TNColorQuantizer.Create(IDFIndex8Rec.PalSize, 7);
        for I := 0 to Image.Count - 1 do
          ConvertLevelFromARGB(Image.Levels[I]);
        GetMem(Image.Palette, IDFIndex8Rec.PalSize * SizeOf(TARGBRec));
        Quant.GetPalette(Image.Palette);
        Quant.Free;
        Image.DataFormat := @IDFIndex8Rec;
      end;
      Result := True;
    end
    else
      if Image.DataFormat = @IDFIndex8Rec then
      begin
        // conversion:  Index8 -> Another
        if DestFormat^.Indexed then
        // formats are related
          ConvertIndexed(Image, DestFormat)
        else
        begin
          // formats are not related -> convert to ARGB and convert
          // by DestFormat's ConvertProc
          Image.DataFormat := @DefFormat;
          for I := 0 to Image.Count - 1 do
            ConvertLevelToARGB(Image.Levels[I]);
          FreeMemNil(Image.Palette);
          DestFormat^.ConvertFunc(Image, DestFormat);
        end;
        Result := True;
      end;
  except
    on E: Exception do
    begin
      ShowError(ErrorStr(nil, 'StdImageDataFormats.ConvertIndex8',
        Format('Error when converting image from [%s] format to [%s].' +
        sLineBreak + sExceptMsg + E.Message, [OrigFmt^.Name,
        DestFormat^.Name])), E.ClassName);
    end;
  end;
end;


{*************************************************************************}
{                         Intensity functions                             }
{*************************************************************************}


var
  RCh, GCh, BCh: array[Byte] of Byte;
  GreyPal: PPalArray;

procedure ConvertIntensity(var Image: TImageData; DestFormat: PImageDataFormat);
var
  I: Long;
  PW, PW2: PWord;
  PB: PByte;
  PU: PLong;
  PS: PSingle;
  Data: Pointer;

  procedure Level8To16A(var Level: TImageLevel);
  var
    I, X, Y: Long;
  begin
    with Level do
    begin
      Size := Width * Height * 2;
      GetMem(Data, Size);
      PW := Data;
      PB := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          PW^ := $FF00 or PB^;
          Inc(PW);
          Inc(PB);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Level.Count - 1 do
        Level8To16A(Level.Levels[I])
    end;
  end;

  procedure Level8To16(var Level: TImageLevel);
  var
    I, X, Y: Long;
  begin
    with Level do
    begin
      Size := Width * Height * 2;
      GetMem(Data, Size);
      PW := Data;
      PB := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          PW^ := PB^ shl 8;
          Inc(PW);
          Inc(PB);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Level.Count - 1 do
        Level8To16(Level.Levels[I])
    end;
  end;

  procedure Level8To32(var Level: TImageLevel);
  var
    I, X, Y: Long;
  begin
    with Level do
    begin
      Size := Width * Height * 4;
      GetMem(Data, Size);
      PU := Data;
      PB := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          PU^ := PB^ shl 24;
          Inc(PU);
          Inc(PB);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Level.Count - 1 do
        Level8To32(Level.Levels[I])
    end;
  end;

  procedure Level8To32F(var Level: TImageLevel);
  var
    I, X, Y: Long;
  begin
    with Level do
    begin
      Size := Width * Height * 4;
      GetMem(Data, Size);
      PS := Data;
      PB := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          PS^ := PB^ / 255;
          Inc(PS);
          Inc(PB);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Level.Count - 1 do
        Level8To32F(Level.Levels[I])
    end;
  end;

  procedure Level16ATo8(var Level: TImageLevel);
  var
    I, X, Y: Long;
  begin
    with Level do
    begin
      Size := Width * Height;
      GetMem(Data, Size);
      PB := Data;
      PW := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          PB^ := PW^;
          Inc(PW);
          Inc(PB);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Level.Count - 1 do
        Level16ATo8(Level.Levels[I])
    end;
  end;

  procedure Level16To8(var Level: TImageLevel);
  var
    I, X, Y: Long;
  begin
    with Level do
    begin
      Size := Width * Height;
      GetMem(Data, Size);
      PB := Data;
      PW := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          PB^ := PW^ shr 8;
          Inc(PW);
          Inc(PB);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Level.Count - 1 do
        Level16To8(Level.Levels[I])
    end;
  end;

  procedure Level16ATo16(var Level: TImageLevel);
  var
    I, X, Y: Long;
  begin
    with Level do
    begin
      Size := Width * Height * 2;
      GetMem(Data, Size);
      PW2 := Data;
      PW := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          PW2^ := (PW^ and $00FF) shl 8;
          Inc(PW);
          Inc(PW2);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Level.Count - 1 do
        Level16ATo16(Level.Levels[I])
    end;
  end;

  procedure Level16To16A(var Level: TImageLevel);
  var
    I, X, Y: Long;
  begin
    with Level do
    begin
      Size := Width * Height * 2;
      GetMem(Data, Size);
      PW2 := Data;
      PW := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          PW2^ := $FF00 or (PW^ shr 8);
          Inc(PW);
          Inc(PW2);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Level.Count - 1 do
        Level16To16A(Level.Levels[I])
    end;
  end;

  procedure Level16ATo32F(var Level: TImageLevel);
  var
    I, X, Y: Long;
  begin
    with Level do
    begin
      Size := Width * Height * 4;
      GetMem(Data, Size);
      PS := Data;
      PW := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          PS^ := (PW^ and $00FF) / 255;
          Inc(PW);
          Inc(PS);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Level.Count - 1 do
        Level16ATo32F(Level.Levels[I])
    end;
  end;

  procedure Level16ATo32(var Level: TImageLevel);
  var
    I, X, Y: Long;
  begin
    with Level do
    begin
      Size := Width * Height * 4;
      GetMem(Data, Size);
      PU := Data;
      PW := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          PU^ := (PW^ and $00FF) shl 24;
          Inc(PW);
          Inc(PU);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Level.Count - 1 do
        Level16ATo32(Level.Levels[I])
    end;
  end;

  procedure Level16To32F(var Level: TImageLevel);
  var
    I, X, Y: Long;
  begin
    with Level do
    begin
      Size := Width * Height * 4;
      GetMem(Data, Size);
      PS := Data;
      PW := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          PS^ := PW^ / 65535;
          Inc(PW);
          Inc(PS);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Level.Count - 1 do
        Level16To32F(Level.Levels[I])
    end;
  end;

  procedure Level16To32(var Level: TImageLevel);
  var
    I, X, Y: Long;
  begin
    with Level do
    begin
      Size := Width * Height * 4;
      GetMem(Data, Size);
      PU := Data;
      PW := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          PU^ := PW^ shl 16;
          Inc(PW);
          Inc(PU);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Level.Count - 1 do
        Level16To32(Level.Levels[I])
    end;
  end;

  procedure Level32FTo8(var Level: TImageLevel);
  var
    I, X, Y: Long;
  begin
    with Level do
    begin
      Size := Width * Height;
      GetMem(Data, Size);
      PB := Data;
      PS := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          PB^ := Round(PS^ * 255);
          Inc(PS);
          Inc(PB);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Level.Count - 1 do
        Level32FTo8(Level.Levels[I])
    end;
  end;

  procedure Level32To8(var Level: TImageLevel);
  var
    I, X, Y: Long;
  begin
    with Level do
    begin
      Size := Width * Height;
      GetMem(Data, Size);
      PB := Data;
      PU := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          PB^ := PU^ shr 24;
          Inc(PU);
          Inc(PB);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Level.Count - 1 do
        Level32To8(Level.Levels[I])
    end;
  end;

  procedure Level32FTo16A(var Level: TImageLevel);
  var
    I, X, Y: Long;
  begin
    with Level do
    begin
      Size := Width * Height * 2;
      GetMem(Data, Size);
      PW := Data;
      PS := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          PW^ := $FF00 or Round(PS^ * 255);
          Inc(PW);
          Inc(PS);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Level.Count - 1 do
        Level32FTo16A(Level.Levels[I])
    end;
  end;

  procedure Level32To16A(var Level: TImageLevel);
  var
    I, X, Y: Long;
  begin
    with Level do
    begin
      Size := Width * Height * 2;
      GetMem(Data, Size);
      PW := Data;
      PU := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          PW^ := $FF00 or (PU^ shr 24);
          Inc(PW);
          Inc(PU);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Level.Count - 1 do
        Level32To16A(Level.Levels[I])
    end;
  end;

  procedure Level32FTo16(var Level: TImageLevel);
  var
    I, X, Y: Long;
  begin
    with Level do
    begin
      Size := Width * Height * 2;
      GetMem(Data, Size);
      PW := Data;
      PS := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          PW^ := Round(PS^ * 65535);
          Inc(PW);
          Inc(PS);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Level.Count - 1 do
        Level32FTo16(Level.Levels[I])
    end;
  end;

  procedure Level32To16(var Level: TImageLevel);
  var
    I, X, Y: Long;
  begin
    with Level do
    begin
      Size := Width * Height * 2;
      GetMem(Data, Size);
      PW := Data;
      PU := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          PW^ := PU^ shr 16;
          Inc(PW);
          Inc(PU);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Level.Count - 1 do
        Level32To16(Level.Levels[I])
    end;
  end;

  procedure Level32FTo32(var Level: TImageLevel);
  var
    I, X, Y: Long;
  begin
    with Level do
    begin
      Size := Width * Height * 4;
      GetMem(Data, Size);
      PU := Data;
      PS := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          PU^ := Round(PS^ * (MaxULong - 1));
          Inc(PU);
          Inc(PS);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Level.Count - 1 do
        Level32FTo32(Level.Levels[I])
    end;
  end;

  procedure Level32To32F(var Level: TImageLevel);
  var
    I, X, Y: Long;
  begin
    with Level do
    begin
      Size := Width * Height * 4;
      GetMem(Data, Size);
      PS := Data;
      PU := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          PS^ := PU^ / (MaxULong - 1);
          Inc(PU);
          Inc(PS);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Level.Count - 1 do
        Level32To32F(Level.Levels[I])
    end;
  end;

begin
  for I := 0 to Image.Count - 1 do
    case Image.DataFormat^.BitCount of
      8: case DestFormat^.BitCount of
          16:
            if DestFormat^.HasAlpha then
              Level8To16A(Image.Levels[I])
            else
              Level8To16(Image.Levels[I]);
          32:
            if DestFormat^.FloatFormat then
              Level8To32F(Image.Levels[I])
            else
              Level8To32(Image.Levels[I]);
        end;
      16: case DestFormat^.BitCount of
          8:
            if Image.DataFormat^.HasAlpha then
              Level16ATo8(Image.Levels[I])
            else
              Level16To8(Image.Levels[I]);
          16:
            if Image.DataFormat^.HasAlpha then
              Level16ATo16(Image.Levels[I])
            else
              Level16To16A(Image.Levels[I]);
          32:
            if Image.DataFormat^.HasAlpha then
            begin
              if DestFormat^.FloatFormat then
                Level16ATo32F(Image.Levels[I])
              else
                Level16ATo32(Image.Levels[I]);
            end
            else
            begin
              if DestFormat^.FloatFormat then
                Level16To32F(Image.Levels[I])
              else
                Level16To32(Image.Levels[I]);
            end;
        end;
      32: case DestFormat^.BitCount of
          8:
            if Image.DataFormat^.FloatFormat then
              Level32FTo8(Image.Levels[I])
            else
              Level32To8(Image.Levels[I]);
          16:
            if Image.DataFormat^.FloatFormat then
            begin
              if DestFormat^.HasAlpha then
                Level32FTo16A(Image.Levels[I])
              else
                Level32FTo16(Image.Levels[I]);
            end
            else
            begin
              if DestFormat^.HasAlpha then
                Level32To16A(Image.Levels[I])
              else
                Level32To16(Image.Levels[I]);
            end;
          32:
            if Image.DataFormat^.FloatFormat then
              Level32FTo32(Image.Levels[I])
            else
              Level32To32F(Image.Levels[I]);
        end;
    end;
  Image.DataFormat := DestFormat;
end;

function TransARGB_Intensity(Format: PImageDataFormat; ARGB: TARGB; Pal:
  PPalArray): ULong;
begin
  Result := RCh[Byte(ARGB shr 16)] + GCh[Byte(ARGB shr 8)] + BCh[Byte(ARGB)];
  case Format^.BitCount of
    16:
      if Format^.HasAlpha then
        Result := (ARGB and $FF000000 shr 16) or Result
      else
        Result := Result shl 8;
    32: Result := Result shl 24;
  end;
end;

function TransColor_Intensity(Format: PImageDataFormat; Color: ULong; Pal:
  PPalArray): TARGB;
var
  Idx: Long;
begin
  Idx := 0;
  case Format^.BitCount of
    8: Idx := Color;
    16:
      if Format^.HasAlpha then
        Idx := Byte(Color)
      else
        Idx := Color shr 8;
    32: Idx := Color shr 24;
  end;
  Result := GreyPal[Idx].Color;
  if Format^.HasAlpha then
    Result := (Color shl 16) or ($00FFFFFF and Result);
end;


{*************************************************************************}
{                             Intensity8                                  }
{*************************************************************************}


function ConvertIntensity8(var Image: TImageData; DestFormat: PImageDataFormat):
  Boolean;
var
  I: Long;
  OrigFmt: PImageDataFormat;

  procedure ConvertLevelToARGB(var Level: TImageLevel);
  var
    Data: Pointer;
    PB: PByte;
    PU: PULong;
    I, X, Y: Long;
  begin
    with Level do
    begin
      Size := Width * Height * DefFormat.BitCount shr 3;
      GetMem(Data, Size);
      PU := Data;
      PB := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          PU^ := GreyPal[PB^].Color;
          Inc(PU);
          Inc(PB);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Count - 1 do
        ConvertLevelToARGB(Levels[I]);
    end;
  end;

  procedure ConvertLevelFromARGB(var Level: TImageLevel);
  var
    I, X, Y: Long;
    CA: PARGBRec;
    Data: Pointer;
    PB: PByte;
  begin
    with Level do
    begin
      Size := Width * Height;
      GetMem(Data, Size);
      PB := Data;
      CA := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          PB^ := RCh[CA.R] + GCh[CA.G] + BCh[CA.B];
          Inc(PB);
          Inc(CA);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Count - 1 do
        ConvertLevelFromARGB(Levels[I]);
    end;
  end;

begin
  Result := False;
  OrigFmt := Image.DataFormat;
  if not TestImage(Image) then Exit;
  if Image.DataFormat = DestFormat then
  begin
    Result := True;
    Exit;
  end;
  try
    if DestFormat = @IDFIntensity8Rec then
    begin
      // conversion:  Another -> Intensity8
      if Image.DataFormat^.OneChannelFormat then
        // formats are related
        ConvertIntensity(Image, @IDFIntensity8Rec)
      else
      begin
        // formats are not related -> convert to ARGB by
        //DestFormat's ConvertProc and then convert to own format
        if Image.DataFormat <> @DefFormat then
          Image.DataFormat^.ConvertFunc(Image, @DefFormat);
        for I := 0 to Image.Count - 1 do
          ConvertLevelFromARGB(Image.Levels[I]);
        Image.DataFormat := @IDFIntensity8Rec;
      end;
      Result := True;
    end
    else
      if Image.DataFormat = @IDFIntensity8Rec then
      begin
        // conversion:  Intensity8 -> Another
        if DestFormat^.OneChannelFormat then
          // formats are related
          ConvertIntensity(Image, DestFormat)
        else
        begin
          // formats are not related -> convert to ARGB and convert
          // by DestFormat's ConvertProc
          Image.DataFormat := @DefFormat;
          for I := 0 to Image.Count - 1 do
            ConvertLevelToARGB(Image.Levels[I]);
          DestFormat^.ConvertFunc(Image, DestFormat);
        end;
        Result := True;
      end;
  except
    on E: Exception do
    begin
      ShowError(ErrorStr(nil, 'StdImageDataFormats.ConvertIntensity8',
        Format('Error when converting image from [%s] format to [%s].' +
        sLineBreak + sExceptMsg + E.Message, [OrigFmt^.Name,
        DestFormat^.Name])), E.ClassName);
    end;
  end;
end;


{*************************************************************************}
{                             Intensity16                                 }
{*************************************************************************}


function ConvertIntensity16(var Image: TImageData; DestFormat:
  PImageDataFormat): Boolean;
var
  I: Long;
  OrigFmt: PImageDataFormat;

  procedure ConvertLevelToARGB(var Level: TImageLevel);
  var
    Data: Pointer;
    PW: PWord;
    PU: PULong;
    I, X, Y: Long;
  begin
    with Level do
    begin
      Size := Width * Height * DefFormat.BitCount shr 3;
      GetMem(Data, Size);
      PU := Data;
      PW := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          PU^ := GreyPal[PW^ shr 8].Color;
          Inc(PU);
          Inc(PW);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Count - 1 do
        ConvertLevelToARGB(Levels[I]);
    end;
  end;

  procedure ConvertLevelFromARGB(var Level: TImageLevel);
  var
    I, X, Y: Long;
    CA: PARGBRec;
    Data: Pointer;
    PW: PWord;
  begin
    with Level do
    begin
      Size := Width * Height * 2;
      GetMem(Data, Size);
      PW := Data;
      CA := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          PW^ := (RCh[CA.R] + GCh[CA.G] + BCh[CA.B]) shl 8;
          Inc(PW);
          Inc(CA);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Count - 1 do
        ConvertLevelFromARGB(Levels[I]);
    end;
  end;

begin
  Result := False;
  OrigFmt := Image.DataFormat;
  if not TestImage(Image) then Exit;
  if Image.DataFormat = DestFormat then
  begin
    Result := True;
    Exit;
  end;
  try
    if DestFormat = @IDFIntensity16Rec then
    begin
      // conversion:  Another -> Intensity16
      if Image.DataFormat^.OneChannelFormat then
        // formats are related
        ConvertIntensity(Image, @IDFIntensity16Rec)
      else
      begin
        // formats are not related -> convert to ARGB by
        //DestFormat's ConvertProc and then convert to own format
        if Image.DataFormat <> @DefFormat then
          Image.DataFormat^.ConvertFunc(Image, @DefFormat);
        for I := 0 to Image.Count - 1 do
          ConvertLevelFromARGB(Image.Levels[I]);
        Image.DataFormat := @IDFIntensity16Rec;
      end;
      Result := True;
    end
    else
      if Image.DataFormat = @IDFIntensity16Rec then
      begin
        // conversion:  Intensity16 -> Another
        if DestFormat^.OneChannelFormat then
          // formats are related
          ConvertIntensity(Image, DestFormat)
        else
        begin
          // formats are not related -> convert to ARGB and convert
          // by DestFormat's ConvertProc
          Image.DataFormat := @DefFormat;
          for I := 0 to Image.Count - 1 do
            ConvertLevelToARGB(Image.Levels[I]);
          DestFormat^.ConvertFunc(Image, DestFormat);
        end;
        Result := True;
      end;
  except
    on E: Exception do
    begin
      ShowError(ErrorStr(nil, 'StdImageDataFormats.ConvertIntensity16',
        Format('Error when converting image from [%s] format to [%s].' +
        sLineBreak + sExceptMsg + E.Message, [OrigFmt^.Name,
        DestFormat^.Name])), E.ClassName);
    end;
  end;
end;


{*************************************************************************}
{                             Intensity32                                 }
{*************************************************************************}


function ConvertIntensity32(var Image: TImageData; DestFormat:
  PImageDataFormat): Boolean;
var
  I: Long;
  OrigFmt: PImageDataFormat;

  procedure ConvertLevelToARGB(var Level: TImageLevel);
  var
    Data: Pointer;
    PL: PULong;
    PU: PULong;
    I, X, Y: Long;
  begin
    with Level do
    begin
      Size := Width * Height * DefFormat.BitCount shr 3;
      GetMem(Data, Size);
      PU := Data;
      PL := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          PU^ := GreyPal[PL^ shr 24].Color;
          Inc(PU);
          Inc(PL);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Count - 1 do
        ConvertLevelToARGB(Levels[I]);
    end;
  end;

  procedure ConvertLevelFromARGB(var Level: TImageLevel);
  var
    I, X, Y: Long;
    CA: PARGBRec;
    Data: Pointer;
    PL: PULong;
  begin
    with Level do
    begin
      Size := Width * Height * 4;
      GetMem(Data, Size);
      PL := Data;
      CA := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          PL^ := (RCh[CA.R] + GCh[CA.G] + BCh[CA.B]) shl 24;
          Inc(PL);
          Inc(CA);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Count - 1 do
        ConvertLevelFromARGB(Levels[I]);
    end;
  end;

begin
  Result := False;
  OrigFmt := Image.DataFormat;
  if not TestImage(Image) then Exit;
  if Image.DataFormat = DestFormat then
  begin
    Result := True;
    Exit;
  end;
  try
    if DestFormat = @IDFIntensity32Rec then
    begin
      // conversion:  Another -> Intensity32
      if Image.DataFormat^.OneChannelFormat then
        // formats are related
        ConvertIntensity(Image, @IDFIntensity32Rec)
      else
      begin
        // formats are not related -> convert to ARGB by
        //DestFormat's ConvertProc and then convert to own format
        if Image.DataFormat <> @DefFormat then
          Image.DataFormat^.ConvertFunc(Image, @DefFormat);
        for I := 0 to Image.Count - 1 do
          ConvertLevelFromARGB(Image.Levels[I]);
        Image.DataFormat := @IDFIntensity32Rec;
      end;
      Result := True;
    end
    else
      if Image.DataFormat = @IDFIntensity32Rec then
      begin
        // conversion:  Intensity32 -> Another
        if DestFormat^.OneChannelFormat then
          // formats are related
          ConvertIntensity(Image, DestFormat)
        else
        begin
          // formats are not related -> convert to ARGB and convert
          // by DestFormat's ConvertProc
          Image.DataFormat := @DefFormat;
          for I := 0 to Image.Count - 1 do
            ConvertLevelToARGB(Image.Levels[I]);
          DestFormat^.ConvertFunc(Image, DestFormat);
        end;
        Result := True;
      end;
  except
    on E: Exception do
    begin
      ShowError(ErrorStr(nil, 'StdImageDataFormats.ConvertIntensity32',
        Format('Error when converting image from [%s] format to [%s].' +
        sLineBreak + sExceptMsg + E.Message, [OrigFmt^.Name,
        DestFormat^.Name])), E.ClassName);
    end;
  end;
end;


{*************************************************************************}
{                             Intensity32F                                }
{*************************************************************************}


function ConvertIntensity32F(var Image: TImageData; DestFormat:
  PImageDataFormat): Boolean;
var
  I: Long;
  OrigFmt: PImageDataFormat;

  procedure ConvertLevelToARGB(var Level: TImageLevel);
  var
    Data: Pointer;
    PS: PSingle;
    PU: PULong;
    I, X, Y: Long;
  begin
    with Level do
    begin
      Size := Width * Height * DefFormat.BitCount shr 3;
      GetMem(Data, Size);
      PU := Data;
      PS := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          PU^ := GreyPal[Round(PS^ * 255)].Color;
          Inc(PU);
          Inc(PS);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Count - 1 do
        ConvertLevelToARGB(Levels[I]);
    end;
  end;

  procedure ConvertLevelFromARGB(var Level: TImageLevel);
  var
    I, X, Y: Long;
    CA: PARGBRec;
    Data: Pointer;
    PS: PSingle;
  begin
    with Level do
    begin
      Size := Width * Height * 4;
      GetMem(Data, Size);
      PS := Data;
      CA := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          PS^ := (RCh[CA.R] + GCh[CA.G] + BCh[CA.B]) / 255;
          Inc(PS);
          Inc(CA);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Count - 1 do
        ConvertLevelFromARGB(Levels[I]);
    end;
  end;

begin
  Result := False;
  OrigFmt := Image.DataFormat;
  if not TestImage(Image) then Exit;
  if Image.DataFormat = DestFormat then
  begin
    Result := True;
    Exit;
  end;
  try
    if DestFormat = @IDFIntensity32FRec then
    begin
      // conversion:  Another -> Intensity32F
      if Image.DataFormat^.OneChannelFormat then
        // formats are related
        ConvertIntensity(Image, @IDFIntensity32FRec)
      else
      begin
        // formats are not related -> convert to ARGB by
        //DestFormat's ConvertProc and then convert to own format
        if Image.DataFormat <> @DefFormat then
          Image.DataFormat^.ConvertFunc(Image, @DefFormat);
        for I := 0 to Image.Count - 1 do
          ConvertLevelFromARGB(Image.Levels[I]);
        Image.DataFormat := @IDFIntensity32FRec;
      end;
      Result := True;
    end
    else
      if Image.DataFormat = @IDFIntensity32FRec then
      begin
        // conversion:  Intensity32F -> Another
        if DestFormat^.OneChannelFormat then
          // formats are related
          ConvertIntensity(Image, DestFormat)
        else
        begin
          // formats are not related -> convert to ARGB and convert
          // by DestFormat's ConvertProc
          Image.DataFormat := @DefFormat;
          for I := 0 to Image.Count - 1 do
            ConvertLevelToARGB(Image.Levels[I]);
          DestFormat^.ConvertFunc(Image, DestFormat);
        end;
        Result := True;
      end;
  except
    on E: Exception do
    begin
      ShowError(ErrorStr(nil, 'StdImageDataFormats.ConvertIntensity32F',
        Format('Error when converting image from [%s] format to [%s].' +
        sLineBreak + sExceptMsg + E.Message, [OrigFmt^.Name,
        DestFormat^.Name])), E.ClassName);
    end;
  end;
end;


{*************************************************************************}
{                             A8Intensity8                                }
{*************************************************************************}


function ConvertA8Intensity8(var Image: TImageData; DestFormat:
  PImageDataFormat): Boolean;
var
  I: Long;
  OrigFmt: PImageDataFormat;

  procedure ConvertLevelToARGB(var Level: TImageLevel);
  var
    Data: Pointer;
    PW: PWord;
    PU: PULong;
    I, X, Y: Long;
  begin
    with Level do
    begin
      Size := Width * Height * DefFormat.BitCount shr 3;
      GetMem(Data, Size);
      PU := Data;
      PW := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          PU^ := ((PW^ shr 8) shl 24) or (GreyPal[Byte(PW^)].Color and
            $00FFFFFF);
          Inc(PU);
          Inc(PW);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Count - 1 do
        ConvertLevelToARGB(Levels[I]);
    end;
  end;

  procedure ConvertLevelFromARGB(var Level: TImageLevel);
  var
    I, X, Y: Long;
    CA: PARGBRec;
    Data: Pointer;
    PW: PWord;
  begin
    with Level do
    begin
      Size := Width * Height * 2;
      GetMem(Data, Size);
      PW := Data;
      CA := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          PW^ := (CA.A shl 8) or (RCh[CA.R] + GCh[CA.G] + BCh[CA.B]);
          Inc(PW);
          Inc(CA);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Count - 1 do
        ConvertLevelFromARGB(Levels[I]);
    end;
  end;

begin
  Result := False;
  OrigFmt := Image.DataFormat;
  if not TestImage(Image) then Exit;
  if Image.DataFormat = DestFormat then
  begin
    Result := True;
    Exit;
  end;
  try
    if DestFormat = @IDFA8Intensity8Rec then
    begin
      // conversion:  Another -> A8Intensity8
      if Image.DataFormat^.OneChannelFormat then
        // formats are related
        ConvertIntensity(Image, @IDFA8Intensity8Rec)
      else
      begin
        // formats are not related -> convert to ARGB by
        //DestFormat's ConvertProc and then convert to own format
        if Image.DataFormat <> @DefFormat then
          Image.DataFormat^.ConvertFunc(Image, @DefFormat);
        for I := 0 to Image.Count - 1 do
          ConvertLevelFromARGB(Image.Levels[I]);
        Image.DataFormat := @IDFA8Intensity8Rec;
      end;
      Result := True;
    end
    else
      if Image.DataFormat = @IDFA8Intensity8Rec then
      begin
        // conversion:  A8Intensity8 -> Another
        if DestFormat^.OneChannelFormat then
          // formats are related
          ConvertIntensity(Image, DestFormat)
        else
        begin
          // formats are not related -> convert to ARGB and convert
          // by DestFormat's ConvertProc
          Image.DataFormat := @DefFormat;
          for I := 0 to Image.Count - 1 do
            ConvertLevelToARGB(Image.Levels[I]);
          Result := DestFormat^.ConvertFunc(Image, DestFormat);
        end;
        Result := True;
      end;
  except
    on E: Exception do
    begin
      ShowError(ErrorStr(nil, 'StdImageDataFormats.ConvertA8Intensity8',
        Format('Error when converting image from [%s] format to [%s].' +
        sLineBreak + sExceptMsg + E.Message, [OrigFmt^.Name,
        DestFormat^.Name])), E.ClassName);
    end;
  end;
end;


{*************************************************************************}
{                         Pixel Format functions                          }
{*************************************************************************}


procedure ConvertLevelToARGB16(var Level: TImageLevel; const PF: TPixelFormat);
var
  I, X, Y: Long;
  CA: PARGBRec;
  PW: PWord;
  Data: Pointer;
  A, R, G, B: Byte;
begin
  with Level do
  begin
    Size := Width * Height * DefFormat.BitCount shr 3;
    GetMem(Data, Size);
    PW := Bits;
    CA := Data;
    for X := 0 to Width - 1 do
      for Y := 0 to Height - 1 do
      begin
        PFGetARGB(PF, PW^, A, R, G, B);
        CA.A := A;
        CA.R := R;
        CA.G := G;
        CA.B := B;
        Inc(CA);
        Inc(PW);
      end;
    FreeMemNil(Bits);
    Bits := Data;
    for I := 0 to Count - 1 do
      ConvertLevelToARGB16(Levels[I], PF);
  end;
end;

procedure ConvertLevelFromARGB16(var Level: TImageLevel; const PF:
  TPixelFormat);
var
  I, X, Y: Long;
  CA: PARGBRec;
  PW: PWord;
  Data: Pointer;
begin
  with Level do
  begin
    Size := Width * Height * 2;
    GetMem(Data, Size);
    PW := Data;
    CA := Bits;
    for X := 0 to Width - 1 do
      for Y := 0 to Height - 1 do
      begin
        PW^ := PFSetARGB(PF, CA.A, CA.R, CA.G, CA.B);
        Inc(CA);
        Inc(PW);
      end;
    FreeMemNil(Bits);
    Bits := Data;
    for I := 0 to Count - 1 do
      ConvertLevelFromARGB16(Levels[I], PF);
  end;
end;


procedure ConvertLevelToARGB8(var Level: TImageLevel; const PF: TPixelFormat);
var
  I, X, Y: Long;
  CA: PARGBRec;
  PB: PByte;
  Data: Pointer;
  A, R, G, B: Byte;
  Al, Rl, Gl, Bl: Byte;
begin
  with Level do
  begin
    Size := Width * Height * DefFormat.BitCount shr 3;
    GetMem(Data, Size);
    // Here I use color additions. In most cases when
    // converting from pixel formats with few bits per channel
    // converted image appears darker than original. This additions helps
    // a little.
    Al := 1 shl (7 - PF.ABitCount);
    Rl := 1 shl (7 - PF.RBitCount);
    Gl := 1 shl (7 - PF.GBitCount);
    Bl := 1 shl (7 - PF.BBitCount);
    PB := Bits;
    CA := Data;
    for X := 0 to Width - 1 do
      for Y := 0 to Height - 1 do
      begin
        PFGetARGB(PF, PB^, A, R, G, B);
        CA.A := A + Al;
        CA.R := R + Rl;
        CA.G := G + Gl;
        CA.B := B + Bl;
        Inc(CA);
        Inc(PB);
      end;
    FreeMemNil(Bits);
    Bits := Data;
    for I := 0 to Count - 1 do
      ConvertLevelToARGB8(Levels[I], PF);
  end;
end;

procedure ConvertLevelFromARGB8(var Level: TImageLevel; const PF:
  TPixelFormat);
var
  I, X, Y: Long;
  CA: PARGBRec;
  PB: PByte;
  Data: Pointer;
begin
  with Level do
  begin
    Size := Width * Height;
    GetMem(Data, Size);
    PB := Data;
    CA := Bits;
    for X := 0 to Width - 1 do
      for Y := 0 to Height - 1 do
      begin
        PB^ := PFSetARGB(PF, CA.A, CA.R, CA.G, CA.B);
        Inc(CA);
        Inc(PB);
      end;
    FreeMemNil(Bits);
    Bits := Data;
    for I := 0 to Count - 1 do
      ConvertLevelFromARGB8(Levels[I], PF);
  end;
end;

procedure Convert16To16(var Image: TImageData; DestFormat: PImageDataFormat);
var
  I: Long;

  procedure ConvertLevel16To16(var Level: TImageLevel; const SrcPF, DestPF:
    TPixelFormat);
  var
    I, X, Y: Long;
    PW: PWord;
    C: TARGB;
  begin
    with Level do
    begin
      PW := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          C := PFGetARGB(SrcPF, PW^);
          PW^ := PFSetARGB(DestPF, C);
          Inc(PW);
        end;
      for I := 0 to Count - 1 do
        ConvertLevel16To16(Levels[I], SrcPF, DestPF);
    end;
  end;

begin
  for I := 0 to Image.Count - 1 do
    ConvertLevel16To16(Image.Levels[I], Image.DataFormat^.PixelFormat,
      DestFormat^.PixelFormat);
  Image.DataFormat := DestFormat;
end;

procedure Convert8To8(var Image: TImageData; DestFormat: PImageDataFormat);
var
  I: Long;

  procedure ConvertLevel8To8(var Level: TImageLevel; const SrcPF, DestPF:
    TPixelFormat);
  var
    I, X, Y: Long;
    PB: PByte;
    C: TARGB;
  begin
    with Level do
    begin
      PB := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          C := PFGetARGB(SrcPF, PB^);
          PB^ := PFSetARGB(DestPF, C);
          Inc(PB);
        end;
      for I := 0 to Count - 1 do
        ConvertLevel8To8(Levels[I], SrcPF, DestPF);
    end;
  end;

begin
  for I := 0 to Image.Count - 1 do
    ConvertLevel8To8(Image.Levels[I], Image.DataFormat^.PixelFormat,
      DestFormat^.PixelFormat);
  Image.DataFormat := DestFormat;
end;

function TransARGB_PF(Format: PImageDataFormat; ARGB: TARGB; Pal: PPalArray):
  ULong;
begin
  Result := PFSetARGB(Format^.PixelFormat, ARGB);
end;

function TransColor_PF(Format: PImageDataFormat; Color: ULong; Pal: PPalArray):
  TARGB;
begin
  Result := PFGetARGB(Format^.PixelFormat, Color);
  // some colors must be manually adhusted to get original image
  if Format^.BitCount = 8 then
  begin
    if Format^.OneChannelSize > 1 then
      with PARGBRec(@Result)^ do
      begin
        A := A + 1 shl (7 - Format^.PixelFormat.ABitCount);
        R := R + 1 shl (7 - Format^.PixelFormat.RBitCount);
        G := G + 1 shl (7 - Format^.PixelFormat.GBitCount);
        B := B + 1 shl (7 - Format^.PixelFormat.BBitCount);
      end
    else
      with PARGBRec(@Result)^ do
      begin
        if A = 128 then
          A := 240
        else
          A := 10;
        if R = 128 then
          R := 240
        else
          R := 10;
        if G = 128 then
          G := 240
        else
          G := 10;
        if B = 128 then
          B := 240
        else
          B := 10;
      end;
  end
  else
    if Format = @IDFA1R5G5B5Rec then
      with PARGBRec(@Result)^ do
        if A = 128 then
          A := 255
        else
          A := 0;
end;


{*************************************************************************}
{                             R5G5B5                                      }
{*************************************************************************}


function ConvertR5G5B5(var Image: TImageData; DestFormat: PImageDataFormat):
  Boolean;
var
  I: Long;
  OrigFmt: PImageDataFormat;
begin
  Result := False;
  OrigFmt := Image.DataFormat;
  if not TestImage(Image) then Exit;
  if Image.DataFormat = DestFormat then
  begin
    Result := True;
    Exit;
  end;
  try
    if DestFormat = @IDFR5G5B5Rec then
    begin
      // conversion:  Another -> R5G5B5
      if (not Image.DataFormat^.FixedPixelFormat) and
        (Image.DataFormat^.BitCount = 16) then
        // formats are related
        Convert16To16(Image, @IDFR5G5B5Rec)
      else
      begin
        // formats are not related -> convert to ARGB by
        //DestFormat's ConvertProc and then convert to own format
        if Image.DataFormat <> @DefFormat then
          Image.DataFormat^.ConvertFunc(Image, @DefFormat);
        for I := 0 to Image.Count - 1 do
          ConvertLevelFromARGB16(Image.Levels[I], IDFR5G5B5Rec.PixelFormat);
        Image.DataFormat := @IDFR5G5B5Rec;
      end;
      Result := True;
    end
    else
      if Image.DataFormat = @IDFR5G5B5Rec then
      begin
        // conversion:  R5G5B5 -> Another
        if (not DestFormat^.FixedPixelFormat) and
          (DestFormat^.BitCount = 16) then
          // formats are related
          Convert16To16(Image, DestFormat)
        else
        begin
          // formats are not related -> convert to ARGB and convert
          // by DestFormat's ConvertProc
          Image.DataFormat := @DefFormat;
          for I := 0 to Image.Count - 1 do
            ConvertLevelToARGB16(Image.Levels[I], IDFR5G5B5Rec.PixelFormat);
          DestFormat^.ConvertFunc(Image, DestFormat);
        end;
        Result := True;
      end;
  except
    on E: Exception do
    begin
      ShowError(ErrorStr(nil, 'StdImageDataFormats.ConvertR5G5B5',
        Format('Error when converting image from [%s] format to [%s].' +
        sLineBreak + sExceptMsg + E.Message, [OrigFmt^.Name,
        DestFormat^.Name])), E.ClassName);
    end;
  end;
end;


{*************************************************************************}
{                             A1R5G5B5                                    }
{*************************************************************************}


function ConvertA1R5G5B5(var Image: TImageData; DestFormat: PImageDataFormat):
  Boolean;
var
  I: Long;
  OrigFmt: PImageDataFormat;

  // when converting from A1R5G5B5 alpha must be manually adjusted
  procedure CheckAlpha(var Level: TImageLevel);
  var
    I, X, Y: Long;
    PC: PARGBRec;
  begin
    PC := Level.Bits;
    for X := 0 to Level.Width - 1 do
      for Y := 0 to Level.Height - 1 do
      begin
        if PC.A = 128 then
          PC.A := 255;
        Inc(PC);
      end;
    for I := 0 to Level.Count - 1 do
      CheckAlpha(Level.Levels[I]);
  end;

begin
  Result := False;
  OrigFmt := Image.DataFormat;
  if not TestImage(Image) then Exit;
  if Image.DataFormat = DestFormat then
  begin
    Result := True;
    Exit;
  end;
  try
    if DestFormat = @IDFA1R5G5B5Rec then
    begin
      // conversion:  Another -> A1R5G5B5
      if (not Image.DataFormat^.FixedPixelFormat) and
        (Image.DataFormat^.BitCount = 16) then
       // formats are related
        Convert16To16(Image, @IDFA1R5G5B5Rec)
      else
      begin
        // formats are not related -> convert to ARGB by
        //DestFormat's ConvertProc and then convert to own format
        if Image.DataFormat <> @DefFormat then
          Image.DataFormat^.ConvertFunc(Image, @DefFormat);
        for I := 0 to Image.Count - 1 do
          ConvertLevelFromARGB16(Image.Levels[I], IDFA1R5G5B5Rec.PixelFormat);
        Image.DataFormat := @IDFA1R5G5B5Rec;
      end;
      Result := True;
    end
    else
      if Image.DataFormat = @IDFA1R5G5B5Rec then
      begin
        // conversion:  A1R5G5B5 -> Another
        if (not DestFormat^.FixedPixelFormat) and
          (DestFormat^.BitCount = 16) then
          // formats are related
          Convert16To16(Image, DestFormat)
        else
        begin
          // formats are not related -> convert to ARGB and convert
          // by DestFormat's ConvertProc
          Image.DataFormat := @DefFormat;
          for I := 0 to Image.Count - 1 do
          begin
            ConvertLevelToARGB16(Image.Levels[I], IDFA1R5G5B5Rec.PixelFormat);
            CheckAlpha(Image.Levels[I]);
          end;
          DestFormat^.ConvertFunc(Image, DestFormat);
        end;
        Result := True;
      end;
  except
    on E: Exception do
    begin
      ShowError(ErrorStr(nil, 'StdImageDataFormats.ConvertA1R5G5B5',
        Format('Error when converting image from [%s] format to [%s].' +
        sLineBreak + sExceptMsg + E.Message, [OrigFmt^.Name,
        DestFormat^.Name])), E.ClassName);
    end;
  end;
end;


{*************************************************************************}
{                             A4R4G4B4                                    }
{*************************************************************************}


function ConvertA4R4G4B4(var Image: TImageData; DestFormat: PImageDataFormat):
  Boolean;
var
  I: Long;
  OrigFmt: PImageDataFormat;
begin
  Result := False;
  OrigFmt := Image.DataFormat;
  if not TestImage(Image) then Exit;
  if Image.DataFormat = DestFormat then
  begin
    Result := True;
    Exit;
  end;
  try
    if DestFormat = @IDFA4R4G4B4Rec then
    begin
      // conversion:  Another -> A4R4G4B4
      if (not Image.DataFormat^.FixedPixelFormat) and
        (Image.DataFormat^.BitCount = 16) then
        // formats are related
        Convert16To16(Image, @IDFA4R4G4B4Rec)
      else
      begin
        // formats are not related -> convert to ARGB by
        //DestFormat's ConvertProc and then convert to own format
        if Image.DataFormat <> @DefFormat then
          Image.DataFormat^.ConvertFunc(Image, @DefFormat);
        for I := 0 to Image.Count - 1 do
          ConvertLevelFromARGB16(Image.Levels[I], IDFA4R4G4B4Rec.PixelFormat);
        Image.DataFormat := @IDFA4R4G4B4Rec;
      end;
      Result := True;
    end
    else
      if Image.DataFormat = @IDFA4R4G4B4Rec then
      begin
        // conversion:  A4R4G4B4 -> Another
        if (not DestFormat^.FixedPixelFormat) and
          (DestFormat^.BitCount = 16) then
          // formats are related
          Convert16To16(Image, DestFormat)
        else
        begin
          // formats are not related -> convert to ARGB and convert
          // by DestFormat's ConvertProc
          Image.DataFormat := @DefFormat;
          for I := 0 to Image.Count - 1 do
            ConvertLevelToARGB16(Image.Levels[I], IDFA4R4G4B4Rec.PixelFormat);
          DestFormat^.ConvertFunc(Image, DestFormat);
        end;
        Result := False;
      end;
  except
    on E: Exception do
    begin
      ShowError(ErrorStr(nil, 'StdImageDataFormats.ConvertA4R4G4B4',
        Format('Error when converting image from [%s] format to [%s].' +
        sLineBreak + sExceptMsg + E.Message, [OrigFmt^.Name,
        DestFormat^.Name])), E.ClassName);
    end;
  end;
end;


{*************************************************************************}
{                             R3G3B2                                      }
{*************************************************************************}


function ConvertR3G3B2(var Image: TImageData; DestFormat: PImageDataFormat):
  Boolean;
var
  I: Long;
  OrigFmt: PImageDataFormat;
begin
  Result := False;
  OrigFmt := Image.DataFormat;
  if not TestImage(Image) then Exit;
  if Image.DataFormat = DestFormat then
  begin
    Result := True;
    Exit;
  end;
  try
    if DestFormat = @IDFR3G3B2Rec then
    begin
      // conversion:  Another -> R3G3B2
      if (not Image.DataFormat^.FixedPixelFormat) and
        (Image.DataFormat^.BitCount = 8) then
        // formats are related
        Convert8To8(Image, @IDFR3G3B2Rec)
      else
      begin
        // formats are not related -> convert to ARGB by
        //DestFormat's ConvertProc and then convert to own format
        if Image.DataFormat <> @DefFormat then
          Image.DataFormat^.ConvertFunc(Image, @DefFormat);
        for I := 0 to Image.Count - 1 do
          ConvertLevelFromARGB8(Image.Levels[I], IDFR3G3B2Rec.PixelFormat);
        Image.DataFormat := @IDFR3G3B2Rec;
      end;
      Result := True;
    end
    else
      if Image.DataFormat = @IDFR3G3B2Rec then
      begin
        // conversion:  R3G3B2 -> Another
        if (not DestFormat^.FixedPixelFormat) and
          (DestFormat^.BitCount = 8) then
          // formats are related
          Convert8To8(Image, @IDFR3G3B2Rec)
        else
        begin
          // formats are not related -> convert to ARGB and convert
          // by DestFormat's ConvertProc
          Image.DataFormat := @DefFormat;
          for I := 0 to Image.Count - 1 do
            ConvertLevelToARGB8(Image.Levels[I], IDFR3G3B2Rec.PixelFormat);
          DestFormat^.ConvertFunc(Image, DestFormat);
        end;
        Result := True;
      end;
  except
    on E: Exception do
    begin
      ShowError(ErrorStr(nil, 'StdImageDataFormats.ConvertR3G3B2',
        Format('Error when converting image from [%s] format to [%s].' +
        sLineBreak + sExceptMsg + E.Message, [OrigFmt^.Name,
        DestFormat^.Name])), E.ClassName);
    end;
  end;
end;


{*************************************************************************}
{                             R1G1B1                                      }
{*************************************************************************}


function ConvertR1G1B1(var Image: TImageData; DestFormat: PImageDataFormat):
  Boolean;
var
  I: Long;
  OrigFmt: PImageDataFormat;

// when converting from R1G1B1 colors must be manually adjusted
  procedure CheckColors(var Level: TImageLevel);
  var
    X, Y, I: LOng;
    PC: PARGBRec;
  begin
    PC := Level.Bits;
    for X := 0 to Level.Width - 1 do
      for Y := 0 to Level.Height - 1 do
      begin
        if PC.R = 192 then
          PC.R := 240
        else
          PC.R := 10;
        if PC.G = 192 then
          PC.G := 240
        else
          PC.G := 10;
        if PC.B = 192 then
          PC.B := 240
        else
          PC.B := 10;
        Inc(PC);
      end;
    for I := 0 to Level.Count - 1 do
      CheckColors(Level.Levels[I]);
  end;

begin
  Result := False;
  OrigFmt := Image.DataFormat;
  if not TestImage(Image) then Exit;
  if Image.DataFormat = DestFormat then
  begin
    Result := True;
    Exit;
  end;
  try
    if DestFormat = @IDFR1G1B1Rec then
    begin
      // conversion:  Another -> R1G1B1
      if (not Image.DataFormat^.FixedPixelFormat) and
        (Image.DataFormat^.BitCount = 8) then
        // formats are related
        Convert8To8(Image, @IDFR1G1B1Rec)
      else
      begin
        // formats are not related -> convert to ARGB by
        //DestFormat's ConvertProc and then convert to own format
        if Image.DataFormat <> @DefFormat then
          Image.DataFormat^.ConvertFunc(Image, @DefFormat);
        for I := 0 to Image.Count - 1 do
          ConvertLevelFromARGB8(Image.Levels[I], IDFR1G1B1Rec.PixelFormat);
        Image.DataFormat := @IDFR1G1B1Rec;
      end;
      Result := True;
    end
    else
      if Image.DataFormat = @IDFR1G1B1Rec then
      begin
        // conversion:  R1G1B1 -> Another
        if (not DestFormat^.FixedPixelFormat) and
          (DestFormat^.BitCount = 8) then
          // formats are related
          Convert8To8(Image, @IDFR1G1B1Rec)
        else
        begin
          // formats are not related -> convert to ARGB and convert
          // by DestFormat's ConvertProc
          Image.DataFormat := @DefFormat;
          for I := 0 to Image.Count - 1 do
          begin
            ConvertLevelToARGB8(Image.Levels[I], IDFR1G1B1Rec.PixelFormat);
            CheckColors(Image.Levels[I]);
          end;
          DestFormat^.ConvertFunc(Image, DestFormat);
        end;
        Result := True;
      end;
  except
    on E: Exception do
    begin
      ShowError(ErrorStr(nil, 'StdImageDataFormats.ConvertR1G1B1',
        Format('Error when converting image from [%s] format to [%s].' +
        sLineBreak + sExceptMsg + E.Message, [OrigFmt^.Name,
        DestFormat^.Name])), E.ClassName);
    end;
  end;
end;


{*************************************************************************}
{                         48/64bit images functions                       }
{*************************************************************************}


procedure Convert48To64(var Image: TImageData);
var
  I: Long;

  procedure ConvertLevel48To64(var Level: TImageLevel);
  var
    I, X, Y: Long;
    CA64: PARGB64Rec;
    C48: PRGB48Rec;
    Data: Pointer;
  begin
    with Level do
    begin
      Size := Width * Height * 8;
      GetMem(Data, Size);
      CA64 := Data;
      C48 := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          CA64.A := $FFFF;
          CA64.R := C48.R;
          CA64.G := C48.G;
          CA64.B := C48.B;
          Inc(CA64);
          Inc(C48);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Count - 1 do
        ConvertLevel48To64(Levels[I]);
    end;
  end;

begin
  for I := 0 to Image.Count - 1 do
    ConvertLevel48To64(Image.Levels[I]);
  Image.DataFormat := @IDFA16R16G16B16Rec;
end;

procedure Convert64To48(var Image: TImageData);
var
  I: Long;

  procedure ConvertLevel64To48(var Level: TImageLevel);
  var
    I, X, Y: Long;
    CA64: PARGB64Rec;
    C48: PRGB48Rec;
    Data: Pointer;
  begin
    with Level do
    begin
      Size := Width * Height * 6;
      GetMem(Data, Size);
      C48 := Data;
      CA64 := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          C48.R := CA64.R;
          C48.G := CA64.G;
          C48.B := CA64.B;
          Inc(CA64);
          Inc(C48);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Count - 1 do
        ConvertLevel64To48(Levels[I]);
    end;
  end;

begin
  for I := 0 to Image.Count - 1 do
    ConvertLevel64To48(Image.Levels[I]);
  Image.DataFormat := @IDFR16G16B16Rec;
end;


{*************************************************************************}
{                             A16R16G16B16                                }
{*************************************************************************}


function ConvertA16R16G16B16(var Image: TImageData; DestFormat:
  PImageDataFormat): Boolean;
var
  I: Long;
  OrigFmt: PImageDataFormat;

  procedure ConvertLevelToARGB(var Level: TImageLevel);
  var
    I, X, Y: Long;
    CA: PARGBRec;
    CA64: PARGB64Rec;
    Data: Pointer;
  begin
    with Level do
    begin
      Size := Width * Height * DefFormat.BitCount shr 3;
      GetMem(Data, Size);
      CA := Data;
      CA64 := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          CA.A := CA64.A shr 8;
          CA.R := CA64.R shr 8;
          CA.G := CA64.G shr 8;
          CA.B := CA64.B shr 8;
          Inc(CA);
          Inc(CA64);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Count - 1 do
        ConvertLevelToARGB(Levels[I]);
    end;
  end;

  procedure ConvertLevelFromARGB(var Level: TImageLevel);
  var
    I, X, Y: Long;
    CA: PARGBRec;
    CA64: PARGB64Rec;
    Data: Pointer;
  begin
    with Level do
    begin
      Size := Width * Height * 8;
      GetMem(Data, Size);
      CA64 := Data;
      CA := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          CA64.A := CA.A shl 8;
          CA64.R := CA.R shl 8;
          CA64.G := CA.G shl 8;
          CA64.B := CA.B shl 8;
          Inc(CA);
          Inc(CA64);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Count - 1 do
        ConvertLevelFromARGB(Levels[I]);
    end;
  end;

begin
  Result := False;
  OrigFmt := Image.DataFormat;
  if not TestImage(Image) then Exit;
  if Image.DataFormat = DestFormat then
  begin
    Result := True;
    Exit;
  end;
  try
    if DestFormat = @IDFA16R16G16B16Rec then
    begin
      // conversion:  Another -> A16R16G16B16
      if Image.DataFormat = @IDFR16G16B16Rec then
        Convert48To64(Image)
      else
      begin
        // not related -> convert to ARGB by DestFormat's ConvertProc and then
        // convert to own format
        if Image.DataFormat <> @DefFormat then
          Image.DataFormat^.ConvertFunc(Image, @DefFormat);
        for I := 0 to Image.Count - 1 do
          ConvertLevelFromARGB(Image.Levels[I]);
        Image.DataFormat := @IDFA16R16G16B16Rec;
      end;
      Result := True;
    end
    else
      if Image.DataFormat = @IDFA16R16G16B16Rec then
      begin
        if DestFormat = @IDFR16G16B16Rec then
          Convert64To48(Image)
        else
        begin
          // conversion:  A16R16G16B16 -> Another
          // convert to ARGB and convert by DestFormat's ConvertProc
          Image.DataFormat := @DefFormat;
          for I := 0 to Image.Count - 1 do
            ConvertLevelToARGB(Image.Levels[I]);
          DestFormat^.ConvertFunc(Image, DestFormat);
        end;
        Result := True;
      end;
  except
    on E: Exception do
    begin
      ShowError(ErrorStr(nil, 'StdImageDataFormats.ConvertA16R16G16B16',
        Format('Error when converting image from [%s] format to [%s].' +
        sLineBreak + sExceptMsg + E.Message, [OrigFmt^.Name,
        DestFormat^.Name])), E.ClassName);
    end;
  end;
end;


{*************************************************************************}
{                             R16G16B16                                   }
{*************************************************************************}


function ConvertR16G16B16(var Image: TImageData; DestFormat:
  PImageDataFormat): Boolean;
var
  I: Long;
  OrigFmt: PImageDataFormat;

  procedure ConvertLevelToARGB(var Level: TImageLevel);
  var
    I, X, Y: Long;
    CA: PARGBRec;
    C48: PRGB48Rec;
    Data: Pointer;
  begin
    with Level do
    begin
      Size := Width * Height * DefFormat.BitCount shr 3;
      GetMem(Data, Size);
      CA := Data;
      C48 := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          CA.A := $FF;
          CA.R := C48.R shr 8;
          CA.G := C48.G shr 8;
          CA.B := C48.B shr 8;
          Inc(CA);
          Inc(C48);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Count - 1 do
        ConvertLevelToARGB(Levels[I]);
    end;
  end;

  procedure ConvertLevelFromARGB(var Level: TImageLevel);
  var
    I, X, Y: Long;
    CA: PARGBRec;
    C48: PRGB48Rec;
    Data: Pointer;
  begin
    with Level do
    begin
      Size := Width * Height * 6;
      GetMem(Data, Size);
      C48 := Data;
      CA := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          C48.R := CA.R shl 8;
          C48.G := CA.G shl 8;
          C48.B := CA.B shl 8;
          Inc(CA);
          Inc(C48);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Count - 1 do
        ConvertLevelFromARGB(Levels[I]);
    end;
  end;

begin
  Result := False;
  OrigFmt := Image.DataFormat;
  if not TestImage(Image) then Exit;
  if Image.DataFormat = DestFormat then
  begin
    Result := True;
    Exit;
  end;
  try
    if DestFormat = @IDFR16G16B16Rec then
    begin
      // conversion:  Another -> R16G16B16
      if Image.DataFormat = @IDFA16R16G16B16Rec then
        Convert64To48(Image)
      else
      begin
        // not related -> convert to ARGB by DestFormat's ConvertProc and then
        // convert to own format
        if Image.DataFormat <> @DefFormat then
          Image.DataFormat^.ConvertFunc(Image, @DefFormat);
        for I := 0 to Image.Count - 1 do
          ConvertLevelFromARGB(Image.Levels[I]);
        Image.DataFormat := @IDFR16G16B16Rec;
      end;
      Result := True;
    end
    else
      if Image.DataFormat = @IDFR16G16B16Rec then
      begin
        if DestFormat = @IDFA16R16G16B16Rec then
          Convert48To64(Image)
        else
        begin
          // conversion:  R16G16B16 -> Another
          // convert to ARGB and convert by DestFormat's ConvertProc
          Image.DataFormat := @DefFormat;
          for I := 0 to Image.Count - 1 do
            ConvertLevelToARGB(Image.Levels[I]);
          DestFormat^.ConvertFunc(Image, DestFormat);
        end;
        Result := True;
      end;
  except
    on E: Exception do
    begin
      ShowError(ErrorStr(nil, 'StdImageDataFormats.ConvertR16G16B16',
        Format('Error when converting image from [%s] format to [%s].' +
        sLineBreak + sExceptMsg + E.Message, [OrigFmt^.Name,
        DestFormat^.Name])), E.ClassName);
    end;
  end;
end;


{*************************************************************************}
{                         Floating-point functions                        }
{*************************************************************************}


procedure ConvertFloat(var Image: TImageData; DestFormat: PImageDataFormat);
begin
  {TODO :Add code here. When some related formats will be added.}
end;


{*************************************************************************}
{                             A32R32G32B32F                               }
{*************************************************************************}


function ConvertA32R32G32B32F(var Image: TImageData; DestFormat:
  PImageDataFormat): Boolean;
var
  I: Long;
  OrigFmt: PImageDataFormat;

  procedure ConvertLevelToARGB(var Level: TImageLevel);
  var
    I, X, Y: Long;
    CA: PARGBRec;
    CAF: PARGBFRec;
    Data: Pointer;
  begin
    with Level do
    begin
      Size := Width * Height * DefFormat.BitCount shr 3;
      GetMem(Data, Size);
      CA := Data;
      CAF := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          CA.A := Round(CAF.A * 255);
          CA.R := Round(CAF.R * 255);
          CA.G := Round(CAF.G * 255);
          CA.B := Round(CAF.B * 255);
          Inc(CAF);
          Inc(CA);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Count - 1 do
        ConvertLevelToARGB(Levels[I]);
    end;
  end;

  procedure ConvertLevelFromARGB(var Level: TImageLevel);
  var
    I, X, Y: Long;
    CA: PARGBRec;
    CAF: PARGBFRec;
    Data: Pointer;
  begin
    with Level do
    begin
      Size := Width * Height * 16;
      GetMem(Data, Size);
      CAF := Data;
      CA := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          CAF.A := CA.A / 255;
          CAF.R := CA.R / 255;
          CAF.G := CA.G / 255;
          CAF.B := CA.B / 255;
          Inc(CAF);
          Inc(CA);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Count - 1 do
        ConvertLevelFromARGB(Levels[I]);
    end;
  end;

begin
  Result := False;
  OrigFmt := Image.DataFormat;
  if not TestImage(Image) then Exit;
  if Image.DataFormat = DestFormat then
  begin
    Result := True;
    Exit;
  end;
  try
    if DestFormat = @IDFA32R32G32B32FRec then
    begin
      // conversion:  Another -> A32R32G32B32F
      if (Image.DataFormat^.FloatFormat) and
        (not Image.DataFormat^.OneChannelFormat) then
        // formats are related
        ConvertFloat(Image, @IDFA32R32G32B32FRec)
      else
      begin
        // formats are not related -> convert to ARGB by
        //DestFormat's ConvertProc and then convert to own format
        if Image.DataFormat <> @DefFormat then
          Image.DataFormat^.ConvertFunc(Image, @DefFormat);
        for I := 0 to Image.Count - 1 do
          ConvertLevelFromARGB(Image.Levels[I]);
        Image.DataFormat := @IDFA32R32G32B32FRec;
      end;
      Result := True;
    end
    else
      if Image.DataFormat = @IDFA32R32G32B32FRec then
      begin
        // conversion:  A32R32G32B32F -> Another
        if (DestFormat^.FloatFormat) and
          (not DestFormat^.OneChannelFormat) then
          // formats are related
          ConvertFloat(Image, DestFormat)
        else
        begin
          // formats are not related -> convert to ARGB and convert
          // by DestFormat's ConvertProc
          Image.DataFormat := @DefFormat;
          for I := 0 to Image.Count - 1 do
            ConvertLevelToARGB(Image.Levels[I]);
          DestFormat^.ConvertFunc(Image, DestFormat);
        end;
        Result := True;
      end;
  except
    on E: Exception do
    begin
      ShowError(ErrorStr(nil, 'StdImageDataFormats.ConvertA32R32G32B32F',
        Format('Error when converting image from [%s] format to [%s].' +
        sLineBreak + sExceptMsg + E.Message, [OrigFmt^.Name,
        DestFormat^.Name])), E.ClassName);
    end;
  end;
end;

var
  Loop: Long;

initialization

  InitFormats;

  // A8R8G8B8 is the first registered format so it will
  // be used as Default format
  {(*}
  RegisterImageDataFormats
  ([
    @IDFA8R8G8B8Rec,
    @IDFR8G8B8Rec,
    @IDFIndex8Rec,
    @IDFIntensity8Rec,
    @IDFIntensity16Rec,
    @IDFIntensity32Rec,
    @IDFIntensity32FRec,
    @IDFA8Intensity8Rec,
    @IDFR5G5B5Rec,
    @IDFA1R5G5B5Rec,
    @IDFA4R4G4B4Rec,
    @IDFR3G3B2Rec,
    @IDFR1G1B1Rec,
    @IDFA16R16G16B16Rec,
    @IDFR16G16B16Rec,
    @IDFA32R32G32B32FRec
  ]);
  {*)}

  for Loop := 0 to 255 do
  begin
    RCh[Loop] := Trunc(0.3588 * Loop);
    GCh[Loop] := Trunc(0.4020 * Loop);
    BCh[Loop] := Trunc(0.2392 * Loop);
  end;

  GreyPal := GreyScalePal(256);

finalization

  FreeMemNil(GreyPal);

end.

