{*************************************************************************}
{*   Compiler                                                            *}
{*   Copyright (C) 2003 Marek Mauder, Vampyre SoftWorks                  *}
{*   Engine\Include\Compiler.inc                                         *}
{*************************************************************************}


{*************************************************************************}
{                         Compilation settings                            }
{*************************************************************************}


{$ALIGN 4}             // Fields alignment: 4 B
{$ASSERTIONS ON}       // Assertions: on
{$BOOLEVAL OFF}        // Boolean eval: off
{$EXTENDEDSYNTAX ON}   // Extended syntax: on
{$LONGSTRINGS ON}      // string = AnsiString: on
{$MINENUMSIZE 4}       // Min enum size: 4 B
{$TYPEDADDRESS OFF}    // Typed pointers: off
{$WRITEABLECONST OFF}  // Writeable constants: off


{*************************************************************************}
{                         Engine settings                                 }
{*************************************************************************}


{Define this symbol when creating module DLLs.
 This disables usage of some units (which you don't need in library)
 your DLL will be significantly smaller.}
{ $DEFINE MODLIBRARY}

{Use our custom memory manager. Always use custom mem manager when
 creating DLL module in Windows (DLL and EXE must use tha same manager!).
 However, you can use it on other platforms too because
 if DEBUG and LOGTOFILES are defined mem mgr writes lot of
 info to memory debug log (memory leaks also).}
{$DEFINE USEMEMMGR}

{Define this symbol to use assembler versions of some functions and
 procedures (faster). You should add no-asm version of all asm functions
 you put into engine. Because maybe some day engine will work on non X86
 architecture. However you dont have to add non-asm versions in platform
 dependent units (which are used only with specific platform) e.g. CPUID
 in CPUX86Info (which can not be done without asm).
 Note that under Linux libraries are compiled to PIC code so assembler code
 can not use global variable (it can of course but asm code is different
 from normal). In this case some asm procedure which use globals are not used
 and non-asm ones are called (or PIC asm ones are called).}
{$DEFINE ASMVERSIONS}

{Define which floating point format will be used as default for engine.
 If you define Double then Single is automaticaly undefined further.}
{$DEFINE FLOATSINGLE}
{ $DEFINE FLOATDOUBLE}

{Define this symbol to allow application to save logs into files.}
{$DEFINE LOGTOFILES}

{Define this when you are creating console application.
 All messages,... are written into console. If not defined all messages, ...
 creates windows for them (Some window managers and renderers are not
 accessible in console apps).
 Important: Under Windows we are checking IsConsole variable which
 is set according to APPTYPE CONSOLE directive.
 Under Linux (and DOS) all apps are console apps so use this define when
 you want engine to behave as console app.}
{$DEFINE CONSOLE}

{Define this when you want more info about program flow and other.
 Note that app compiled in debug mode runs slower.}
{ $DEFINE DEBUG}


{*************************************************************************}
{                         Platform and Compiler                           }
{*************************************************************************}


{Only x86 CPU is supported now.}
{$DEFINE CPUX86}

{Compilers supported: DELPHI 4+, KYLIX 1+, FPC 1.9.2+, BCB 4+}
{Operating systems supported: Win32, Linux, DOS DPMI}

{$IFDEF WIN32}
  {$IFNDEF MSWINDOWS}
    {$DEFINE MSWINDOWS}
  {$ENDIF}
{$ENDIF}

{$IFDEF DPMI}
  {$DEFINE MSDOS}
  {$UNDEF USEMEMMGR}
{$ENDIF}

{$IFDEF UNIX}
  {$IFNDEF LINUX}
    {$DEFINE LINUX}
  {$ENDIF}
{$ENDIF}

{$IFDEF CLR}
  {$DEFINE DOTNET}
  {$UNSAFECODE ON}
  {$UNDEF AMSVERSIONS}
  {$UNDEF USEMEMMGR}
{$ENDIF}

{$IFDEF FPC}
  {$ASMMODE INTEL}
  {$MODE DELPHI}
  {$SMARTLINK ON}
  {$GOTO ON}
  {$PACKRECORDS 4}
{$ENDIF}

{$IFDEF VER150}
   {$IFNDEF BCB}
     {$DEFINE DELPHI}
     {$DEFINE DELPHI7}
   {$ENDIF}
   {$WARN UNSAFE_CODE OFF}
   {$WARN UNSAFE_TYPE OFF}
   {$WARN UNSAFE_CAST OFF}
   {$WARN SYMBOL_DEPRECATED OFF}
   {$WARN UNIT_DEPRECATED OFF}
{$ENDIF}

{$IFDEF VER140}
   {$IFNDEF BCB}
     {$IFNDEF LINUX}
       {$DEFINE DELPHI}
       {$DEFINE DELPHI6}
     {$ENDIF}
   {$ENDIF}
{$ENDIF}

{$IFDEF VER130}
   {$IFNDEF BCB}
     {$DEFINE DELPHI}
     {$DEFINE DELPHI5}
   {$ENDIF}
{$ENDIF}

{$IFDEF VER120}
   {$DEFINE DELPHI}
   {$DEFINE DELPHI4}
{$ENDIF}

{$IFDEF LINUX}
  {$IFNDEF FPC}
    {$DEFINE KYLIX}
    {$DEFINE BORLAND}
  {$ENDIF}
{$ENDIF}

{$IFDEF DELPHI}
  {$DEFINE BORLAND}
{$ENDIF}

{$IFDEF BCB}
  {$DEFINE BORLAND}
{$ENDIF}

{$IFNDEF DELPHI}
  {$IFNDEF KYLIX}
    {$IFNDEF FPC}
      {$IFNDEF BCB}
        {$DEFINE UNKCOMPILER}
      {$ENDIF}
    {$ENDIF}
  {$ENDIF}
{$ENDIF}


{*************************************************************************}
{                         Settings modifications                          }
{*************************************************************************}


{$IFDEF FLOATSINGLE}
   {$UNDEF FLOATDOUBLE}
{$ENDIF}

{$IFDEF FLOATDOUBLE}
   {$UNDEF FLOATSINGLE}
{$ENDIF}

{$IFNDEF FLOATSINGLE}
  {$IFNDEF FLOATDOUBLE}
     {$DEFINE FLOATSINGLE}
  {$ENDIF}
{$ENDIF}

{$IFDEF ASMVERSIONS}
  {$IFNDEF CPUX86}
    {$UNDEF ASMVERSIONS}
  {$ENDIF}
{$ENDIF}

(*
{$IFDEF LIBRARY}       // Make this work. System.IsLibrary is variable AAAAARGH
  {$IFDEF LOGTOFILES}
    {$UNDEF LOGTOFILES}
  {$ENDIF}
  {$IFNDEF USEMEMMGR}
    {$DEFINE USEMEMMGR}
  {$ENDIF}
{$ELSE}
  {$IFDEF MODLIBRARY}
    {$UNDEF MODLIBRARY}
  {$ENDIF}
{$ENDIF}
*)
