{*************************************************************************}
{*   LinuxMemMgrs                                                        *}
{*   Copyright (C) 2004 Marek Mauder, Vampyre SoftWorks                  *}
{*   Engine\Kernel\Platforms\Linux\LinuxPlatform.pas                     *}
{*************************************************************************}
{                                                                         }
{ This program is free software; you can redistribute it and/or modify it }
{ under the terms of the GNU General Public License as published by the   }
{ Free Software Foundation; either version 2 of the License, or (at your  }
{ option) any later version. This program is distributed in the hope that }
{ it will be useful, but WITHOUT ANY WARRANTY; without even the implied   }
{ warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See   }
{ the GNU General Public License for more details.                        }
{                                                                         }
{ You should have received a copy of the GNU General Public License       }
{ along with this program; if not, write to the Free Software Foundation, }
{ Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.         }
{                                                                         }
{*************************************************************************}
{  Created On: 31st January 2004  (31.01.2004)                            }
{*************************************************************************}

{This unit contains memory managers used on Linux systems. There are no
 such problems like on Windows so there is only defaulr one (two) managers.
 You can find benchmarks of managers somewhere in Doc directory.}

unit LinuxMemMgrs;

interface

{$I Compiler.inc}

uses
  Libc;

var
  {Standard mem mgr set by Kylix (or FPC). In Kylix this mgr calls
   the same functions as CMemoryManager.}
  DelphiMemoryManager: TMemoryManager;
  {Uses standard C functions malloc/realloc/free imported form libc.}
  CMemoryManager: TMemoryManager;

{Returns pointer to memory manager that is actually used.}
function ActiveMemoryManager: PMemoryManager;
{Returns pointer to manager used in MemMgrs.pas when DEBUG is on.}
function DebugMemoryManager: PMemoryManager;


{*************************************************************************}
{                             IMPLEMEMTATION                              }
{*************************************************************************}


implementation

function ActiveMemoryManager: PMemoryManager;
begin
  Result := @CMemoryManager;
end;

function DebugMemoryManager: PMemoryManager;
begin
  Result := @CMemoryManager;
end;

{* C memory manager *}

const
  LibName = 'libc.so.6';

function Malloc(Size: size_t): Pointer; cdecl; external LibName name 'malloc';
procedure Free(P: Pointer); cdecl; external LibName name 'free';
function ReAlloc(P: Pointer; Size: size_t): Pointer; cdecl; external LibName
  name 'realloc';

function CGetMem(Size: Integer): Pointer;
begin
  Result := Malloc(Size);
end;

function CFreeMem(P: Pointer): Integer;
begin
  Free(P);
  Result := 0;
end;

function CReallocMem(P: Pointer; Size: Integer): Pointer;
begin
  Result := Realloc(P, Size);
end;


{*************************************************************************}
{                         Internal stuff                                  }
{*************************************************************************}


initialization

  GetMemoryManager(DelphiMemoryManager);

  CMemoryManager.GetMem := CGetMem;
  CMemoryManager.FreeMem := CFreeMem;
  CMemoryManager.ReallocMem := CReallocMem;

end.

