{*************************************************************************}
{*   LinuxSysInfo                                                        *}
{*   Copyright (C) 2003 Marek Mauder, Vampyre SoftWorks                  *}
{*   Engine\Kernel\Platforms\Linux\LinuxSysInfo.pas                      *}
{*************************************************************************}
{                                                                         }
{ This program is free software; you can redistribute it and/or modify it }
{ under the terms of the GNU General Public License as published by the   }
{ Free Software Foundation; either version 2 of the License, or (at your  }
{ option) any later version. This program is distributed in the hope that }
{ it will be useful, but WITHOUT ANY WARRANTY; without even the implied   }
{ warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See   }
{ the GNU General Public License for more details.                        }
{                                                                         }
{ You should have received a copy of the GNU General Public License       }
{ along with this program; if not, write to the Free Software Foundation, }
{ Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.         }
{                                                                         }
{*************************************************************************}
{  Created On: 4th October 2003  (04.10.2003)                             }
{*************************************************************************}

{Contains function for retrieve CPU, memory and other system info under
 Linux.}

unit LinuxSysInfo;

{$I Compiler.inc}

interface
uses
  SysUtils, Kernel, Platforms, Libc
{$IFNDEF MODLIBRARY}
  , X86CpuInfo
{$ENDIF};

procedure GetCPUInfo(var Info: TCPUInfo);
procedure GetOSInfo(var Version: TVersionInfo; var Name: NString);
procedure GetMemoryInfo(var Info: TMemoryInfo);


{*************************************************************************}
{                             Implementation                              }
{*************************************************************************}


implementation

function LinuxPITFunction: Int64;
var
  TV:TTimeVal;
begin
  GetTimeOfDay(TV, nil);
  Result := Int64(TV.tv_sec) * 1000000 + TV.tv_usec;
end;

procedure GetCPUInfo(var Info: TCPUInfo);
begin
  X86CpuInfo.GetX86CPUInfo(Info, 1000000, LinuxPITFunction);
  Info.ProcessorCount :=1;
end;

procedure GetOSInfo(var Version: TVersionInfo; var Name: NString);
var
  S: TUTSName;
  Rel: NString;
  I: Long;
begin
  UName(S);
  Name := S.SysName;
  Rel := S.Release;
  if Pos('.', Rel) > 0 then
  try
    I := Pos('.', Rel);
    Version.Major := StrToInt(Copy(Rel, 1, I - 1));
    Delete(Rel, 1, I);
    I := Pos('.', Rel);
    Version.Minor := StrToInt(Copy(Rel, 1, I - 1));
    Delete(Rel, 1, I);
    I := Pos('.', Rel);
    if Pos('-', Rel) < I then
      I := Pos('-', Rel);
    Version.Build := StrToInt(Copy(Rel, 1, I - 1));
    Delete(Rel, 1, I);
  except
    Version.Build := 0;
  end
  else
    Version.Version := 0;
end;

procedure GetMemoryInfo(var Info: TMemoryInfo);
var
  SysI: TSysInfo;
begin
  LibC.SysInfo(SysI);
  Info.CurrentLoad := SysI.Loads[0];
  Info.PhysicalTotal := SysI.TotalRam div 1024;
  Info.PhysicalFree := SysI.FreeRam div 1024;
  Info.PageFileTotal := SysI.TotalSwap div 1024;
  Info.PageFileFree := SysI.FreeSwap div 1024;
  Info.VirtualTotal := Info.PhysicalTotal + Info.PageFileTotal;
  Info.VirtualFree := Info.PhysicalFree + Info.PageFileFree;
end;

end.

 