{*************************************************************************}
{*   Win32Registry                                                       *}
{*   Copyright (C) 2004 Marek Mauder, Vampyre SoftWorks                  *}
{*   Engine\Kernel\Platforms\Win32\Win32Registry                         *}
{*************************************************************************}
{                                                                         }
{ This program is free software; you can redistribute it and/or modify it }
{ under the terms of the GNU General Public License as published by the   }
{ Free Software Foundation; either version 2 of the License, or (at your  }
{ option) any later version. This program is distributed in the hope that }
{ it will be useful, but WITHOUT ANY WARRANTY; without even the implied   }
{ warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See   }
{ the GNU General Public License for more details.                        }
{                                                                         }
{ You should have received a copy of the GNU General Public License       }
{ along with this program; if not, write to the Free Software Foundation, }
{ Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.         }
{                                                                         }
{*************************************************************************}
{  Created On: 30th January 2004  (30.01.2004)                            }
{*************************************************************************}

{This unit contains functions to access Windows registry without
 using standard Registry.pas unit.}

unit Win32Registry;

{$I Compiler.inc}

interface

uses
  SysUtils, Kernel, Windows;

{Creates and initializes new registry key. Returns True if succesful.}
function RegCreateKey(const RootKey: HKEY; const Key, Value: NString): Boolean;
{Deletes entry Name from registry key named Key. Returns True if successful.}
function RegDeleteEntry(const RootKey: HKEY; const Key, Name: NString): Boolean;
{Deletes Key and all of its subkeys. Returns True if successful.}
function RegDeleteKeyTree(const RootKey: HKEY; const Key: NString): Boolean;
{Returns True if given Key exists in registry.}
function RegKeyExists(const RootKey: HKEY; const Key: NString): Boolean;

{Reads integer value named Name from registry key named Key. If reading fails
 then Def is returned.}
function RegReadInt(const RootKey: HKEY; const Key, Name: NString;
  Def: Long = 0): Long;
{Reads NString value named Name from registry key named Key. If reading fails
 then Def is returned.}
function RegReadStr(const RootKey: HKEY; const Key, Name: NString;
  const Def: NString = ''): NString;
{Reads binary data from registry. Returns number of bytes realy read
 from registry.}
function RegReadBinary(const RootKey: HKEY; const Key, Name: NString; var Value;
  Size: Long): Long;

{Writes integer value named Name to registry key named Key. Returns True if successsful.}
function RegWriteInt(const RootKey: HKEY; const Key, Name: NString; Value: Long):
  Boolean;
{Writes NString value named Name to registry key named Key. Returns True if successsful.}
function RegWriteStr(const RootKey: HKEY; const Key, Name, Value: NString):
  Boolean;
{Wrires binary data to registry. Returns True if successful.}
function RegWriteBinary(const RootKey: HKEY; const Key, Name: NString; var Value;
  Size: Long): Boolean;


{*************************************************************************}
{                             IMPLEMEMTATION                              }
{*************************************************************************}


implementation

function RelativeKey(const Key: NString): PChar;
begin
  Result := PChar(Key);
  if (Key <> '') and (Key[1] = '\') then
    Inc(Result);
end;

function RegCreateKey(const RootKey: HKEY; const Key, Value: NString): Boolean;
begin
  Result := RegSetValue(RootKey, RelativeKey(Key), REG_SZ, PChar(Value),
    Length(Value)) = ERROR_SUCCESS;
end;

function RegDeleteEntry(const RootKey: HKEY; const Key, Name: NString): Boolean;
var
  RegKey: HKEY;
begin
  Result := False;
  if RegOpenKeyEx(RootKey, RelativeKey(Key), 0, KEY_SET_VALUE, RegKey) =
    ERROR_SUCCESS then
  begin
    Result := RegDeleteValue(RegKey, PChar(Name)) = ERROR_SUCCESS;
    RegCloseKey(RegKey);
  end;
end;

function RegDeleteKeyTree(const RootKey: HKEY; const Key: NString): Boolean;
var
  RegKey: HKEY;
  I: DWord;
  Size: DWord;
  NumSubKeys: DWord;
  MaxSubKeyLen: DWord;
  KeyName: NString;
begin
  Result := RegOpenKeyEx(RootKey, RelativeKey(Key), 0, KEY_ALL_ACCESS, RegKey) =
    ERROR_SUCCESS;
  if Result then
  begin
    RegQueryInfoKey(RegKey, nil, nil, nil, @NumSubKeys, @MaxSubKeyLen, nil, nil,
      nil, nil, nil, nil);
    if NumSubKeys <> 0 then
      for I := NumSubKeys - 1 downto 0 do
      begin
        Size := MaxSubKeyLen + 1;
        SetLength(KeyName, Size);
        RegEnumKeyEx(RegKey, I, PChar(KeyName), Size, nil, nil, nil, nil);
        SetLength(KeyName, StrLen(PChar(KeyName)));
        Result := RegDeleteKeyTree(RootKey, Key + '\' + KeyName);
        if not Result then Break;
      end;
    RegCloseKey(RegKey);
    if Result then
      Result := Windows.RegDeleteKey(RootKey, RelativeKey(Key)) = ERROR_SUCCESS;
  end;
end;

function RegKeyExists(const RootKey: HKEY; const Key: NString): Boolean;
var
  RegKey: HKEY;
begin
  Result := (RegOpenKeyEx(RootKey, RelativeKey(Key), 0, KEY_READ, RegKey) =
    ERROR_SUCCESS);
  if Result then RegCloseKey(RegKey);
end;

function RegReadInt(const RootKey: HKEY; const Key, Name: NString; Def: Long):
  Long;
var
  RegKey: HKEY;
  Size: DWord;
  IntVal: Long;
  RegKind: DWord;
begin
  Result := Def;
  if RegOpenKeyEx(RootKey, RelativeKey(Key), 0, KEY_READ, RegKey) =
    ERROR_SUCCESS then
  begin
    RegKind := 0;
    Size := SizeOf(Long);
    if RegQueryValueEx(RegKey, PChar(Name), nil, @RegKind, @IntVal, @Size) =
      ERROR_SUCCESS then
      if RegKind = REG_DWORD then Result := IntVal;
    RegCloseKey(RegKey);
  end;
end;

function RegReadStr(const RootKey: HKEY; const Key, Name, Def: NString):
  NString;
var
  RegKey: HKEY;
  Size: DWord;
  StrVal: NString;
  RegKind: DWord;
begin
  Result := Def;
  if RegOpenKeyEx(RootKey, RelativeKey(Key), 0, KEY_READ, RegKey) =
    ERROR_SUCCESS then
  begin
    RegKind := 0;
    Size := 0;
    if RegQueryValueEx(RegKey, PChar(Name), nil, @RegKind, nil, @Size) =
      ERROR_SUCCESS then
      if RegKind in [REG_SZ, REG_EXPAND_SZ] then
      begin
        SetLength(StrVal, Size);
        if RegQueryValueEx(RegKey, PChar(Name), nil, @RegKind, PByte(StrVal),
          @Size) = ERROR_SUCCESS then
        begin
          SetLength(StrVal, StrLen(PChar(StrVal)));
          Result := StrVal;
        end;
      end;
    RegCloseKey(RegKey);
  end;
end;

function RegReadBinary(const RootKey: HKEY; const Key, Name: NString; var Value;
  Size: Long): Long;
var
  RegKey: HKEY;
  RSize: Long;
  RegKind: DWord;
  Ret: Long;
begin
  Result := 0;
  if RegOpenKeyEx(RootKey, RelativeKey(Key), 0, KEY_READ, RegKey) =
    ERROR_SUCCESS then
  begin
    RegKind := 0;
    RSize := 0;
    Ret := RegQueryValueEx(RegKey, PChar(Name), nil, @RegKind, nil, @RSize);
    if Ret = ERROR_SUCCESS then
      if RegKind in [REG_BINARY, REG_MULTI_SZ] then
      begin
        if RSize > Size then RSize := Size;
        RegQueryValueEx(RegKey, PChar(Name), nil, @RegKind, @Value, @RSize);
        Result := RSize;
      end;
    RegCloseKey(RegKey);
  end;
end;

function RegWriteInt(const RootKey: HKEY; const Key, Name: NString; Value: Long):
  Boolean;
var
  RegKey: HKEY;
  Ret: Long;
begin
  Result := False;
  if RegOpenKeyEx(RootKey, RelativeKey(Key), 0, KEY_SET_VALUE, RegKey) =
    ERROR_SUCCESS then
  begin
    Ret := RegSetValueEx(RegKey, PChar(Name), 0, REG_DWORD, @Value,
      SizeOf(Long));
    RegCloseKey(RegKey);
    Result := Ret = ERROR_SUCCESS;
  end;
end;

function RegWriteStr(const RootKey: HKEY; const Key, Name, Value: NString):
  Boolean;
var
  RegKey: HKEY;
  Ret: Long;
begin
  Result := False;
  if RegOpenKeyEx(RootKey, RelativeKey(Key), 0, KEY_SET_VALUE, RegKey) =
    ERROR_SUCCESS then
  begin
    Ret := RegSetValueEx(RegKey, PChar(Name), 0, REG_SZ, PByte(Value),
      Length(Value) + 1);
    RegCloseKey(RegKey);
    Result := Ret = ERROR_SUCCESS;
  end;
end;

function RegWriteBinary(const RootKey: HKEY; const Key, Name: NString; var Value;
  Size: Long):Boolean;
var
  RegKey: HKEY;
  Ret: Long;
begin
  Result := False;
  if RegOpenKeyEx(RootKey, RelativeKey(Key), 0, KEY_SET_VALUE, RegKey) =
    ERROR_SUCCESS then
  begin
    Ret := RegSetValueEx(RegKey, PChar(Name), 0, REG_BINARY, @Value, Size);
    RegCloseKey(RegKey);
    Result := Ret = ERROR_SUCCESS;
  end;
end;

end.

