{*************************************************************************}
{*   Win32SysInfo                                                        *}
{*   Copyright (C) 2003 Marek Mauder, Vampyre SoftWorks                  *}
{*   Engine\Kernel\Platforms\Win32\Win32SysInfo                          *}
{*************************************************************************}
{                                                                         }
{ This program is free software; you can redistribute it and/or modify it }
{ under the terms of the GNU General Public License as published by the   }
{ Free Software Foundation; either version 2 of the License, or (at your  }
{ option) any later version. This program is distributed in the hope that }
{ it will be useful, but WITHOUT ANY WARRANTY; without even the implied   }
{ warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See   }
{ the GNU General Public License for more details.                        }
{                                                                         }
{ You should have received a copy of the GNU General Public License       }
{ along with this program; if not, write to the Free Software Foundation, }
{ Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.         }
{                                                                         }
{*************************************************************************}
{  Created On: 12th August 2003  (12.08.2003)                             }
{*************************************************************************}

{Contains function for retrieve CPU, memory and other system info under
 Win32.}

unit Win32SysInfo;

{$I Compiler.inc}

interface

uses
 Kernel, Windows, SysUtils, Platforms, X86CpuInfo, Win32Registry;

procedure GetCPUInfo(var Info: TCPUInfo);
procedure GetOSInfo(var Version: TVersionInfo; var Name: NString);
procedure GetMemoryInfo(var Info: TMemoryInfo);


{*************************************************************************}
{                             Implementation                              }
{*************************************************************************}


implementation

const

  VER_NT_WORKSTATION = $0000001;
  VER_NT_DOMAIN_CONTROLLER = $0000002;
  VER_NT_SERVER = $0000003;

  VER_SUITE_SMALLBUSINESS = $00000001;
  VER_SUITE_ENTERPRISE = $00000002;
  VER_SUITE_BACKOFFICE = $00000004;
  VER_SUITE_COMMUNICATIONS = $00000008;
  VER_SUITE_TERMINAL = $00000010;
  VER_SUITE_SMALLBUSINESS_RESTRICTED = $00000020;
  VER_SUITE_EMBEDDEDNT = $00000040;
  VER_SUITE_DATACENTER = $00000080;
  VER_SUITE_SINGLEUSERTS = $00000100;
  VER_SUITE_PERSONAL = $00000200;
  VER_SUITE_SERVERAPPLIANCE = $00000400;

type

  POSVersionInfoEx = ^TOSVersionInfoEx;
  TOSVersionInfoEx = record
    dwOSVersionInfoSize: DWORD;
    dwMajorVersion: DWORD;
    dwMinorVersion: DWORD;
    dwBuildNumber: DWORD;
    dwPlatformId: DWORD;
    szCSDVersion: array[0..127] of Char;
    wServicePackMajor: Word;
    wServicePackMinor: Word;
    wSuiteMask: Word;
    wProductType: Byte;
    wReserved: Byte;
  end;

function GetVersionEx(VI: POSVersionInfoEx): BOOL; stdcall;
  external kernel32 name 'GetVersionExA';

function Win32PITFunction: Int64;
begin
  QueryPerformanceCounter(Result);
end;

procedure GetCPUInfo(var Info: TCpuInfo);
var
  Key, Name: NString;
  PITRate: Int64;
  Freq: Long;
  SystemInfo: TSystemInfo;
begin
  Freq := 0;
  Name := '';
  Key := 'Hardware\Description\System\CentralProcessor\0';
  if RegKeyExists(HKEY_LOCAL_MACHINE, Key) then
  try
    Freq := RegReadInt(HKEY_LOCAL_MACHINE, Key, '~MHz');
    // AMD and P4
    Name := Trim(RegReadStr(HKEY_LOCAL_MACHINE, Key, 'ProcessorNameString'));
  except
    // show no exceptions !!
  end;
  QueryPerformanceFrequency(PITRate);
  X86CpuInfo.GetX86CPUInfo(Info, PITRate, Win32PITFunction);
  if Freq > 0 then
    Info.Frequency := Freq;
  if Name <> '' then
    Info.Name := Name;
  ZeroMemory(@SystemInfo, SizeOf(SystemInfo));
  GetSystemInfo(SystemInfo);
  Info.ProcessorCount := SystemInfo.dwNumberOfProcessors;
end;

procedure GetOSInfo(var Version: TVersionInfo; var Name: NString);
var
  OS: TOSVersionInfoEx;
begin
  ZeroMemory(@OS, SizeOf(OS));
  OS.dwOSVersionInfoSize := SizeOf(OS);
  GetVersionEx(@OS);
  Version.Major := OS.dwMajorVersion;
  Version.Minor := OS.dwMinorVersion;
  Version.Build := OS.dwBuildNumber;

  Name := 'MS Windows';
  case OS.dwPlatformId of
    VER_PLATFORM_WIN32_WINDOWS:
      case Version.Minor of
        0..9:
          if Trim(OS.szCSDVersion) = 'B' then
            Name := 'MS Windows 95 OSR2'
          else
            Name := 'MS Windows 95';
        10..89:
          if Version.Build <> 2222 then
            Name := 'MS Windows 98'
          else
            Name := 'MS Windows 98 SE';
        90: Name := 'MS Windows Me';
      end;
    VER_PLATFORM_WIN32_NT:
      case Version.Major of
        3: case Version.Minor of
            1: Name := 'MS Windows NT 3.1';
            5: Name := 'MS Windows NT 3.5';
            51: Name := 'MS Windows NT 3.51';
          end;
        4: Name := 'MS Windows NT 4.0';
        5: case Version.Minor of
            0:
              begin
                Name := 'MS Windows 2000';
                if (OS.wProductType = VER_NT_SERVER) then
                begin
                  if (OS.wSuiteMask and VER_SUITE_DATACENTER) =
                    VER_SUITE_DATACENTER then
                    Name := Name + ' Data Center'
                  else
                    if (OS.wSuiteMask and VER_SUITE_ENTERPRISE) =
                      VER_SUITE_ENTERPRISE then
                      Name := Name + ' Advanced Server'
                    else
                      Name := Name + ' Server';
                end
                else
                  if (OS.wProductType = VER_NT_WORKSTATION) then
                    Name := Name + ' Professional';
              end;
            1:
              begin
                Name := 'MS Windows XP';
                if (OS.wProductType = VER_NT_WORKSTATION) then
                begin
                  if (OS.wSuiteMask and VER_SUITE_PERSONAL) = VER_SUITE_PERSONAL
                    then
                    Name := Name + ' Personal'
                  else
                    Name := Name + ' Professional';
                end;
              end;
            2: Name := 'MS Windows Server 2003';
          end;
      end;
  end;
end;

procedure GetMemoryInfo(var Info: TMemoryInfo);
var
  MS: TMemoryStatus;
begin
  MS.dwLength := SizeOf(MS);
  GlobalMemoryStatus(MS);
  Info.CurrentLoad := MS.dwMemoryLoad;
  Info.PhysicalTotal := MS.dwTotalPhys div 1024 + 640;
  Info.PhysicalFree := MS.dwAvailPhys div 1024;
  Info.PageFileTotal := MS.dwTotalPageFile div 1024;
  Info.PageFileFree := MS.dwAvailPageFile div 1024;
  Info.VirtualTotal := MS.dwTotalVirtual div 1024;
  Info.VirtualFree := MS.dwAvailVirtual div 1024;
end;
 
end.

