{*************************************************************************}
{*   BsaFormat                                                           *}
{*   Copyright (C) 2003 Marek Mauder, Vampyre SoftWorks                  *}
{*   Modules\Archives\Bsa\BsaFormat.pas                                  *}
{*************************************************************************}
{                                                                         }
{ This program is free software; you can redistribute it and/or modify it }
{ under the terms of the GNU General Public License as published by the   }
{ Free Software Foundation; either version 2 of the License, or (at your  }
{ option) any later version. This program is distributed in the hope that }
{ it will be useful, but WITHOUT ANY WARRANTY; without even the implied   }
{ warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See   }
{ the GNU General Public License for more details.                        }
{                                                                         }
{ You should have received a copy of the GNU General Public License       }
{ along with this program; if not, write to the Free Software Foundation, }
{ Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.         }
{                                                                         }
{*************************************************************************}
{  Created On: 7th November 2003  (07.11.2003)                            }
{*************************************************************************}

{This unit containts archiver that can read Bethesda's bsa archives.}

unit BsaFormat;

{$I Compiler.inc}

interface

uses
  Kernel, Objects, Platforms, Files, BsaArchive;

type

  {This archiver can read Bethesda's bsa archives. But only older version
   (Arena, Terminator, Daggerfall) -  no Morrowind.}
  TNBsaFormat = class(TNArchiveFormat)
  protected
    function DoReadFile(Archive: TNArchive; const FileName: NString; var Data:
      Pointer; var Size: Long): Boolean; override;
    function DoOpen(const FileName: NString): TNArchive; override;
    procedure DoClose(Archive: TNArchive); override;
    procedure DoFillFileList(Archive: TNArchive); override;
  public
    constructor Create; override;
    function TestFormat(Stream: TNStream): Boolean; override;
  end;

const

  sBsaExtensions = 'bsa';
  sBsaFormatName = 'Bsa Bethesda Archive';
  BsaCaps: TArchiveCaps = [acReadFile];

var

  BsaInstance: TNBsaFormat = nil;


{*************************************************************************}
{                             IMPLEMEMTATION                              }
{*************************************************************************}


implementation


{*************************************************************************}
{                        TNBsaFormat methods                              }
{*************************************************************************}


constructor TNBsaFormat.Create;
begin
  inherited Create;
  FExtensions := sBsaExtensions;
  FFormatName := sBsaFormatName;
  FCaps := BsaCaps;
end;

function TNBsaFormat.DoReadFile(Archive: TNArchive; const FileName: NString; var
  Data: Pointer; var Size: Long): Boolean;
begin
  TNBsaArchive(Archive.Handle).ExtractFile(FileName, Data, Size);
  Result := Size > 0;
end;

function TNBsaFormat.DoOpen(const FileName: NString): TNArchive;
var
  Arc: TNBsaArchive;
begin
  Arc := TNBsaArchive.Create(FileName);
  Result := TNArchive.Create(Self, Arc);
end;

procedure TNBsaFormat.DoClose(Archive: TNArchive);
begin
  TNBsaArchive(Archive.Handle).Free;
end;

procedure TNBsaFormat.DoFillFileList(Archive: TNArchive);
begin
  TNBsaArchive(Archive.Handle).GetFileList(Archive.Files);
end;

function TNBsaFormat.TestFormat(Stream: TNStream): Boolean;
begin
  // There is no quick way how to check format of bsa from TES: Arena.
  Result := True;
end;


{*************************************************************************}
{                         Internal Stuff                                  }
{*************************************************************************}


initialization

  BsaInstance := TNBsaFormat.Create;

finalization

  FreeAndNil(BsaInstance);

end.

