{*************************************************************************}
{*   BZip2Format                                                         *}
{*   Copyright (C) 2003 Marek Mauder, Vampyre SoftWorks                  *}
{*   Modules\Compressions\BZip2\BZip2Format.pas                          *}
{*************************************************************************}
{                                                                         }
{ This program is free software; you can redistribute it and/or modify it }
{ under the terms of the GNU General Public License as published by the   }
{ Free Software Foundation; either version 2 of the License, or (at your  }
{ option) any later version. This program is distributed in the hope that }
{ it will be useful, but WITHOUT ANY WARRANTY; without even the implied   }
{ warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See   }
{ the GNU General Public License for more details.                        }
{                                                                         }
{ You should have received a copy of the GNU General Public License       }
{ along with this program; if not, write to the Free Software Foundation, }
{ Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.         }
{                                                                         }
{*************************************************************************}
{  Created On: 7th November 2003  (07.11.2003)                            }
{*************************************************************************}

{This unit contains class for compress/decompress data using very efficient
 but slow BZip2 algorithm.}

unit BZip2Format;

{$I Compiler.inc}

interface

uses
  Kernel, Objects, Platforms, Files, BZip2;

type

  {This compression format provides BZip2 compression/decompression.}
  TNBZip2Format = class(TNCompression)
  protected
    procedure DoCompress(InData: Pointer; InSize: Long; var OutData: Pointer; var
      OutSize: Long);override;
    procedure DoDecompress(InData: Pointer; InSize: Long; var OutData: Pointer;
      var OutSize: Long);override;
  public
    constructor Create; override;
  end;

const

  sBZip2Extensions = 'bz2,bzip,tbz2';
  sBZip2FormatName = 'BZip2 compression/decompression';

var

  BZip2Instance: TNBZip2Format = nil;


{*************************************************************************}
{                             IMPLEMEMTATION                              }
{*************************************************************************}


implementation


{*************************************************************************}
{                        TNBZip2Format methods                            }
{*************************************************************************}


constructor TNBZip2Format.Create;
begin
  inherited Create;
  FExtensions := sBZip2Extensions;
  FFormatName := sBZip2FormatName;
  FCanLoad := True;
  FCanSave := True;
end;

procedure TNBZip2Format.DoCompress(InData: Pointer; InSize: Long;
  var OutData: Pointer; var OutSize: Long);
begin
  BZip2.CompressBuffer(InData, InSize, OutData, OutSize);
end;

procedure TNBZip2Format.DoDecompress(InData: Pointer; InSize: Long;
  var OutData: Pointer; var OutSize: Long);
begin
  BZip2.DecompressBuffer(InData, InSize, OutData, OutSize);
end;


{*************************************************************************}
{                         Internal Stuff                                  }
{*************************************************************************}


initialization

  BZip2Instance := TNBZip2Format.Create;

finalization

  FreeAndNil(BZip2Instance);

end.

