{*************************************************************************}
{*   LzhFormat                                                           *}
{*   Copyright (C) 2003 Marek Mauder, Vampyre SoftWorks                  *}
{*   Modules\Files\Compressions\Lzh\LzhFormat.pas                        *}
{*************************************************************************}
{                                                                         }
{ This program is free software; you can redistribute it and/or modify it }
{ under the terms of the GNU General Public License as published by the   }
{ Free Software Foundation; either version 2 of the License, or (at your  }
{ option) any later version. This program is distributed in the hope that }
{ it will be useful, but WITHOUT ANY WARRANTY; without even the implied   }
{ warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See   }
{ the GNU General Public License for more details.                        }
{                                                                         }
{ You should have received a copy of the GNU General Public License       }
{ along with this program; if not, write to the Free Software Foundation, }
{ Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.         }
{                                                                         }
{*************************************************************************}
{  Created On: 10th November 2003  (10.11.2003)                           }
{*************************************************************************}

{This unit contains class for compress/decompress data using Lzh algorithm.}

unit LzhFormat;

{$I Compiler.inc}

interface

uses
  Kernel, Objects, Platforms, Files, Lzh;

type

  {This compression format provides Lzh compression/decompression.}
  TNLzhFormat = class(TNCompression)
  protected
    {Some vars used by callbacks.}
    FInput: Pointer;
    FOutput: Pointer;
    FInSize: Long;
    FOutSize: Long;
    FInPos: Long;
    FOutPos: Long;

    {Callbacks for Lzh procedures.}
    procedure Write(var Data; NBytes: Word; var BytesPut: Word);
    procedure Read(var Data; NBytes: Word; var BytesGot: Word);

    procedure DoCompress(InData: Pointer; InSize: Long; var OutData: Pointer; var
      OutSize: Long); override;
    procedure DoDecompress(InData: Pointer; InSize: Long; var OutData: Pointer;
      var OutSize: Long); override;
  public
    constructor Create; override;
  end;

const

  sLzhExtensions = 'lzh';
  sLzhFormatName = 'Lzh compression/decompression';

var

  LzhInstance: TNLzhFormat = nil;


{*************************************************************************}
{                             IMPLEMEMTATION                              }
{*************************************************************************}


implementation


{*************************************************************************}
{                        TNLzhFormat methods                              }
{*************************************************************************}


constructor TNLzhFormat.Create;
begin
  inherited Create;
  FExtensions := sLzhExtensions;
  FFormatName := sLzhFormatName;
  FCanLoad := True;
  FCanSave := True;
end;

procedure TNLzhFormat.Read(var Data; NBytes: Word; var BytesGot: Word);
begin
  if FInPos + NBytes > FInSize then
    BytesGot := FInSize - FInPos
  else
    BytesGot := NBytes;
  if BytesGot > 1 then
    Move(Pointer(Long(FInput) + FInPos)^, Data, BytesGot)
  else
    Byte(Data) := PByte(Long(FInput) + FInPos)^;
  Inc(FInPos, BytesGot);
end;

procedure TNLzhFormat.Write(var Data; NBytes: Word; var BytesPut: Word);
begin
  if FOutPos + NBytes > FOutSize then
  begin
    FOutSize := FOutSize + FInSize div 4;
    ReallocMem(FOutput, FOutSize);
  end;
  BytesPut := NBytes;
  if BytesPut > 1 then
    Move(Data, Pointer(Long(FOutput) + FOutPos)^, BytesPut)
  else
    PByte(Long(FOutput) + FOutPos)^ := Byte(Data);
  Inc(FOutPos, BytesPut);
end;

procedure TNLzhFormat.DoCompress(InData: Pointer; InSize: Long;
  var OutData: Pointer; var OutSize: Long);
var
  I: Word;
begin
  FInSize := InSize;
  FOutSize := InSize div 2;
  FInput := InData;
  GetMem(FOutput, FOutSize);
  FInPos := 0;
  FOutPos := 0;
  Write(InSize, SizeOf(InSize), I);
  LZHPack(InSize, Read, Write);
  Write(I, 0, I); // flush buffer
  OutSize := FOutPos;
  OutData := FOutput;
  ReallocMem(OutData, OutSize);
end;

procedure TNLzhFormat.DoDecompress(InData: Pointer; InSize: Long;
  var OutData: Pointer; var OutSize: Long);
var
  I: Word;
begin
  OutSize := InSize;
  FInSize := InSize;
  FOutSize := InSize;
  FInput := InData;
  FInPos := 0;
  FOutPos := 0;
  Read(FOutSize, SizeOf(FOutSize), I);
  GetMem(FOutput, FOutSize);
  LZHUnpack(FOutSize, Read, Write);
  Write(I, 0, I); // flush buffer
  OutSize := FOutPos;
  OutData := FOutPut;
  ReallocMem(OutData, OutSize);
end;


{*************************************************************************}
{                         Internal Stuff                                  }
{*************************************************************************}


initialization

  LzhInstance := TNLzhFormat.Create;

finalization

  FreeAndNil(LzhInstance);

end.

