{*************************************************************************}
{*   LzoFormat                                                           *}
{*   Copyright (C) 2003 Marek Mauder, Vampyre SoftWorks                  *}
{*   Modules\Files\Compressions\Lzo\LzoFormat.pas                        *}
{*************************************************************************}
{                                                                         }
{ This program is free software; you can redistribute it and/or modify it }
{ under the terms of the GNU General Public License as published by the   }
{ Free Software Foundation; either version 2 of the License, or (at your  }
{ option) any later version. This program is distributed in the hope that }
{ it will be useful, but WITHOUT ANY WARRANTY; without even the implied   }
{ warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See   }
{ the GNU General Public License for more details.                        }
{                                                                         }
{ You should have received a copy of the GNU General Public License       }
{ along with this program; if not, write to the Free Software Foundation, }
{ Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.         }
{                                                                         }
{*************************************************************************}
{  Created On: 7th January 2004  (07.01.2004)                             }
{*************************************************************************}

{This unit contains class for compress/decompress data
 MiniLZO compression library. Compress ration is not the best but
 it is extremely fast.}

unit LzoFormat;

{$I Compiler.inc}

interface

uses
  Kernel, Objects, Platforms, Files, MiniLZO;

type

  {This compression format provides Lzo compression/decompression.}
  TNLzoFormat = class(TNCompression)
  protected
    procedure DoCompress(InData: Pointer; InSize: Long; var OutData: Pointer; var
      OutSize: Long); override;
    procedure DoDecompress(InData: Pointer; InSize: Long; var OutData: Pointer;
      var OutSize: Long); override;
  public
    constructor Create; override;
  end;

const

  sLzoExtensions = 'lzo';
  sLzoFormatName = 'Lzo compression/decompression';

var

  LzoInstance: TNLzoFormat = nil;


{*************************************************************************}
{                             IMPLEMEMTATION                              }
{*************************************************************************}


implementation


{*************************************************************************}
{                        TNLzoFormat methods                              }
{*************************************************************************}


constructor TNLzoFormat.Create;
begin
  inherited Create;
  FExtensions := sLzoExtensions;
  FFormatName := sLzoFormatName;
  FCanLoad := True;
  FCanSave := True;
end;

procedure TNLzoFormat.DoCompress(InData: Pointer; InSize: Long;
  var OutData: Pointer; var OutSize: Long);
begin
  MiniLZO.CompressBuffer(InData,InSize,OutData,OutSize);
end;

procedure TNLzoFormat.DoDecompress(InData: Pointer; InSize: Long;
  var OutData: Pointer; var OutSize: Long);
begin
  MiniLZO.DecompressBuffer(InData,InSize,OutData,OutSize);
end;


{*************************************************************************}
{                         Internal Stuff                                  }
{*************************************************************************}


initialization

  LzoInstance := TNLzoFormat.Create;

finalization

  FreeAndNil(LzoInstance);

end.
