{*************************************************************************}
{*   DaggerGraphic                                                       *}
{*   Copyright (C) 2003 Marek Mauder, Vampyre SoftWorks                  *}
{*   Modules\ImageFileFormats\Daggerfall\DaggerGraphic.pas               *}
{*************************************************************************}
{                                                                         }
{ This program is free software; you can redistribute it and/or modify it }
{ under the terms of the GNU General Public License as published by the   }
{ Free Software Foundation; either version 2 of the License, or (at your  }
{ option) any later version. This program is distributed in the hope that }
{ it will be useful, but WITHOUT ANY WARRANTY; without even the implied   }
{ warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See   }
{ the GNU General Public License for more details.                        }
{                                                                         }
{ You should have received a copy of the GNU General Public License       }
{ along with this program; if not, write to the Free Software Foundation, }
{ Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.         }
{                                                                         }
{*************************************************************************}
{  Created On: 8th November 2003  (08.11.2003)                            }
{*************************************************************************}

{This unit contains types used in loading of Daggerfall graphic data, default
 palette and abstract file module for daggerfall graphic formats.}

unit DaggerGraphic;

{$I Compiler.inc}

{TODO: Resize images when saving to ImageSize fit in Word in ConvertToDagg.}

interface

uses
  Kernel, Objects, Platforms, Graphic, Images;

type

  {Abstract class for all Daggerfall modules.
   They require loaded palette (*.pal;*.col in Dag\Arena2 directory).
   Now default palette pal.pal is contained in this unit - it works
   well for 99% of Daggerfall graphics.
   They do not support continuous streams - exact size must be known
   before loading (because some files are without headers).}
  TNDaggerFormat = class(TNImageFileFormat)
  protected
    FPalette: TPalette;
    procedure SetPalette(const Value: TPalette);
    procedure PrepareImage(const Image: TImageData; var Clone: TImageData);
      override;
  public
    constructor Create; override;
    function TestFormat(Stream: TNStream): Boolean; override;
    {Converts image to 8bit indexed using Daggerfall loaded (or default) palette.}
    procedure ConvertToDagg(var Image: TImageData);
    {Loads palette and sets active pal to this value.
    Size must be total size of palette in bytes.}
    function LoadPal(Data: Pointer; Size: Long): Boolean;
    {Actual palette used when loading/saving images.}
    property Palette: TPalette read FPalette write SetPalette;
  end;

  {(*}

  {Header for Img files}
  TImgHeader = packed record
    XOff: Word;
    YOff: Word;
    Width: Word;
    Height: Word;
    Unk: Word;             // Might indicate compressed data or not
    ImageSize: Word;       // Size of Image data (but not always)
  end;


  {Header for Cif Group files}
  TCifGroup = packed record
    Width: Word;
    Height: Word;
    XOff: Word;
    YOff: Word;
    Unk: Word;
    ImageSize: Word;       // Size of Image data (but not always)
    Offsets: array[0..31] of Word; // Offsets from beginning of header to
                                   //  image datas. Last offset points to next
                                   //  group header
  end;


  {Main header for texture}
  TTexHeader = packed record
    ImgCount: Word;        // Number of images in texture
    TexName: array[0..23] of Char; // Name of texture
  end;


  {Offset list  for texture}
  TOffset = packed record
    Type1: Word;           // ??
    HdrOffset: ULong;      // Contains offsetof Img header from the origin
                           //  of the file
    Type2: Word;           // ??
    Unk: ULong;            // Ranges from 0 to 4 (0 in 90%)
    Null1: ULong;          // Always 0
    Null2: ULong;          // Always 0
  end;


  TOffsetList = array[Word] of TOffset;
  POffsetList = ^TOffsetList;


  {Image header for texture}
  TTexImgHeader = packed record
    XOff: Word;
    YOff: Word;
    Width: Word;
    Height: Word;
    Unk1: Word;            // $0108 = Image has subimages which are RLE
                           //  compressed data.
                           //	$1108 = Image has RLE type compressed data with
                           //  a row offset section before the single image data.
    ImageSize: Long;       // Image size (including header)
    ImageOff: Long;        // Pointer to start of image data from this header
    Unk2: Word;            // $0000 = Image has subimages in special
                           //  compressed format.
                           // $00C0 = Usual value, regular single image.
                           // NonZero = Regular single image.Unknown what the
                           //  differences indicate
    SubImages: Word;       // Number of subimages (1 = single image)
    Unk3: Long;
    Unk4: Word;
  end;

  {*)}

{Decodes RLE used in IMG and CIF files. OutSize must be known before calling!
 (use Out = Width * Height)}
procedure DagRLEDecode(InData: Pointer; InSize, OutSize: Long; out OutData:
  Pointer);
{Deletes non-valid chars from texture name.}
function RepairName(const S: NString): NString;

const

  {(*}

  {This is default Daggerfall's palette (C:\Dagger\Arena2\pal.pal).
   Every Daggerfall module loads this pal in constructor.}
  DaggerfallPal: TPalRGB = (
    (B: 0; G: 0; R: 0), (B: 255; G: 0; R: 255), (B: 255; G: 0; R: 255),
    (B: 255; G: 0; R: 255),(B: 255; G: 0; R: 255), (B: 255; G: 0; R: 255),
    (B: 255; G: 0; R: 255),(B: 255; G: 0; R: 255), (B: 255; G: 0; R: 255),
    (B: 255; G: 0; R: 255), (B: 255; G: 0; R: 255), (B: 255; G: 0; R: 255),
    (B: 255; G: 0; R: 255), (B: 255; G: 0; R: 255), (B: 255; G: 0; R: 255),
    (B: 255; G: 0; R: 255), (B: 255; G: 0; R: 255), (B: 255; G: 0; R: 255),
    (B: 255; G: 0; R: 255), (B: 255; G: 0; R: 255), (B: 255; G: 0; R: 255),
    (B: 255; G: 0; R: 255), (B: 255; G: 0; R: 255), (B: 255; G: 0; R: 255),
    (B: 255; G: 0; R: 255), (B: 255; G: 0; R: 255), (B: 255; G: 0; R: 255),
    (B: 255; G: 0; R: 255), (B: 255; G: 0; R: 255), (B: 255; G: 0; R: 255),
    (B: 255; G: 0; R: 255), (B: 255; G: 0; R: 255), (B: 244; G: 202; R: 167),
    (B: 227; G: 180; R: 144), (B: 207; G: 152; R: 118), (B: 193; G: 133; R: 100),
    (B: 180; G: 113; R: 80), (B: 165; G: 100; R: 70), (B: 152; G: 93; R: 63),
    (B: 140; G: 86; R: 55), (B: 129; G: 79; R: 48), (B: 122; G: 75; R: 43),
    (B: 112; G: 70; R: 40), (B: 103; G: 64; R: 39), (B: 91; G: 67; R: 38),
    (B: 79; G: 63; R: 43), (B: 66; G: 54; R: 41), (B: 54; G: 50; R: 40),
    (B: 232; G: 196; R: 196), (B: 220; G: 177; R: 177), (B: 204; G: 157; R: 157),
    (B: 188; G: 138; R: 138), (B: 175; G: 122; R: 122), (B: 155; G: 105; R: 106),
    (B: 143; G: 94; R: 97), (B: 126; G: 81; R: 89), (B: 109; G: 72; R: 88),
    (B: 101; G: 68; R: 85), (B: 86; G: 61; R: 77), (B: 75; G: 55; R: 71),
    (B: 67; G: 51; R: 63), (B: 63; G: 47; R: 56), (B: 56; G: 45; R: 52),
    (B: 46; G: 44; R: 46), (B: 245; G: 212; R: 172), (B: 229; G: 193; R: 150),
    (B: 213; G: 174; R: 128), (B: 196; G: 154; R: 105), (B: 183; G: 140; R: 88),
    (B: 173; G: 127; R: 78), (B: 160; G: 118; R: 74), (B: 151; G: 110; R: 69),
    (B: 134; G: 103; R: 65), (B: 123; G: 92; R: 60), (B: 109; G: 85; R: 54),
    (B: 96; G: 76; R: 51), (B: 83; G: 71; R: 44), (B: 69; G: 63; R: 42),
    (B: 61; G: 54; R: 38), (B: 50; G: 45; R: 34), (B: 205; G: 205; R: 224),
    (B: 188; G: 188; R: 199), (B: 165; G: 165; R: 174), (B: 145; G: 145; R: 159),
    (B: 135; G: 135; R: 149), (B: 122; G: 122; R: 137), (B: 114; G: 114; R: 127),
    (B: 103; G: 103; R: 116), (B: 94; G: 94; R: 109), (B: 85; G: 85; R: 96),
    (B: 75; G: 75; R: 85), (B: 68; G: 68; R: 80), (B: 61; G: 61; R: 67),
    (B: 53; G: 53; R: 59), (B: 48; G: 48; R: 50), (B: 44; G: 44; R: 45),
    (B: 176; G: 205; R: 255), (B: 147; G: 185; R: 244), (B: 123; G: 164; R: 230),
    (B: 104; G: 152; R: 217), (B: 87; G: 137; R: 205), (B: 68; G: 124; R: 192),
    (B: 68; G: 112; R: 179), (B: 62; G: 105; R: 167), (B: 55; G: 97; R: 154),
    (B: 49; G: 90; R: 142), (B: 45; G: 82; R: 122), (B: 51; G: 77; R: 102),
    (B: 52; G: 69; R: 87), (B: 50; G: 62; R: 73), (B: 47; G: 59; R: 60),
    (B: 44; G: 48; R: 49), (B: 220; G: 220; R: 220), (B: 197; G: 197; R: 197),
    (B: 185; G: 185; R: 185), (B: 174; G: 174; R: 174), (B: 162; G: 162; R: 162),
    (B: 147; G: 147; R: 147), (B: 132; G: 132; R: 132), (B: 119; G: 119; R: 119),
    (B: 110; G: 110; R: 110), (B: 99; G: 99; R: 99), (B: 87; G: 87; R: 87),
    (B: 78; G: 78; R: 78), (B: 67; G: 67; R: 67), (B: 58; G: 58; R: 58),
    (B: 51; G: 51; R: 51), (B: 44; G: 44; R: 44), (B: 182; G: 218; R: 227),
    (B: 158; G: 202; R: 202), (B: 134; G: 187; R: 187), (B: 109; G: 170; R: 170),
    (B: 87; G: 154; R: 154), (B: 77; G: 142; R: 142), (B: 70; G: 135; R: 135),
    (B: 62; G: 124; R: 124), (B: 54; G: 112; R: 112), (B: 46; G: 103; R: 103),
    (B: 39; G: 91; R: 91), (B: 40; G: 83; R: 83), (B: 45; G: 72; R: 72),
    (B: 47; G: 63; R: 63), (B: 50; G: 55; R: 55), (B: 45; G: 48; R: 48),
    (B: 255; G: 246; R: 103), (B: 241; G: 238; R: 45), (B: 226; G: 220; R: 0),
    (B: 212; G: 203; R: 0), (B: 197; G: 185; R: 0), (B: 183; G: 168; R: 0),
    (B: 168; G: 150; R: 0), (B: 154; G: 133; R: 0), (B: 139; G: 115; R: 0),
    (B: 127; G: 106; R: 4), (B: 116; G: 97; R: 7), (B: 104; G: 87; R: 11),
    (B: 93; G: 78; R: 14), (B: 81; G: 69; R: 18), (B: 69; G: 60; R: 21),
    (B: 58; G: 51; R: 25), (B: 202; G: 221; R: 196), (B: 175; G: 200; R: 168),
    (B: 148; G: 176; R: 141), (B: 123; G: 156; R: 118), (B: 107; G: 144; R: 109),
    (B: 93; G: 130; R: 94), (B: 82; G: 116; R: 86), (B: 77; G: 110; R: 78),
    (B: 68; G: 99; R: 67), (B: 61; G: 89; R: 53), (B: 52; G: 77; R: 45),
    (B: 46; G: 68; R: 37), (B: 39; G: 60; R: 39), (B: 30; G: 55; R: 30),
    (B: 34; G: 51; R: 34), (B: 40; G: 47; R: 40), (B: 179; G: 107; R: 83),
    (B: 175; G: 95; R: 75), (B: 175; G: 87; R: 67), (B: 163; G: 79; R: 59),
    (B: 155; G: 75; R: 51), (B: 147; G: 71; R: 47), (B: 155; G: 91; R: 47),
    (B: 139; G: 83; R: 43), (B: 127; G: 75; R: 39), (B: 115; G: 67; R: 35),
    (B: 99; G: 63; R: 31), (B: 87; G: 55; R: 27), (B: 75; G: 47; R: 23),
    (B: 59; G: 39; R: 19), (B: 47; G: 31; R: 15), (B: 35; G: 23; R: 11),
    (B: 216; G: 227; R: 162), (B: 185; G: 205; R: 127), (B: 159; G: 183; R: 101),
    (B: 130; G: 162; R: 77), (B: 109; G: 146; R: 66), (B: 101; G: 137; R: 60),
    (B: 92; G: 127; R: 54), (B: 84; G: 118; R: 48), (B: 76; G: 108; R: 42),
    (B: 65; G: 98; R: 37), (B: 53; G: 87; R: 34), (B: 51; G: 75; R: 35),
    (B: 45; G: 64; R: 37), (B: 43; G: 56; R: 39), (B: 38; G: 51; R: 40),
    (B: 43; G: 46; R: 45), (B: 179; G: 115; R: 79), (B: 175; G: 111; R: 75),
    (B: 171; G: 107; R: 71), (B: 167; G: 103; R: 67), (B: 159; G: 99; R: 63),
    (B: 155; G: 95; R: 59), (B: 151; G: 91; R: 55), (B: 143; G: 87; R: 51),
    (B: 40; G: 40; R: 40), (B: 38; G: 38; R: 38), (B: 35; G: 35; R: 35),
    (B: 31; G: 31; R: 31), (B: 27; G: 27; R: 27), (B: 23; G: 23; R: 23),
    (B: 19; G: 19; R: 19), (B: 15; G: 15; R: 15), (B: 254; G: 255; R: 199),
    (B: 254; G: 245; R: 185), (B: 254; G: 235; R: 170), (B: 254; G: 225; R: 156),
    (B: 255; G: 215; R: 141), (B: 255; G: 205; R: 127), (B: 255; G: 195; R: 112),
    (B: 255; G: 185; R: 98), (B: 255; G: 175; R: 83), (B: 241; G: 167; R: 54),
    (B: 234; G: 155; R: 50), (B: 226; G: 143; R: 46), (B: 219; G: 131; R: 43),
    (B: 212; G: 119; R: 39), (B: 205; G: 107; R: 35), (B: 198; G: 95; R: 31),
    (B: 190; G: 84; R: 27), (B: 183; G: 72; R: 23), (B: 176; G: 60; R: 19),
    (B: 169; G: 48; R: 15), (B: 162; G: 36; R: 12), (B: 154; G: 24; R: 8),
    (B: 147; G: 12; R: 4), (B: 130; G: 22; R: 0), (B: 111; G: 34; R: 0),
    (B: 102; G: 33; R: 1), (B: 92; G: 33; R: 3), (B: 83; G: 32; R: 10),
    (B: 74; G: 39; R: 27), (B: 65; G: 41; R: 33), (B: 57; G: 43; R: 39),
    (B: 48; G: 45; R: 45));

    {*)}


{*************************************************************************}
{                             IMPLEMEMTATION                              }
{*************************************************************************}


implementation


procedure DagRLEDecode(InData: Pointer; InSize, OutSize: Long; out OutData:
  Pointer);
var
  I, Pos, CByte: Long;
  Rle, B: Byte;
begin
  GetMem(OutData, OutSize);
  Pos := 0;
  CByte := 0;
  while Pos < OutSize do
  begin
    Rle := PByte(Long(InData) + CByte)^;
    CByte := CByte + 1;
    if Rle < 128 then
    begin
      Rle := Rle + 1;
      Move(PByte(Long(InData) + CByte)^, PByte(Long(OutData) + Pos)^, Rle);
      CByte := CByte + Rle;
      Pos := Pos + Rle;
    end
    else
    begin
      Rle := Rle - 127;
      B := PByte(Long(InData) + CByte)^;
      CByte := CByte + 1;
      for I := 0 to Rle - 1 do
      begin
        PByte(Long(OutData) + Pos)^ := B;
        Pos := Pos + 1;
      end;
    end;
  end;
end;

function RepairName(const S: NString): NString;
var
  I: Long;
  First: boolean;
begin
  I := 1;
  Result := S;
  First := False;
  while I <= Length(Result) do
    if (Ord(Result[I]) < 32) or
      ((Ord(Result[I]) = 32) and (not First)) then
      Delete(Result, I, 1)
    else
    begin
      Inc(I);
      First := True;
    end;
end;


{*************************************************************************}
{                         TNDaggerFormat methods                          }
{*************************************************************************}


constructor TNDaggerFormat.Create;
begin
  inherited Create;
  FPalette := RGBPalToARGBPal(DaggerfallPal);
end;

procedure TNDaggerFormat.SetPalette(const Value: TPalette);
begin
  FPalette := Value;
end;

procedure TNDaggerFormat.PrepareImage(const Image: TImageData;
  var Clone: TImageData);
begin
  inherited PrepareImage(Image, Clone);
  ConvertToDagg(Clone);
end;

function TNDaggerFormat.TestFormat(Stream: TNStream): Boolean;
begin
  Result := True;
end;

procedure TNDaggerFormat.ConvertToDagg(var Image: TImageData);

  procedure ConvertLevel(var Level: TImageLevel);
  var
    I, X, Y: Long;
    Data: Pointer;
    B: PByte;
    C: PARGBRec;
  begin
    with Level do
    begin
      GetMem(Data, Width * Height);
      Size := Width * Height;
      B := Data;
      C := Bits;
      for X := 0 to Width - 1 do
        for Y := 0 to Height - 1 do
        begin
          B^ := ClosestColorInPal(Image.Palette, 256, C.Color);
          Inc(B);
          Inc(C);
        end;
      FreeMemNil(Bits);
      Bits := Data;
      for I := 0 to Level.Count - 1 do
        ConvertLevel(Levels[I]);
    end;
  end;

begin
  with Image do
  begin
    Image.DataFormat^.ConvertFunc(Image, FindImageDataFormat(idfA8R8G8B8));
    GetMem(Image.Palette, 256 * SizeOf(TARGB));
    Image.DataFormat := FindImageDataFormat(idfIndex8);
    Move(FPalette, Image.Palette^, 256 * SizeOf(TARGB));
    PalSwapChannels(Image.Palette, 256);
    ConvertLevel(Image.Levels[0]);
  end;
end;


function TNDaggerFormat.LoadPal(Data: Pointer; Size: Long): Boolean;
var
  Pal: TPalRGB;
  Pos: Long;
begin
  Result := False;
  if (Size = 768) or (Size = 776) then
  try
    Pos := 0;
    if Size = 776 then
      Pos := 8;
    Move(Pointer(Long(Data) + Pos)^, Pal, 256 * SizeOf(TRGBRec));
    FPalette := RGBPalToARGBPal(Pal, $FF);
    Result := True;
  except
    Result := False;
  end;
end;

end.

