{*************************************************************************}
{*   TexFormat                                                           *}
{*   Copyright (C) 2003 Marek Mauder, Vampyre SoftWorks                  *}
{*   Modules\ImageFileFormats\Daggerfall\TexFormat.pas                   *}
{*************************************************************************}
{                                                                         }
{ This program is free software; you can redistribute it and/or modify it }
{ under the terms of the GNU General Public License as published by the   }
{ Free Software Foundation; either version 2 of the License, or (at your  }
{ option) any later version. This program is distributed in the hope that }
{ it will be useful, but WITHOUT ANY WARRANTY; without even the implied   }
{ warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See   }
{ the GNU General Public License for more details.                        }
{                                                                         }
{ You should have received a copy of the GNU General Public License       }
{ along with this program; if not, write to the Free Software Foundation, }
{ Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.         }
{                                                                         }
{*************************************************************************}
{  Created On: 9th November 2003  (09.11.2003)                            }
{*************************************************************************}

{This unit contains image file format loader for Daggerfall Textures.}

unit TexFormat;

{$I Compiler.inc}

interface

uses
  Kernel, Objects, Platforms, Graphic, Images, DaggerGraphic;

type

  {This module proveides loading of Daggerfall textures.
  This format is very complicated ( more images with subimages,
  non-standard RLE, many unknowns) so module supports only loading.}
  TNTexFormat = class(TNDaggerFormat)
  protected
    FLastName: NString;
    procedure DoLoadImage(var Image: TImageData; Stream: TNStream); override;
  public
    constructor Create; override;
    {Stores name of last loaded texture.}
    property LastTextureName: NString read FLastName;
  end;

const

  sTexExtensions = 'texture';
  sTexFormatName = 'Daggerfall Texture';

var

  TexInstance: TNTexFormat = nil;


{*************************************************************************}
{                             IMPLEMEMTATION                              }
{*************************************************************************}


implementation


{*************************************************************************}
{                           TNTexFormat methods                           }
{*************************************************************************}


constructor TNTexFormat.Create;
begin
  inherited Create;
  FExtensions := sTexExtensions;
  FFormatName := sTexFormatName;
  FCanLoad := True;
  FCanSave := False;
end;

procedure TNTexFormat.DoLoadImage(var Image: TImageData; Stream: TNStream);
var
  Hdr: TTexHeader;
  List: POffsetList;
  IHdr: TTexImgHeader;
  COff, I, HPos: Long;
  Owner: PImageLevel;

  procedure LoadUncomp;
  var
    I: Long;
  begin
    AddImageLevel(Image);
    with Image.Levels[Image.Count - 1] do
    begin
      Width := IHdr.Width;
      Height := IHdr.Height;
      Size := Width * Height;
      GetMem(Bits, Size);
      for I := 0 to Height - 1 do
      begin
        Stream.Read(Pointer(Long(Bits) + I * Width)^, Width);
        Stream.Position := Stream.Position + 256 - Width;
      end;
    end;
  end;

  procedure LoadUnSubs;
  var
    SubOffs: packed array[0..63] of Long;
    I, Start, J, RPos: Long;
    Z, NZ: Byte;
    W, H: Word;
  begin
    AddImageLevel(Image);
    Owner := @Image.Levels[Image.Count - 1];
    // read subimages offset list
    Start := Stream.Position;
    FillChar(SubOffs, SizeOf(SubOffs), 0);
    Stream.Read(SubOffs, IHdr.SubImages * 4);
    // process subimages
    for I := 0 to IHdr.SubImages - 1 do
    begin
      AddImageLevel(Image, Owner);
      with Owner.Levels[Owner.Count - 1] do
      begin
        Width := IHdr.Width;
        Height := IHdr.Height;
        Size := Width * Height;
        GetMem(Bits, Size);
        Stream.Position := Start + SubOffs[I];
        Stream.Read(W, 2);
        Stream.Read(H, 2);
        // read rows
        for J := 0 to H - 1 do
        begin
          RPos := 0;
          while RPos < W do
          begin
            Stream.Read(Z, 1);
            FillChar(PByte(Long(Bits) + J * W + RPos)^, Z, 0);
            RPos := RPos + Z;
            Stream.Read(NZ, 1);
            Stream.Read(PByte(Long(Bits) + J * W + RPos)^, NZ);
            RPos := RPos + NZ;
          end;
        end;
      end;
    end;
  end;

  procedure LoadRLESubs;
  type
    TRowOff = packed record
      Off: Word;
      Null: Byte;
      RLEIdx: Byte;
    end;

  var
    RowOffs: packed array[0..255] of TRowOff;
    I, J, RPos, NextOff: Long;
    Bytec, C, Ind: Byte;
  begin
    AddImageLevel(Image);
    Owner := @Image.Levels[Image.Count - 1];
    NextOff := Stream.Position;
    // process subimages
    for I := 0 to IHdr.SubImages - 1 do
    begin
      FillChar(RowOffs, SizeOf(RowOffs), 0);
      Stream.Position := NextOff;
      Stream.Read(RowOffs, IHdr.Height * SizeOf(TRowOff));
      NextOff := Stream.Position;
      AddImageLevel(Image, Owner);
      with Owner.Levels[Owner.Count - 1] do
      begin
        Width := IHdr.Width;
        Height := IHdr.Height;
        Size := Width * Height;
        GetMem(Bits, Size);
        // read rows
        for J := 0 to Height - 1 do
        begin
          Stream.Position := HPos + RowOffs[J].Off;
          if RowOffs[J].RLEIdx = 128 then
          begin
            Stream.Read(C, 2);
            RPos := 0;
            while RPos < Width do
            begin
              Stream.Read(Bytec, 1);
              Stream.Read(Ind, 1);
              if Ind = 0 then
              begin
                Stream.Read(PByte(Long(Bits) + J * Width + RPos)^, Bytec);
                RPos := RPos + Bytec;
              end
              else
              begin
                Stream.Read(C, 1);
                FillChar(PByte(Long(Bits) + J * Width + RPos)^, 256 - Bytec, C);
                RPos := RPos + 256 - Bytec;
              end;
            end;
          end
          else
            Stream.Read(PByte(Long(Bits) + J * Width)^, Width);
        end;
      end;
    end;
  end;

begin
  with Stream, Image do
  begin
    Image.DataFormat := FindImageDataFormat(idfIndex8);
    GetMem(Palette, 256 * SizeOf(TARGBRec));
    Move(FPalette, Palette^, SizeOf(FPalette));
    PalSwapChannels(Image.Palette, 256);
    Read(Hdr, SizeOf(Hdr));
    FLastName := RepairName(Hdr.TexName);
    // handle texture.001 and texture.000 which contain color info
    // these contain only indices to palette, we create small
    // levels with colors defined by these indices
    if Stream.Size = 2586 then
    begin
      Coff := 0;
      if PosNoCase('B', FLastName) > 0 then
        Coff := Coff + 128;
      for I := 0 to Hdr.ImgCount - 1 do
      begin
        AddImageLevel(Image);
        with Image.Levels[Image.Count - 1] do
        begin
          Width := 8;
          Height := 8;
          Size := Width * Height;
          GetMem(Bits, Size);
          FillMemory(Bits, Size, I + COff);
        end;
      end;
    end
    else
      if Stream.Size = 46 then
        NewImage(Image, 8, 8, FindImageDataFormat(idfIndex8))
      else
        if Stream.Size = 266 then
          NewImage(Image, 8, 8, FindImageDataFormat(idfIndex8))
        else
        begin
          GetMem(List, Hdr.ImgCount * SizeOf(TOffset));
            // load offsets
          for I := 0 to Hdr.ImgCount - 1 do
            Read(List[I], SizeOf(TOffset));
          // process offsets
          for I := 0 to Hdr.ImgCount - 1 do
          begin
            // jump at position of image header
            Position := List[I].HdrOffset;
            HPos := Position;
            Read(IHdr, SizeOf(IHdr));
            begin
              Position := HPos + IHdr.ImageOff;
              if (IHdr.SubImages = 1) and (IHdr.Unk1 <> $1108) and (IHdr.Unk1
                <> $0108) then
                LoadUncomp;
              if (IHdr.SubImages = 1) and (IHdr.Unk1 = $1108) then
                LoadRLESubs;
              if (IHdr.SubImages > 1) and (IHdr.Unk1 <> $0108) then
                LoadUnSubs;
              if (IHdr.Unk1 = $0108) then
                LoadRLESubs;
            end;
          end;
          FreeMem(List);
        end;
  end;
end;


{*************************************************************************}
{                         Internal Stuff                                  }
{*************************************************************************}


initialization

  TexInstance := TNTexFormat.Create;

finalization

  FreeAndNil(TexInstance);

end.

