unit ZipUtils;

{ ziputils.pas - IO on .zip files using zlib
  - definitions, declarations and routines used by both
    zip.pas and unzip.pas
    The file IO is implemented here.

  based on work by Gilles Vollant

  March 23th, 2000,
  Copyright (C) 2000 Jacques Nomssi Nzali }

interface

uses
  Classes, SysUtils;

{ -------------------------------------------------------------- }

type
  FILEptr = TFileStream;
type
  seek_mode = (SEEK_SET, SEEK_CUR, SEEK_END);
  open_mode = (fopenread, fopenwrite, fappendwrite);

function fopen(filename: PChar; mode: open_mode): FILEptr;

procedure fclose(fp: FILEptr);

function fseek(fp: FILEptr; recPos: Cardinal; mode: seek_mode): LongInt;

function fread(buf: Pointer; recSize: Cardinal;
  recCount: Cardinal; fp: FILEptr): Cardinal;

function fwrite(buf: Pointer; recSize: Cardinal;
  recCount: Cardinal; fp: FILEptr): Cardinal;

function ftell(fp: FILEptr): Cardinal; { ZIP }

function feof(fp: FILEptr): Cardinal; { MiniZIP }

{ ------------------------------------------------------------------- }

type
  zipFile = Pointer;
  unzFile = Pointer;
type
  z_off_t = LongInt;

{ tm_zip contain date/time info }
type
  tm_zip = record
    tm_sec: Cardinal; { seconds after the minute - [0,59] }
    tm_min: Cardinal; { minutes after the hour - [0,59] }
    tm_hour: Cardinal; { hours since midnight - [0,23] }
    tm_mday: Cardinal; { day of the month - [1,31] }
    tm_mon: Cardinal; { months since January - [0,11] }
    tm_year: Cardinal; { years - [1980..2044] }
  end;

  tm_unz = tm_zip;

const
  Z_BUFSIZE = (16384);
  Z_MAXFILENAMEINZIP = (256);

const
  CENTRALHEADERMAGIC = $02014B50;

const
  SIZECENTRALDIRITEM = $2E;
  SIZEZIPLOCALHEADER = $1E;

function ALLOC(size: LongInt): Pointer;

procedure TRYFREE(p: Pointer);

const
  Paszip_copyright: PChar = ' Paszip Copyright 2000 Jacques Nomssi Nzali ';

implementation

function ALLOC(size: LongInt): Pointer;
begin
  GetMem(Result, Size);
end;

procedure TRYFREE(p: Pointer);
begin
  if Assigned(p) then
    FreeMem(P);
end;

{ ---------------------------------------------------------------- }

function fopen(filename: PChar; mode: open_mode): FILEptr;
var
  fp: FILEptr;
begin
  fp := nil;
  try
    case mode of
      fopenread: fp := TFileStream.Create(filename, fmOpenRead);
      fopenwrite: fp := TFileStream.Create(filename, fmCreate);
      fappendwrite:
        begin
          fp := TFileStream.Create(filename, fmOpenReadWrite);
          fp.Seek(soFromEnd, 0);
        end;
    end;
  except
    on EFOpenError do
      fp := nil;
  end;
  fopen := fp;
end;

procedure fclose(fp: FILEptr);
begin
  fp.Free;
end;

function fread(buf: Pointer;
  recSize: Cardinal;
  recCount: Cardinal;
  fp: FILEptr): Cardinal;
var
  totalSize, readcount: Cardinal;
begin
  if Assigned(buf) then
  begin
    totalSize := recCount * Cardinal(recSize);
    readCount := fp.Read(buf^, totalSize);
    if (readcount <> totalSize) then
      fread := readcount div recSize
    else
      fread := recCount;
  end
  else
    fread := 0;
end;

function fwrite(buf: Pointer;
  recSize: Cardinal;
  recCount: Cardinal;
  fp: FILEptr): Cardinal;
var
  totalSize, written: Cardinal;
begin
  if Assigned(buf) then
  begin
    totalSize := recCount * Cardinal(recSize);
    written := fp.Write(buf^, totalSize);
    if (written <> totalSize) then
      fwrite := written div recSize
    else
      fwrite := recCount;
  end
  else
    fwrite := 0;
end;

function fseek(fp: FILEptr;
  recPos: Cardinal;
  mode: seek_mode): LongInt;
const
  fsmode: array[seek_mode] of Word
  = (soFromBeginning, soFromCurrent, soFromEnd);
begin
  fp.Seek(recPos, fsmode[mode]);
  fseek := 0; { = 0 for success }
end;

function ftell(fp: FILEptr): Cardinal;
begin
  ftell := fp.Position;
end;

function feof(fp: FILEptr): Cardinal;
begin
  feof := 0;
  if Assigned(fp) then
    if fp.Position = fp.Size then
      feof := 1
    else
      feof := 0;
end;

{ ---------------------------------------------------------------- }

end.

