{*******************************************************}
{                                                       }
{       Borland Delphi Runtime Library                  }
{       JPEG Image Compression/Decompression Unit       }
{                                                       }
{       Copyright (c) 1997,99 Inprise Corporation       }
{                                                       }
{*******************************************************}


{*************************************************************************}
{*   JpegLib                                                             *}
{*   Marek Mauder, Vampyre SoftWorks                                     *}
{*************************************************************************}
{                                                                         }
{    Allmost all code was removed to get smaller exefile. Using only      }
{    raw external functions and some ugly types.                          }
{                                                                         }
{*************************************************************************}
{  Modified On: 9th March 2003  (09.03.2003)                              }
{*************************************************************************}


unit JpegLib;

interface

uses
  SysUtils, Classes;

 { The following types and external function declarations are used to
  call into functions of the Independent JPEG Group's (IJG) implementation
  of the JPEG image compression/decompression public standard.  The IJG
  library's C source code is compiled into OBJ files and linked into
  the Delphi application. Only types and functions needed by this unit
  are declared; all IJG internal structures are stubbed out with
  generic pointers to reduce internal source code congestion.

  IJG source code copyright (C) 1991-1996, Thomas G. Lane. }

{$Z4} // Minimum enum size = dword
{$A4}

const
  JPEG_LIB_VERSION = 62; { Version 6b }

  JPEG_RST0 = $D0; { RST0 marker code }
  JPEG_EOI = $D9; { EOI marker code }
  JPEG_APP0 = $E0; { APP0 marker code }
  JPEG_COM = $FE; { COM marker code }

  DCTSIZE = 8; { The basic DCT block is 8x8 samples }
  DCTSIZE2 = 64; { DCTSIZE squared; # of elements in a block }
  NUM_QUANT_TBLS = 4; { Quantization tables are numbered 0..3 }
  NUM_HUFF_TBLS = 4; { Huffman tables are numbered 0..3 }
  NUM_ARITH_TBLS = 16; { Arith-coding tables are numbered 0..15 }
  MAX_COMPS_IN_SCAN = 4; { JPEG limit on # of components in one scan }
  MAX_SAMP_FACTOR = 4; { JPEG limit on sampling factors }
  C_MAX_BLOCKS_IN_MCU = 10; { compressor's limit on blocks per MCU }
  D_MAX_BLOCKS_IN_MCU = 10; { decompressor's limit on blocks per MCU }
  MAX_COMPONENTS = 10; { maximum number of image components (color channels) }

  MAXJSAMPLE = 255;
  CENTERJSAMPLE = 128;

type
  JSAMPLE = byte;
  GETJSAMPLE = LongInt;
  JCOEF = LongInt;
  JCOEF_PTR = ^JCOEF;
  UINT8 = byte;
  UINT16 = Word;
  UINT = Cardinal;
  INT16 = SmallInt;
  INT32 = LongInt;
  INT32PTR = ^INT32;
  JDIMENSION = Cardinal;

  JOCTET = Byte;
  jTOctet = 0..(MaxInt div SizeOf(JOCTET)) - 1;
  JOCTET_FIELD = array[jTOctet] of JOCTET;
  JOCTET_FIELD_PTR = ^JOCTET_FIELD;
  JOCTETPTR = ^JOCTET;

  JSAMPLE_PTR = ^JSAMPLE;
  JSAMPROW_PTR = ^JSAMPROW;

  jTSample = 0..(MaxInt div SIZEOF(JSAMPLE)) - 1;
  JSAMPLE_ARRAY = array[jTSample] of JSAMPLE; {far}
  JSAMPROW = ^JSAMPLE_ARRAY; { ptr to one image row of pixel samples. }

  jTRow = 0..(MaxInt div SIZEOF(JSAMPROW)) - 1;
  JSAMPROW_ARRAY = array[jTRow] of JSAMPROW;
  JSAMPARRAY = ^JSAMPROW_ARRAY; { ptr to some rows (a 2-D sample array) }

  jTArray = 0..(MaxInt div SIZEOF(JSAMPARRAY)) - 1;
  JSAMP_ARRAY = array[jTArray] of JSAMPARRAY;
  JSAMPIMAGE = ^JSAMP_ARRAY; { a 3-D sample array: top index is color }

const
  CSTATE_START = 100; { after create_compress }
  CSTATE_SCANNING = 101; { start_compress done, write_scanlines OK }
  CSTATE_RAW_OK = 102; { start_compress done, write_raw_data OK }
  CSTATE_WRCOEFS = 103; { jpeg_write_coefficients done }
  DSTATE_START = 200; { after create_decompress }
  DSTATE_INHEADER = 201; { reading header markers, no SOS yet }
  DSTATE_READY = 202; { found SOS, ready for start_decompress }
  DSTATE_PRELOAD = 203; { reading multiscan file in start_decompress}
  DSTATE_PRESCAN = 204; { performing dummy pass for 2-pass quant }
  DSTATE_SCANNING = 205; { start_decompress done, read_scanlines OK }
  DSTATE_RAW_OK = 206; { start_decompress done, read_raw_data OK }
  DSTATE_BUFIMAGE = 207; { expecting jpeg_start_output }
  DSTATE_BUFPOST = 208; { looking for SOS/EOI in jpeg_finish_output }
  DSTATE_RDCOEFS = 209; { reading file in jpeg_read_coefficients }
  DSTATE_STOPPING = 210; { looking for EOI in jpeg_finish_decompress }

{ Known color spaces. }

type
  J_COLOR_SPACE = (
    JCS_UNKNOWN, { error/unspecified }
    JCS_GRAYSCALE, { monochrome }
    JCS_RGB, { red/green/blue }
    JCS_YCbCr, { Y/Cb/Cr (also known as YUV) }
    JCS_CMYK, { C/M/Y/K }
    JCS_YCCK { Y/Cb/Cr/K }
    );

{ DCT/IDCT algorithm options. }

type
  J_DCT_METHOD = (
    JDCT_ISLOW, { slow but accurate LongInt algorithm }
    JDCT_IFAST, { faster, less accurate LongInt method }
    JDCT_FLOAT { floating-point: accurate, fast on fast HW (Pentium)}
    );

{ Dithering options for decompression. }

type
  J_DITHER_MODE = (
    JDITHER_NONE, { no dithering }
    JDITHER_ORDERED, { simple ordered dither }
    JDITHER_FS { Floyd-Steinberg error diffusion dither }
    );

{ Error handler }

const
  JMSG_LENGTH_MAX = 200; { recommended size of format_message buffer }
  JMSG_STR_PARM_MAX = 80;

  JPOOL_PERMANENT = 0; // lasts until master record is destroyed
  JPOOL_IMAGE = 1; // lasts until done with image/datastream

type
  jpeg_error_mgr_ptr = ^jpeg_error_mgr;
  jpeg_progress_mgr_ptr = ^jpeg_progress_mgr;

  j_common_ptr = ^jpeg_common_struct;
  j_compress_ptr = ^jpeg_compress_struct;
  j_decompress_ptr = ^jpeg_decompress_struct;

{ Routine signature for application-supplied marker processing methods.
  Need not pass marker code since it is stored in cinfo^.unread_marker. }

  jpeg_marker_parser_method = function(cinfo: j_decompress_ptr): LongBool;
{$IFDEF LINUX}cdecl;{$ENDIF}

  { The decompressor can save APPn and COM markers in a list of these: }

  jpeg_saved_marker_ptr = ^jpeg_marker_struct;

  jpeg_marker_struct = record
    next: jpeg_saved_marker_ptr; { next in list, or NULL }
    marker: Byte; { marker code: JPEG_COM, or JPEG_APP0+n }
    original_length: LongWord; { # bytes of data in the file }
    data_length: LongWord; { # bytes of data saved at data[] }
    data: JOCTETPTR; { the data contained in the marker }
    { the marker length word is not counted in data_length or original_length }
  end;

{ Marker reading & parsing }
  jpeg_marker_reader_ptr = ^jpeg_marker_reader;
  jpeg_marker_reader = record
    reset_marker_reader: procedure(cinfo: j_decompress_ptr);
{$IFDEF LINUX}cdecl;{$ENDIF}
    { Read markers until SOS or EOI.
      Returns same codes as are defined for jpeg_consume_input:
      JPEG_SUSPENDED, JPEG_REACHED_SOS, or JPEG_REACHED_EOI. }

    read_markers: function(cinfo: j_decompress_ptr): LongInt;
{$IFDEF LINUX}cdecl;{$ENDIF}
    { Read a restart marker --- exported for use by entropy decoder only }
    read_restart_marker: jpeg_marker_parser_method;
    { Application-overridable marker processing methods }
    process_COM: jpeg_marker_parser_method;
    process_APPn: array[0..16 - 1] of jpeg_marker_parser_method;

    { State of marker reader --- nominally internal, but applications
      supplying COM or APPn handlers might like to know the state. }

    saw_SOI: LongBool; { found SOI? }
    saw_SOF: LongBool; { found SOF? }
    next_restart_num: LongInt; { next restart number expected (0-7) }
    discarded_bytes: UINT; { # of bytes skipped looking for a marker }
  end;

  {int8array = Array[0..8-1] of int;}
  int8array = array[0..8 - 1] of LongInt;

  jpeg_error_mgr = record
    { Error exit handler: does not return to caller }
    error_exit: procedure(cinfo: j_common_ptr);
{$IFDEF LINUX}cdecl;{$ENDIF}
    { Conditionally emit a trace or warning message }
    emit_message: procedure(cinfo: j_common_ptr; msg_level: LongInt);
{$IFDEF LINUX}cdecl;{$ENDIF}
    { Routine that actually outputs a trace or error message }
    output_message: procedure(cinfo: j_common_ptr);
{$IFDEF LINUX}cdecl;{$ENDIF}
    { Format a message string for the most recent JPEG error or message }
    format_message: procedure(cinfo: j_common_ptr; buffer: PChar);
{$IFDEF LINUX}cdecl;{$ENDIF}
    { Reset error state variables at start of a new image }
    reset_error_mgr: procedure(cinfo: j_common_ptr);
{$IFDEF LINUX}cdecl;{$ENDIF}

    { The message ID code and any parameters are saved here.
      A message can have one string parameter or up to 8 int parameters. }

    msg_code: LongInt;

    msg_parm: record
      case byte of
        0: (i: int8array);
        1: (s: array[0..JMSG_STR_PARM_MAX - 1] of char);
    end;
    trace_level: LongInt; { max msg_level that will be displayed }
    num_warnings: LongInt; { number of corrupt-data warnings }
    jpeg_message_table: ^PChar; { Library errors }
    last_jpeg_message: LongInt; { Table contains strings 0..last_jpeg_message }
    { Second table can be added by application (see cjpeg/djpeg for example).
      It contains strings numbered first_addon_message..last_addon_message.
    }
    addon_message_table: ^PChar; { Non-library errors }
    first_addon_message: LongInt; { code for first string in addon table }
    last_addon_message: LongInt; { code for last string in addon table }
  end;


{ Data destination object for compression }
  jpeg_destination_mgr_ptr = ^jpeg_destination_mgr;
  jpeg_destination_mgr = record
    next_output_byte: JOCTETptr; { => next byte to write in buffer }
    free_in_buffer: Longint; { # of byte spaces remaining in buffer }

    init_destination: procedure(cinfo: j_compress_ptr);
{$IFDEF LINUX}cdecl;{$ENDIF}
    empty_output_buffer: function(cinfo: j_compress_ptr): LongBool;
{$IFDEF LINUX}cdecl;{$ENDIF}
    term_destination: procedure(cinfo: j_compress_ptr);
{$IFDEF LINUX}cdecl;{$ENDIF}
  end;


{ Data source object for decompression }

  jpeg_source_mgr_ptr = ^jpeg_source_mgr;
  jpeg_source_mgr = record
    next_input_byte: JOCTETptr; { => next byte to read from buffer }
    bytes_in_buffer: Longint; { # of bytes remaining in buffer }

    init_source: procedure(cinfo: j_decompress_ptr);
{$IFDEF LINUX}cdecl;{$ENDIF}
    fill_input_buffer: function(cinfo: j_decompress_ptr): LongBool;
{$IFDEF LINUX}cdecl;{$ENDIF}
    skip_input_data: procedure(cinfo: j_decompress_ptr; num_bytes: Longint);
{$IFDEF LINUX}cdecl;{$ENDIF}
    resync_to_restart: function(cinfo: j_decompress_ptr;
      desired: LongInt): LongBool;
{$IFDEF LINUX}cdecl;{$ENDIF}
    term_source: procedure(cinfo: j_decompress_ptr);
{$IFDEF LINUX}cdecl;{$ENDIF}
  end;

{ JPEG library memory manger routines }
  jpeg_memory_mgr_ptr = ^jpeg_memory_mgr;
  jpeg_memory_mgr = record
    { Method pointers }
    alloc_small: function(cinfo: j_common_ptr;
      pool_id, sizeofobject: LongInt): pointer;
{$IFDEF LINUX}cdecl;{$ENDIF}
    alloc_large: function(cinfo: j_common_ptr;
      pool_id, sizeofobject: LongInt): pointer;
{$IFDEF LINUX}cdecl;{$ENDIF}
    alloc_sarray: function(cinfo: j_common_ptr; pool_id: LongInt;
      samplesperrow: JDIMENSION; numrows: JDIMENSION): JSAMPARRAY;
{$IFDEF LINUX}cdecl;{$ENDIF}
    alloc_barray: pointer;
    request_virt_sarray: pointer;
    request_virt_barray: pointer;
    realize_virt_arrays: pointer;
    access_virt_sarray: pointer;
    access_virt_barray: pointer;
    free_pool: pointer;
    self_destruct: pointer;
    max_memory_to_use: Longint;
    { Maximum allocation request accepted by alloc_large. }
    max_alloc_chunk: Longint;
  end;

    { Fields shared with jpeg_decompress_struct }
  jpeg_common_struct = record
    err: jpeg_error_mgr_ptr; { Error handler module }
    mem: jpeg_memory_mgr_ptr; { Memory manager module }
    progress: jpeg_progress_mgr_ptr; { Progress monitor, or NIL if none }
    client_data: Pointer; { Available for use by application }
    is_decompressor: LongBool; { so common code can tell which is which }
    global_state: LongInt; { for checking call sequence validity }
  end;

{ Progress monitor object }

  jpeg_progress_mgr = record
    progress_monitor: procedure(const cinfo: jpeg_common_struct);
{$IFDEF LINUX}cdecl;{$ENDIF}
    pass_counter: LongInt; { work units completed in this pass }
    pass_limit: LongInt; { total number of work units in this pass }
    completed_passes: LongInt; { passes completed so far }
    total_passes: LongInt; { total number of passes expected }
  end;


{ Master record for a compression instance }


  jpeg_compress_struct = record
      common: jpeg_common_struct;

      dest: jpeg_destination_mgr_ptr; { Destination for compressed data }

  { Description of source image --- these fields must be filled in by
    outer application before starting compression.  in_color_space must
    be correct before you can even call jpeg_set_defaults(). }

      image_width: JDIMENSION; { input image width }
      image_height: JDIMENSION; { input image height }
      input_components: LongInt; { # of color components in input image }
      in_color_space: J_COLOR_SPACE; { colorspace of input image }
      input_gamma: double; { image gamma of input image }

    // Compression parameters
      data_precision: LongInt; { bits of precision in image data }
      num_components: LongInt; { # of color components in JPEG image }
      jpeg_color_space: J_COLOR_SPACE; { colorspace of JPEG image }
      comp_info: Pointer;
      quant_tbl_ptrs: array[0..NUM_QUANT_TBLS - 1] of Pointer;
      dc_huff_tbl_ptrs: array[0..NUM_HUFF_TBLS - 1] of Pointer;
      ac_huff_tbl_ptrs: array[0..NUM_HUFF_TBLS - 1] of Pointer;
      arith_dc_L: array[0..NUM_ARITH_TBLS - 1] of UINT8;
        { L values for DC arith-coding tables }
      arith_dc_U: array[0..NUM_ARITH_TBLS - 1] of UINT8;
        { U values for DC arith-coding tables }
      arith_ac_K: array[0..NUM_ARITH_TBLS - 1] of UINT8;
        { Kx values for AC arith-coding tables }
      num_scans: LongInt; { # of entries in scan_info array }
      scan_info: Pointer; { script for multi-scan file, or NIL }
      raw_data_in: LongBool; { TRUE=caller supplies downsampled data }
      arith_code: LongBool; { TRUE=arithmetic coding, FALSE=Huffman }
      optimize_coding: LongBool; { TRUE=optimize entropy encoding parms }
      CCIR601_sampling: LongBool; { TRUE=first samples are cosited }
      smoothing_factor: LongInt; { 1..100, or 0 for no input smoothing }
      dct_method: J_DCT_METHOD; { DCT algorithm selector }
      restart_interval: UINT; { MCUs per restart, or 0 for no restart }
      restart_in_rows: LongInt; { if > 0, MCU rows per restart interval }

    { Parameters controlling emission of special markers. }
      write_JFIF_header: LongBool; { should a JFIF marker be written? }
      JFIF_major_version: UINT8; { What to write for a JFIF version number }
      JFIF_minor_version: UINT8;
    { These three values are not used by the JPEG code, merely copied }
    { into the JFIF APP0 marker.  density_unit can be 0 for unknown, }
    { 1 for dots/inch, or 2 for dots/cm.  Note that the pixel aspect }
    { ratio is defined by X_density/Y_density even when density_unit=0. }
      density_unit: UINT8; { JFIF code for pixel size units }
      X_density: UINT16; { Horizontal pixel density }
      Y_density: UINT16; { Vertical pixel density }
      write_Adobe_marker: LongBool; { should an Adobe marker be written? }

    { State variable: index of next scanline to be written to
      jpeg_write_scanlines().  Application may use this to control its
      processing loop, e.g., "while (next_scanline < image_height)". }

      next_scanline: JDIMENSION; { 0 .. image_height-1  }

    { Remaining fields are known throughout compressor, but generally
      should not be touched by a surrounding application. }
      progressive_mode: LongBool; { TRUE if scan script uses progressive mode }
      max_h_samp_factor: LongInt; { largest h_samp_factor }
      max_v_samp_factor: LongInt; { largest v_samp_factor }
      total_iMCU_rows: JDIMENSION; { # of iMCU rows to be input to coef ctlr }
      comps_in_scan: LongInt; { # of JPEG components in this scan }
      cur_comp_info: array[0..MAX_COMPS_IN_SCAN - 1] of Pointer;
      MCUs_per_row: JDIMENSION; { # of MCUs across the image }
      MCU_rows_in_scan: JDIMENSION; { # of MCU rows in the image }
      blocks_in_MCU: LongInt; { # of DCT blocks per MCU }
      MCU_membership: array[0..C_MAX_BLOCKS_IN_MCU - 1] of LongInt;
      Ss, Se, Ah, Al: LongInt; { progressive JPEG parameters for scan }

    { Links to compression subobjects (methods and private variables of modules) }
      master: Pointer;
      main: Pointer;
      prep: Pointer;
      coef: Pointer;
      marker: Pointer;
      cconvert: Pointer;
      downsample: Pointer;
      fdct: Pointer;
      entropy: Pointer;
      script_space: Pointer; { workspace for jpeg_simple_progression }
      script_space_size: LongInt;
    end;


{ Master record for a decompression instance }

    jpeg_decompress_struct = record
      common: jpeg_common_struct;

    { Source of compressed data }
      src: jpeg_source_mgr_ptr;

    { Basic description of image --- filled in by jpeg_read_header(). }
    { Application may inspect these values to decide how to process image. }

      image_width: JDIMENSION; { nominal image width (from SOF marker) }
      image_height: JDIMENSION; { nominal image height }
      num_components: LongInt; { # of color components in JPEG image }
      jpeg_color_space: J_COLOR_SPACE; { colorspace of JPEG image }

    { Decompression processing parameters }
      out_color_space: J_COLOR_SPACE; { colorspace for output }
      scale_num, scale_denom: uint; { fraction by which to scale image }
      output_gamma: double; { image gamma wanted in output }
      buffered_image: LongBool; { TRUE=multiple output passes }
      raw_data_out: LongBool; { TRUE=downsampled data wanted }
      dct_method: J_DCT_METHOD; { IDCT algorithm selector }
      do_fancy_upsampling: LongBool; { TRUE=apply fancy upsampling }
      do_block_smoothing: LongBool; { TRUE=apply interblock smoothing }
      quantize_colors: LongBool; { TRUE=colormapped output wanted }
    { the following are ignored if not quantize_colors: }
      dither_mode: J_DITHER_MODE; { type of color dithering to use }
      two_pass_quantize: LongBool; { TRUE=use two-pass color quantization }
      desired_number_of_colors: LongInt;
        { max # colors to use in created colormap }
    { these are significant only in buffered-image mode: }
      enable_1pass_quant: LongBool; { enable future use of 1-pass quantizer }
      enable_external_quant: LongBool; { enable future use of external colormap }
      enable_2pass_quant: LongBool; { enable future use of 2-pass quantizer }

    { Description of actual output image that will be returned to application.
      These fields are computed by jpeg_start_decompress().
      You can also use jpeg_calc_output_dimensions() to determine these values
      in advance of calling jpeg_start_decompress(). }

      output_width: JDIMENSION; { scaled image width }
      output_height: JDIMENSION; { scaled image height }
      out_color_components: LongInt; { # of color components in out_color_space }
      output_components: LongInt; { # of color components returned }
    { output_components is 1 (a colormap index) when quantizing colors;
      otherwise it equals out_color_components. }

      rec_outbuf_height: LongInt; { min recommended height of scanline buffer }
    { If the buffer passed to jpeg_read_scanlines() is less than this many
      rows high, space and time will be wasted due to unnecessary data
      copying. Usually rec_outbuf_height will be 1 or 2, at most 4. }

    { When quantizing colors, the output colormap is described by these
      fields. The application can supply a colormap by setting colormap
      non-NIL before calling jpeg_start_decompress; otherwise a colormap
      is created during jpeg_start_decompress or jpeg_start_output. The map
      has out_color_components rows and actual_number_of_colors columns. }

      actual_number_of_colors: LongInt; { number of entries in use }
      colormap: JSAMPARRAY; { The color map as a 2-D pixel array }

    { State variables: these variables indicate the progress of decompression.
      The application may examine these but must not modify them. }

    { Row index of next scanline to be read from jpeg_read_scanlines().
      Application may use this to control its processing loop, e.g.,
      "while (output_scanline < output_height)". }

      output_scanline: JDIMENSION; { 0 .. output_height-1  }

    { Current input scan number and number of iMCU rows completed in scan.
      These indicate the progress of the decompressor input side. }

      input_scan_number: LongInt; { Number of SOS markers seen so far }
      input_iMCU_row: JDIMENSION; { Number of iMCU rows completed }

    { The "output scan number" is the notional scan being displayed by the
      output side.  The decompressor will not allow output scan/row number
      to get ahead of input scan/row, but it can fall arbitrarily far behind.}

      output_scan_number: LongInt; { Nominal scan number being displayed }
      output_iMCU_row: LongInt; { Number of iMCU rows read }

      coef_bits: Pointer;

    { Internal JPEG parameters --- the application usually need not look at
      these fields.  Note that the decompressor output side may not use
      any parameters that can change between scans. }

    { Quantization and Huffman tables are carried forward across input
      datastreams when processing abbreviated JPEG datastreams. }

      quant_tbl_ptrs: array[0..NUM_QUANT_TBLS - 1] of Pointer;
      dc_huff_tbl_ptrs: array[0..NUM_HUFF_TBLS - 1] of Pointer;
      ac_huff_tbl_ptrs: array[0..NUM_HUFF_TBLS - 1] of Pointer;

    { These parameters are never carried across datastreams, since they
      are given in SOF/SOS markers or defined to be reset by SOI. }
      data_precision: LongInt; { bits of precision in image data }
      comp_info: Pointer;
      progressive_mode: LongBool; { TRUE if SOFn specifies progressive mode }
      arith_code: LongBool; { TRUE=arithmetic coding, FALSE=Huffman }
      arith_dc_L: array[0..NUM_ARITH_TBLS - 1] of UINT8;
        { L values for DC arith-coding tables }
      arith_dc_U: array[0..NUM_ARITH_TBLS - 1] of UINT8;
        { U values for DC arith-coding tables }
      arith_ac_K: array[0..NUM_ARITH_TBLS - 1] of UINT8;
        { Kx values for AC arith-coding tables }

      restart_interval: UINT; { MCUs per restart interval, or 0 for no restart }

    { These fields record data obtained from optional markers recognized by
      the JPEG library. }
      saw_JFIF_marker: LongBool; { TRUE iff a JFIF APP0 marker was found }
    { Data copied from JFIF marker: only valid if saw_JFIF_marker is TRUE: }
      JFIF_major_version: UINT8; { JFIF version number }
      JFIF_minor_version: UINT8;
      density_unit: UINT8; { JFIF code for pixel size units }
      X_density: UINT16; { Horizontal pixel density }
      Y_density: UINT16; { Vertical pixel density }
      saw_Adobe_marker: LongBool; { TRUE iff an Adobe APP14 marker was found }
      Adobe_transform: UINT8; { Color transform code from Adobe marker }

      CCIR601_sampling: LongBool; { TRUE=first samples are cosited }

    { Aside from the specific data retained from APPn markers known to the
      library, the uninterpreted contents of any or all APPn and COM markers
      can be saved in a list for examination by the application. }
      marker_list: jpeg_saved_marker_ptr; { Head of list of saved markers }

    { Remaining fields are known throughout decompressor, but generally
      should not be touched by a surrounding application. }
      max_h_samp_factor: LongInt; { largest h_samp_factor }
      max_v_samp_factor: LongInt; { largest v_samp_factor }
      min_DCT_scaled_size: LongInt; { smallest DCT_scaled_size of any component }
      total_iMCU_rows: JDIMENSION; { # of iMCU rows in image }
      sample_range_limit: Pointer; { table for fast range-limiting }

    { These fields are valid during any one scan.
      They describe the components and MCUs actually appearing in the scan.
      Note that the decompressor output side must not use these fields. }
      comps_in_scan: LongInt; { # of JPEG components in this scan }
      cur_comp_info: array[0..MAX_COMPS_IN_SCAN - 1] of Pointer;
      MCUs_per_row: JDIMENSION; { # of MCUs across the image }
      MCU_rows_in_scan: JDIMENSION; { # of MCU rows in the image }
      blocks_in_MCU: JDIMENSION; { # of DCT blocks per MCU }
      MCU_membership: array[0..D_MAX_BLOCKS_IN_MCU - 1] of LongInt;
      Ss, Se, Ah, Al: LongInt; { progressive JPEG parameters for scan }

    { This field is shared between entropy decoder and marker parser.
      It is either zero or the code of a JPEG marker that has been
      read from the data source, but has not yet been processed. }
      unread_marker: LongInt;

    { Links to decompression subobjects
      (methods, private variables of modules) }
      master: Pointer;
      main: Pointer;
      coef: Pointer;
      post: Pointer;
      inputctl: Pointer;
      marker: Pointer;
      entropy: Pointer;
      idct: Pointer;
      upsample: Pointer;
      cconvert: Pointer;
      cquantize: Pointer;
    end;

    TJPEGContext = record
      err: jpeg_error_mgr;
      progress: jpeg_progress_mgr;
      FinalDCT: J_DCT_METHOD;
      FinalTwoPassQuant: Boolean;
      FinalDitherMode: J_DITHER_MODE;
      case byte of
        0: (common: jpeg_common_struct);
        1: (d: jpeg_decompress_struct);
        2: (c: jpeg_compress_struct);
    end;

{ Decompression startup: read start of JPEG datastream to see what's there
   function jpeg_read_header (cinfo : j_decompress_ptr;
                              require_image : LongBool) : LongInt;
  Return value is one of: }
  const
    JPEG_SUSPENDED = 0; { Suspended due to lack of input data }
    JPEG_HEADER_OK = 1; { Found valid image datastream }
    JPEG_HEADER_TABLES_ONLY = 2; { Found valid table-specs-only datastream }
{ If you pass require_image = TRUE (normal case), you need not check for
  a TABLES_ONLY return code; an abbreviated file will cause an error exit.
  JPEG_SUSPENDED is only possible if you use a data source module that can
  give a suspension return (the stdio source module doesn't). }


{ function jpeg_consume_input (cinfo : j_decompress_ptr) : LongInt;
  Return value is one of: }

    JPEG_REACHED_SOS = 1; { Reached start of new scan }
    JPEG_REACHED_EOI = 2; { Reached end of image }
    JPEG_ROW_COMPLETED = 3; { Completed one iMCU row }
    JPEG_SCAN_COMPLETED = 4; { Completed last iMCU row of a scan }


{$IFDEF MSWINDOWS}
{$L jdapimin.obj}
{$L jmemmgr.obj}
{$L jmemnobs.obj}
{$L jdinput.obj}
{$L jdatasrc.obj}
{$L jdapistd.obj}
{$L jdmaster.obj}
{$L jdphuff.obj}
{$L jdhuff.obj}
{$L jdmerge.obj}
{$L jdcolor.obj}
{$L jquant1.obj}
{$L jquant2.obj}
{$L jdmainct.obj}
{$L jdcoefct.obj}
{$L jdpostct.obj}
{$L jddctmgr.obj}
{$L jdsample.obj}
{$L jidctflt.obj}
{$L jidctfst.obj}
{$L jidctint.obj}
{$L jidctred.obj}
{$L jdmarker.obj}
{$L jutils.obj}
{$L jcomapi.obj}
{$ENDIF}

procedure jpeg_CreateDecompress(var cinfo: jpeg_decompress_struct;
  version: LongInt; structsize: LongInt);
{$IFDEF LINUX}cdecl;{$ENDIF}
external{$IFDEF LINUX} 'libjpeg.so' name 'jpeg_CreateDecompress'{$ENDIF};
procedure jpeg_stdio_src(var cinfo: jpeg_decompress_struct;
  input_file: TStream);
{$IFDEF LINUX}cdecl;{$ENDIF}
external{$IFDEF LINUX} 'libjpeg.so' name 'jpeg_stdio_src'{$ENDIF};
procedure jpeg_read_header(var cinfo: jpeg_decompress_struct;
  RequireImage: LongBool);
{$IFDEF LINUX}cdecl;{$ENDIF}
external{$IFDEF LINUX} 'libjpeg.so' name 'jpeg_read_header'{$ENDIF};
procedure jpeg_calc_output_dimensions(var cinfo: jpeg_decompress_struct);
{$IFDEF LINUX}cdecl;{$ENDIF}
external{$IFDEF LINUX} 'libjpeg.so' name 'jpeg_calc_output_dimensions'{$ENDIF};
procedure jpeg_save_markers(var cinfo: jpeg_decompress_struct; marker_code:
  LongInt; length_limit: UINT);
{$IFDEF LINUX}cdecl;{$ENDIF}
external{$IFDEF LINUX} 'libjpeg.so' name 'jpeg_save_markers'{$ENDIF};
function jpeg_start_decompress(var cinfo: jpeg_decompress_struct): Longbool;
{$IFDEF LINUX}cdecl;{$ENDIF}
external{$IFDEF LINUX} 'libjpeg.so' name 'jpeg_start_decompress'{$ENDIF};
function jpeg_read_scanlines(var cinfo: jpeg_decompress_struct;
  scanlines: JSAMPARRAY; max_lines: JDIMENSION): JDIMENSION;
{$IFDEF LINUX}cdecl;{$ENDIF}
external{$IFDEF LINUX} 'libjpeg.so' name 'jpeg_read_scanlines'{$ENDIF};
function jpeg_finish_decompress(var cinfo: jpeg_decompress_struct): Longbool;
{$IFDEF LINUX}cdecl;{$ENDIF}
external{$IFDEF LINUX} 'libjpeg.so' name 'jpeg_finish_decompress'{$ENDIF};
procedure jpeg_destroy_decompress(var cinfo: jpeg_decompress_struct);
{$IFDEF LINUX}cdecl;{$ENDIF}
external{$IFDEF LINUX} 'libjpeg.so' name 'jpeg_destroy_decompress'{$ENDIF};
function jpeg_has_multiple_scans(var cinfo: jpeg_decompress_struct): Longbool;
{$IFDEF LINUX}cdecl;{$ENDIF}
external{$IFDEF LINUX} 'libjpeg.so' name 'jpeg_has_multiple_scans'{$ENDIF};
function jpeg_consume_input(var cinfo: jpeg_decompress_struct): LongInt;
{$IFDEF LINUX}cdecl;{$ENDIF}
external{$IFDEF LINUX} 'libjpeg.so' name 'jpeg_consume_input'{$ENDIF};
function jpeg_start_output(var cinfo: jpeg_decompress_struct; scan_number:
  LongInt): Longbool;
{$IFDEF LINUX}cdecl;{$ENDIF}
external{$IFDEF LINUX} 'libjpeg.so' name 'jpeg_start_output'{$ENDIF};
function jpeg_finish_output(var cinfo: jpeg_decompress_struct): LongBool;
{$IFDEF LINUX}cdecl;{$ENDIF}
external{$IFDEF LINUX} 'libjpeg.so' name 'jpeg_finish_output'{$ENDIF};
procedure jpeg_destroy(var cinfo: jpeg_common_struct);
{$IFDEF LINUX}cdecl;{$ENDIF}
external{$IFDEF LINUX} 'libjpeg.so' name 'jpeg_destroy'{$ENDIF};

{$IFDEF MSWINDOWS}
{$L jdatadst.obj}
{$L jcparam.obj}
{$L jcapistd.obj}
{$L jcapimin.obj}
{$L jcinit.obj}
{$L jcmarker.obj}
{$L jcmaster.obj}
{$L jcmainct.obj}
{$L jcprepct.obj}
{$L jccoefct.obj}
{$L jccolor.obj}
{$L jcsample.obj}
{$L jcdctmgr.obj}
{$L jcphuff.obj}
{$L jfdctint.obj}
{$L jfdctfst.obj}
{$L jfdctflt.obj}
{$L jchuff.obj}
{$ENDIF}

procedure jpeg_CreateCompress(var cinfo: jpeg_compress_struct;
  version: LongInt; structsize: LongInt);
{$IFDEF LINUX}cdecl;{$ENDIF}
external{$IFDEF LINUX} 'libjpeg.so' name 'jpeg_CreateCompress'{$ENDIF};
procedure jpeg_stdio_dest(var cinfo: jpeg_compress_struct;
  output_file: TStream);
{$IFDEF LINUX}cdecl;{$ENDIF}
external{$IFDEF LINUX} 'libjpeg.so' name 'jpeg_stdio_dest'{$ENDIF};
procedure jpeg_set_defaults(var cinfo: jpeg_compress_struct);
{$IFDEF LINUX}cdecl;{$ENDIF}
external{$IFDEF LINUX} 'libjpeg.so' name 'jpeg_set_defaults'{$ENDIF};
procedure jpeg_set_quality(var cinfo: jpeg_compress_struct; Quality: LongInt;
  Baseline: Longbool);
{$IFDEF LINUX}cdecl;{$ENDIF}
external{$IFDEF LINUX} 'libjpeg.so' name 'jpeg_set_quality'{$ENDIF};
procedure jpeg_set_colorspace(var cinfo: jpeg_compress_struct;
  colorspace: J_COLOR_SPACE);
{$IFDEF LINUX}cdecl;{$ENDIF}
external{$IFDEF LINUX} 'libjpeg.so' name 'jpeg_set_colorspace'{$ENDIF};
procedure jpeg_simple_progression(var cinfo: jpeg_compress_struct);
{$IFDEF LINUX}cdecl;{$ENDIF}
external{$IFDEF LINUX} 'libjpeg.so' name 'jpeg_simple_progression'{$ENDIF};
procedure jpeg_start_compress(var cinfo: jpeg_compress_struct;
  WriteAllTables: LongBool);
{$IFDEF LINUX}cdecl;{$ENDIF}
external{$IFDEF LINUX} 'libjpeg.so' name 'jpeg_start_compress'{$ENDIF};
function jpeg_write_scanlines(var cinfo: jpeg_compress_struct;
  scanlines: JSAMPARRAY; max_lines: JDIMENSION): JDIMENSION;
{$IFDEF LINUX}cdecl;{$ENDIF}
external{$IFDEF LINUX} 'libjpeg.so' name 'jpeg_write_scanlines'{$ENDIF};
procedure jpeg_finish_compress(var cinfo: jpeg_compress_struct);
{$IFDEF LINUX}cdecl;{$ENDIF}
external{$IFDEF LINUX} 'libjpeg.so' name 'jpeg_finish_compress'{$ENDIF};

{$IFDEF LINUX}
procedure JPEGStdioSrc(var cinfo: jpeg_decompress_struct; Stream: TStream);
procedure JPEGStdioDest(var cinfo: jpeg_compress_struct; outfile: TStream);
{$ENDIF}


implementation


// Stubs for external C RTL functions referenced by JPEG OBJ files.

{$IFDEF MSWINDOWS}
function _malloc(size: LongInt): Pointer; cdecl;
begin
  GetMem(Result, size);
end;

procedure _free(P: Pointer); cdecl;
begin
  FreeMem(P);
end;

procedure _memset(P: Pointer; B: Byte; count: LongInt); cdecl;
begin
  FillChar(P^, count, B);
end;

procedure _memcpy(dest, source: Pointer; count: LongInt); cdecl;
begin
  Move(source^, dest^, count);
end;

function _fread(var buf; recsize, reccount: LongInt; S: TStream): LongInt;
  cdecl;
begin
  Result := S.Read(buf, recsize * reccount);
end;

function _fwrite(const buf; recsize, reccount: LongInt; S: TStream): LongInt;
  cdecl;
begin
  Result := S.Write(buf, recsize * reccount);
end;

function _fflush(S: TStream): LongInt; cdecl;
begin
  Result := 0;
end;

function __ftol: LongInt;
var
  f: double;
begin
  asm
    lea    eax, f             //  BC++ passes floats on the FPU stack
    fstp  qword ptr [eax]     //  Delphi passes floats on the CPU stack
  end;
  Result := LongInt(Trunc(f));
end;

var
  __turboFloat: LongBool = False;
{$ENDIF}

{$IFDEF LINUX}
const
  INPUT_BUF_SIZE = 4096;

type
  my_source_mgr = record
    pub: jpeg_source_mgr;
    infile: TStream;
    Buffer: JOCTETPTR;
    StartOfFile: Boolean;
  end;
  my_source_ptr = ^my_source_mgr;

procedure Init_Source(cinfo: j_decompress_ptr); cdecl; export;
begin
  my_source_ptr(cinfo.src).StartOfFile := True;
end;

function Fill_Input_Buffer(cinfo: j_decompress_ptr): LongBool; cdecl; export;
var
  nBytes: LongInt;
  Src: my_source_ptr;
begin
  Src := my_source_ptr(cinfo.src);
  nBytes := Src.infile.Read(Src.Buffer^, INPUT_BUF_SIZE);

  if nBytes <= 0 then
  begin
    if Src.StartOfFile then
    begin
      cinfo.common.err.msg_code := 1000;
      cinfo.common.err.error_exit(j_common_ptr(cinfo));
    end;
    cinfo.common.err.msg_code := 1001;
    cinfo.common.err.emit_message(j_common_ptr(cinfo), -1);
    PChar(Src.Buffer)[0] := #$FF;
    PChar(Src.Buffer)[1] := Char(JPEG_EOI);
    nBytes := 2;
  end;
  Src.pub.next_input_byte := Src.Buffer;
  Src.pub.bytes_in_buffer := nBytes;
  Src.StartOfFile := False;
  Result := True;
end;

procedure Skip_Input_Data(cinfo: j_decompress_ptr; num_bytes: Longint); cdecl;
  export;
var
  Src: my_source_ptr;
begin
  Src := my_source_ptr(cinfo.src);
  if num_bytes > 0 then
  begin
    while num_bytes > Src.pub.bytes_in_buffer do
    begin
      Dec(num_bytes, Src.pub.bytes_in_buffer);
      Fill_Input_Buffer(cinfo);
    end;
    Inc(LongInt(Src.pub.next_input_byte), num_bytes);
    Dec(Src.Pub.bytes_in_buffer, num_bytes);
  end;
end;

function jpeg_resync_to_restart(cinfo: j_decompress_ptr; desired: LongInt):
  LongBool; cdecl; external 'libjpeg.so' name 'jpeg_resync_to_restart';

procedure Term_Source(cinfo: j_decompress_ptr); cdecl; export;
begin
  { no work nessesary here }
end;

procedure JPEGStdioSrc(var cinfo: jpeg_decompress_struct; Stream: TStream);
var
  Src: my_source_ptr;
begin
  if cinfo.src = nil then
  begin
    cinfo.src := cinfo.common.mem.alloc_small(j_common_ptr(@cinfo),
      JPOOL_PERMANENT,
      SizeOf(my_source_mgr));
    Src := my_source_ptr(cinfo.src);
    Src.Buffer := cinfo.common.mem.alloc_small(j_common_ptr(@cinfo),
      JPOOL_PERMANENT,
      INPUT_BUF_SIZE * SizeOf(JOCTET));
  end;
  Src := my_source_ptr(cinfo.src);
  Src.pub.init_source := Init_Source;
  Src.pub.fill_input_buffer := Fill_Input_Buffer;
  Src.pub.skip_input_data := Skip_Input_Data;
  Src.pub.resync_to_restart := jpeg_resync_to_restart;
  Src.pub.term_source := Term_Source;
  Src.infile := Stream;
  Src.pub.bytes_in_buffer := 0;
  Src.pub.next_input_byte := nil;
end;

type
  my_destination_mgr = record
    pub: jpeg_destination_mgr;
    outfile: TStream;
    Buffer: JOCTETPTR;
  end;
  my_destination_ptr = ^my_destination_mgr;

const
  OUTPUT_BUF_SIZE = 4096;

procedure init_destination(cinfo: j_compress_ptr); cdecl; export;
var
  Dest: my_destination_ptr;
begin
  Dest := my_destination_ptr(cinfo.dest);
  Dest.pub.next_output_byte := Dest.Buffer;
  Dest.pub.free_in_buffer := OUTPUT_BUF_SIZE;
end;

function empty_output_buffer(cinfo: j_compress_ptr): LongBool; cdecl; export;
var
  Dest: my_destination_ptr;
begin
  Dest := my_destination_ptr(cinfo.dest);
  if Dest.outFile.Write(Dest.Buffer^, OUTPUT_BUF_SIZE) <> OUTPUT_BUF_SIZE then
  begin
    cinfo.common.err.msg_code := 1002;
    cinfo.common.err.emit_message(j_common_ptr(cinfo), -1);
  end;
  Dest.pub.next_output_byte := Dest.Buffer;
  Dest.pub.free_in_buffer := OUTPUT_BUF_SIZE;
  Result := True;
end;

procedure term_destination(cinfo: j_compress_ptr); cdecl; export;
var
  Dest: my_destination_ptr;
  DataCount: LongInt;
begin
  Dest := my_destination_ptr(cinfo.dest);
  DataCount := OUTPUT_BUF_SIZE - Dest.pub.free_in_buffer;
  if DataCount > 0 then
    if Dest.outFile.Write(Dest.Buffer^, DataCount) <> DataCount then
    begin
      cinfo.common.err.msg_code := 1002;
      cinfo.common.err.emit_message(j_common_ptr(cinfo), -1);
    end;
end;

procedure JPEGStdioDest(var cinfo: jpeg_compress_struct; outfile: TStream);
var
  Dest: my_destination_ptr;
begin
  if cinfo.dest = nil then
    cinfo.dest := cinfo.common.mem.alloc_small(j_common_ptr(@cinfo),
      JPOOL_PERMANENT, SizeOf(my_destination_mgr));

  Dest := my_destination_ptr(cinfo.dest);
  Dest.Buffer := cinfo.common.mem.alloc_small(j_common_ptr(@cinfo), JPOOL_IMAGE,
    OUTPUT_BUF_SIZE * SIZEOF(JOCTET));
  Dest.pub.init_destination := init_destination;
  Dest.pub.empty_output_buffer := empty_output_buffer;
  Dest.pub.term_destination := term_destination;
  Dest.outfile := outfile;
end;
{$ENDIF}

end.



