{*************************************************************************}
{*   MiniLZO                                                             *}
{*   Copyright (C) 2004 Marek Mauder, Vampyre SoftWorks                  *}
{*   pentar@seznam.cz, http://vampyre.wz.cz                              *}
{*************************************************************************}
{                                                                         }
{ This program is free software; you can redistribute it and/or modify it }
{ under the terms of the GNU General Public License as published by the   }
{ Free Software Foundation; either version 2 of the License, or (at your  }
{ option) any later version. This program is distributed in the hope that }
{ it will be useful, but WITHOUT ANY WARRANTY; without even the implied   }
{ warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See   }
{ the GNU General Public License for more details.                        }
{                                                                         }
{ You should have received a copy of the GNU General Public License       }
{ along with this program; if not, write to the Free Software Foundation, }
{ Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.         }
{                                                                         }
{*************************************************************************}
{                                                                         }
{ Pascal interface to MiniLZO real-time compression                       }
{ library compiled to object file and linked with this unit.              }
{ LZO does not have the best compress ratio but is extremely fast.        }
{ I added CompressBuffer and DecompressBuffer procedures with similar     }
{ syntax and behaviour as Borland's ZLib interface unit so you does not   }
{ have use ugly original named C functions like _lzo1x_1_compress.        }
{                                                                         }
{*************************************************************************}
{                                                                         }
{ Original copyrights of minilzo:                                         }
{                                                                         }
{   minilzo.h - mini subset of the LZO real-time data compression library }
{                                                                         }
{   Copyright (C) 2000 Markus Franz Xaver Johannes Oberhumer              }
{                                                                         }
{   Markus F.X.J. Oberhumer                                               }
{   <markus.oberhumer@jk.uni-linz.ac.at>                                  }
{   http://wildsau.idv.uni-linz.ac.at/mfx/lzo.html                        }
{                                                                         }
{*************************************************************************}

{*************************************************************************}
{                                                                         }
{ 03.01.2004 - first version - Win32 only, minilzo.obj compiled in        }
{              C++ Builder X 1.0.                                         }
{                                                                         }
{*************************************************************************}

unit MiniLZO;

interface

{*****************************************************************************
*  CompressBuffer                                                            *
*                                                                            *
*  pre-conditions                                                            *
*    InBuf    - pointer to uncompressed data                                 *
*    InBytes  - size of InBuf in bytes                                       *
*    OutBuf   - unallocated or nil pointer                                   *
*                                                                            *
*  post-conditions                                                           *
*    OutBuf   - pointer to compressed data                                   *
*    OutBytes - size of OutBuf in bytes                                      *
*****************************************************************************}

procedure CompressBuffer(const InBuf: Pointer; InBytes: LongInt;
  var OutBuf: Pointer; var OutBytes: LongInt);

{*****************************************************************************
*  DecompressBuffer                                                          *
*                                                                            *
*  pre-conditions                                                            *
*    InBuf    - pointer to compressed data                                   *
*    InBytes  - size of InBuf in bytes                                       *
*    OutBuf   - unallocated or nil pointer                                   *
*                                                                            *
*  post-conditions                                                           *
*    OutBuf   - pointer to uncompressed data                                 *
*    OutBytes - size of OutBuf in bytes                                      *
*****************************************************************************}

procedure DecompressBuffer(const InBuf: Pointer; InBytes: LongInt;
  var OutBuf: Pointer; var OutBytes: LongInt);

implementation

uses
  SysUtils;

type
  LZO_Byte = Byte;
  LZO_Bytep = ^Byte;
  LZO_Int = LongInt;
  LZO_Intp = ^LongInt;
  LZO_UInt = Cardinal;
  LZO_UIntp = ^Cardinal;
  LZO_Voidp = Pointer;

const
  LZO_MEM_COMPRESS = 16384 * SizeOf(LZO_Bytep);
  LZO_MEM_DECOMPRESS = 0;

{$IFDEF MSWINDOWS}
{$L minilzo.obj}
{$ENDIF}
{$IFDEF LINUX}
{$L minilzo.o}
{$ENDIF}

function _lzo1x_1_compress(Src: LZO_Bytep; SrcLen: LZO_UInt;
  Dst: LZO_Bytep; DstLen: LZO_UIntp;
  WorkMem: LZO_Voidp): LZO_Int; cdecl; external;

function _lzo1x_decompress(Src: LZO_Bytep; SrcLen: LZO_Int;
  Dst: LZO_Bytep; DstLen: LZO_Intp;
  WorkMem: LZO_Voidp): LZO_Int; cdecl; external;

procedure _memset(P: Pointer; B: Byte; count: LongInt); cdecl;
begin
  FillChar(P^, count, B);
end;

procedure _memcpy(dest, source: Pointer; count: LongInt); cdecl;
begin
  Move(source^, dest^, count);
end;

procedure _memmove(dest, source: Pointer; count: LongInt); cdecl;
begin
  Move(source^, dest^, count);
end;

function _memcmp(dest, source: Pointer; count: LongInt): LongInt; cdecl;
begin
  Result := SysUtils.StrLComp(dest, source, count);
end;

var
  Work: array[0..LZO_MEM_COMPRESS - 1] of Byte;

procedure CompressBuffer(const InBuf: Pointer; InBytes: LongInt;
  var OutBuf: Pointer; var OutBytes: LongInt);
begin
  GetMem(OutBuf, InBytes + InBytes shr 1);
  try
    LZO_Intp(OutBuf)^ := InBytes;
    _lzo1x_1_compress(InBuf, InBytes, LZO_Bytep(LongInt(OutBuf) +
      SizeOf(LongInt)), @OutBytes, @Work);
    OutBytes := OutBytes + SizeOf(LongInt);
    ReallocMem(OutBuf, OutBytes);
  except
    FreeMem(OutBuf);
    OutBuf := nil;
    raise;
  end;
end;

procedure DecompressBuffer(const InBuf: Pointer; InBytes: LongInt;
  var OutBuf: Pointer; var OutBytes: LongInt);
var
  Size: LongInt;
begin
  try
    Size := LZO_Intp(InBuf)^;
    GetMem(OutBuf, Size);
    _lzo1x_decompress(LZO_Bytep(LongInt(InBuf) + SizeOf(LongInt)), InBytes -
      SizeOf(LongInt), OutBuf, @OutBytes, @Work);
    ReallocMem(OutBuf, OutBytes);
  except
    FreeMem(OutBuf);
    OutBuf := nil;
    raise;
  end;
end;

end.

