unit AExpActions;

interface

uses
  SysUtils, Classes, Kernel, Files, CSingletons, CBase, CEditor, CProjects,
  Forms, ComCtrls, Objects;

type

  TExpActions = class(TObject)
  protected
    FSolutionChanged: Boolean;

    function GetTree: TTreeView;
    function GetTab: TTabControl;

    procedure OnSolutionChange;
  public
    constructor Create;
    procedure Idle;
    procedure UpdateSolution;
  end;

function ExpActions: TExpActions;

implementation

uses
  FMain;

var
  ExpActionsInstance: TExpActions = nil;

function ExpActions: TExpActions;
begin
  if ExpActionsInstance = nil then
    ExpActionsInstance := TExpActions.Create;
  Result := ExpActionsInstance;
end;

{ TExpActions }

constructor TExpActions.Create;
begin
  Log.Add('ExpActions created.');
  FSolutionChanged := True;
  Solution.OnChange := OnSolutionChange;
end;

function TExpActions.GetTab: TTabControl;
begin
  Result := FMain.MainForm.TabExp;
end;

function TExpActions.GetTree: TTreeView;
begin
  Result := FMain.MainForm.TreeExp;
end;

procedure TExpActions.Idle;
begin
  case GetTab.TabIndex of
    0: UpdateSolution;
  end;
end;

procedure TExpActions.OnSolutionChange;
begin
  FSolutionChanged := True;
end;

procedure TExpActions.UpdateSolution;
var
  I, J: Long;
  Root, Proj: TTreeNode;
begin
  if FSolutionChanged then
    with GetTree do
    begin
      Items.Clear;
      Selected := nil;
      Root := Items.AddChildObject(nil, Solution.Name + Format(' (%d)',
        [Solution.Count]), Solution);
      for I := 0 to Solution.Count - 1 do
      begin
        Proj := Items.AddChildObject(Root, Solution.Projects[I].Name + ' ',
          Solution.Projects[I]);
        for J := 0 to Solution.Projects[I].Count - 1 do
        begin
          Items.AddChildObject(Proj,
            ExtractFileName(Solution.Projects[I].Files[J].FileName),
            Solution.Projects[I].Files[J]);
        end;
      end;
      FullExpand;
      FSolutionChanged := False;
    end;
end;

end.

