unit FEditor;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, SynEdit, ComCtrls, Types, Kernel, ActnList, VideVclActions, Menus,
  ImgList, ActnMenus, AEditorActions, ActnPopupCtrl, FEditorOptions,
  AMainActions, Platforms, ABase, SynEditTypes, AOptions;

type
  TEditorFrame = class(TFrame)
    TabEditor: TTabControl;
    ActionsEditor: TActionList;
    ImagesEditor: TImageList;
    VideEditUndo: TVideEditUndo;
    VideEditRedo: TVideEditRedo;
    VideEditCut: TVideEditCut;
    VideEditCopy: TVideEditCopy;
    VideEditDelete: TVideEditDelete;
    VideEditPaste: TVideEditPaste;
    VideEditSelectAll: TVideEditSelectAll;
    VideFormatLowerCase: TVideFormatLowerCase;
    VideFormatUpperCase: TVideFormatUpperCase;
    PopupEditor: TPopupActionBarEx;
    Cut1: TMenuItem;
    Copy1: TMenuItem;
    Paste1: TMenuItem;
    Status: TStatusBar;
    SynEdit: TSynEdit;
    N1: TMenuItem;
    PopupTab: TPopupActionBarEx;
    ClosePage2: TMenuItem;
    N2: TMenuItem;
    Properties1: TMenuItem;
    ClosePage: TAction;
    ClosePage1: TMenuItem;
    EditorProperties: TAction;
    Format1: TMenuItem;
    N3: TMenuItem;
    MakeUppercase1: TMenuItem;
    MakeLowercase1: TMenuItem;
    SearchFind: TAction;
    SearchReplace: TAction;
    SearchGoto: TAction;
    SearchAgain: TAction;
    procedure TabEditorMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ClosePageExecute(Sender: TObject);
    procedure EditorPropertiesExecute(Sender: TObject);
    procedure SynEditDropFiles(Sender: TObject; X, Y: Integer;
      AFiles: TStrings);
    procedure TabEditorDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure TabEditorDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure SearchFindExecute(Sender: TObject);
    procedure SearchAgainExecute(Sender: TObject);
    procedure SearchFindUpdate(Sender: TObject);
    procedure SearchReplaceUpdate(Sender: TObject);
    procedure SearchGotoUpdate(Sender: TObject);
    procedure SearchAgainUpdate(Sender: TObject);
    procedure SearchReplaceExecute(Sender: TObject);
    procedure SynEditReplaceText(Sender: TObject; const ASearch,
      AReplace: string; Line, Column: Integer;
      var Action: TSynReplaceAction);
    procedure SearchGotoExecute(Sender: TObject);
    procedure SynEditDblClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

implementation

uses
  FMain, FData;

{$R *.dfm}

procedure TEditorFrame.TabEditorMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  TabEditor.TabIndex := TabEditor.IndexOfTabAt(X, Y);
  EditorActions.ActivateIndex(TabEditor.TabIndex);
  if Button = mbLeft then
    TabEditor.BeginDrag(False);
end;

procedure TEditorFrame.ClosePageExecute(Sender: TObject);
begin
  MainActions.CloseFile;
end;

procedure TEditorFrame.EditorPropertiesExecute(Sender: TObject);
begin
  EditorOptionsForm.ShowModal;
end;

procedure TEditorFrame.SynEditDropFiles(Sender: TObject; X, Y: Integer;
  AFiles: TStrings);
begin
  MainActions.OpenFiles(False, AFiles);
end;

procedure TEditorFrame.TabEditorDragDrop(Sender, Source: TObject; X,
  Y: Integer);
var
  I: Long;
begin
  I := TabEditor.IndexOfTabAt(X, Y);
  if (Source is TTabControl) and (TabEditor.TabIndex <> I) then
    EditorActions.Exchange(I);
end;

procedure TEditorFrame.TabEditorDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
var
  I: Long;
begin
  I := TabEditor.IndexOfTabAt(X, Y);
  Accept := (Source is TTabControl) and (TabEditor.TabIndex <> I);
end;

procedure TEditorFrame.SearchFindExecute(Sender: TObject);
begin
  EditorActions.Search(smFindNew);
end;

procedure TEditorFrame.SearchAgainExecute(Sender: TObject);
begin
  EditorActions.Search(smNext);
end;

procedure TEditorFrame.SearchFindUpdate(Sender: TObject);
begin
  SearchFind.Enabled := EditorActions.EditorVisible;
end;

procedure TEditorFrame.SearchReplaceUpdate(Sender: TObject);
begin
  SearchReplace.Enabled := EditorActions.EditorVisible;
end;

procedure TEditorFrame.SearchGotoUpdate(Sender: TObject);
begin
  SearchGoto.Enabled := EditorActions.EditorVisible;
end;

procedure TEditorFrame.SearchAgainUpdate(Sender: TObject);
begin
  SearchAgain.Enabled := EditorActions.EditorVisible;
end;

procedure TEditorFrame.SearchReplaceExecute(Sender: TObject);
begin
  EditorActions.Search(smReplaceNew);
end;

procedure TEditorFrame.SynEditReplaceText(Sender: TObject; const ASearch,
  AReplace: string; Line, Column: Integer; var Action: TSynReplaceAction);
var
  P: Types.TPoint;
  C: TDisplayCoord;
begin
  C.Column := Column;
  C.Row := Line + 1;
  P := SynEdit.ClientToScreen(SynEdit.RowColumnToPixels(C));
  if P.X + 310 > Screen.Width then
    P.X:=Screen.Width-310;
  if P.Y + 150 > Screen.Height then
    P.Y:=Screen.Height-150;
  case MessageDlgPos(Format('Replace this occurrence of ''%s'' with ''%s''?',
    [ASearch, AReplace]), mtConfirmation, [mbYes, mbNo, mbCancel, mbAll], 0,
    P.X, P.Y) of
    mrYes: action := raReplace;
    mrNo: action := raSkip;
    mrCancel: action := raCancel;
    mrAll: action := raReplaceAll;
  end;
end;

procedure TEditorFrame.SearchGotoExecute(Sender: TObject);
var
  S:string;
  I:Long;
begin
  S:=IntToStr(SynEdit.CaretY);
  if Dialogs.InputQuery('Go To Line Number',
    Format('Enter number of line (max %d):', [SynEdit.Lines.Count]), S) then
   if TryStrToInt(S, I) then
     EditorActions.GotoLine(I)
   else
     MainActions.ErrorDlg(Format('''%s'' is not a valid line number.',[S]), 'Error');
end;

procedure TEditorFrame.SynEditDblClick(Sender: TObject);
begin
if Options.Editor.DblClickLine then
   begin
     SynEdit.BlockBegin:= TBufferCoord(Point(1, SynEdit.CaretY));
     SynEdit.BlockEnd:= TBufferCoord(Point(1, SynEdit.CaretY +1));
   end;
end;

end.

