unit FEditorOptions;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, SynEdit, ExtCtrls, ComCtrls, Spin, WizardHeader,
  SynEditHighlighter, SynHighlighterPas, Kernel, AOptions;

type
  TEditorOptionsForm = class(TForm)
    Pages: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet3: TTabSheet;
    GroupBox1: TGroupBox;
    Panel1: TPanel;
    Button1: TButton;
    Button2: TButton;
    Label1: TLabel;
    Label2: TLabel;
    GroupBox3: TGroupBox;
    CheckSmartTabs: TCheckBox;
    CheckUseTab: TCheckBox;
    CheckBackspace: TCheckBox;
    CheckEOF: TCheckBox;
    CheckEOL: TCheckBox;
    CheckGroup: TCheckBox;
    CheckInsert: TCheckBox;
    CheckHint: TCheckBox;
    CheckHalfPage: TCheckBox;
    CheckScrolls: TCheckBox;
    CheckHome: TCheckBox;
    CheckIndent: TCheckBox;
    Wizard: TWizardHeader;
    TabSheet2: TTabSheet;
    ListElems: TListBox;
    ColorBack: TColorBox;
    ColorFore: TColorBox;
    SynEdit: TSynEdit;
    ComboFontsEd: TComboBox;
    Panel2: TPanel;
    Label4: TLabel;
    LabSampleEd: TLabel;
    GroupBox2: TGroupBox;
    CheckGutVis: TCheckBox;
    CheckLines: TCheckBox;
    CheckLeadZeros: TCheckBox;
    SpinGutWidth: TSpinEdit;
    Label5: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    ComboFontsGut: TComboBox;
    Panel3: TPanel;
    LabSampleGut: TLabel;
    Label11: TLabel;
    ComboFontSizeEd: TComboBox;
    ComboFontSizeGut: TComboBox;
    Label6: TLabel;
    SpinEdgePos: TSpinEdit;
    Label10: TLabel;
    Label12: TLabel;
    GroupBox4: TGroupBox;
    CheckBold: TCheckBox;
    CheckItalic: TCheckBox;
    CheckUnderline: TCheckBox;
    Label13: TLabel;
    CheckDouble: TCheckBox;
    CheckFind: TCheckBox;
    Label3: TLabel;
    SpinUndo: TSpinEdit;
    Panel4: TPanel;
    Label14: TLabel;
    SpinTab: TSpinEdit;
    CheckWrap: TCheckBox;
    CheckKeep: TCheckBox;
    Label15: TLabel;
    Label16: TLabel;
    ComboInsCaret: TComboBox;
    ComboOverCaret: TComboBox;
    PasSyn: TSynPasSyn;
    procedure Button1Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ComboFontsGutChange(Sender: TObject);
    procedure ComboFontsEdChange(Sender: TObject);
    procedure ComboFontSizeEdChange(Sender: TObject);
    procedure ComboFontSizeGutChange(Sender: TObject);
    procedure ListElemsClick(Sender: TObject);
    procedure CheckBoldClick(Sender: TObject);
    procedure CheckItalicClick(Sender: TObject);
    procedure CheckUnderlineClick(Sender: TObject);
    procedure ColorForeChange(Sender: TObject);
    procedure ColorBackChange(Sender: TObject);
    procedure PagesChange(Sender: TObject);
  private
    procedure EnumFonts(List: TStrings);
    procedure EnumFontSizes(const FontName: string; List: TStrings);
  public
    procedure UpdateGeneral;
    procedure UpdateDisplay;
    procedure UpdateColors;
    procedure ApplyGeneral;
    procedure ApplyDisplay;
    procedure ApplyColors;
  end;

var
  EditorOptionsForm: TEditorOptionsForm;

implementation

uses
  FMain;

{$R *.dfm}

{ TEditorOptionsForm }

procedure TEditorOptionsForm.ApplyColors;
begin
  Options.Editor.ApplySyntaxColors(PasSyn);
end;

procedure TEditorOptionsForm.ApplyDisplay;
var
  I: Long;
begin
  with MainForm.EditorFrame do
  begin
    SynEdit.Font.Name := ComboFontsEd.Text;
    if TryStrToInt(ComboFontSizeEd.Text, I) then
      SynEdit.Font.Size := I;

    SynEdit.Gutter.Font.Name := ComboFontsGut.Text;
    if TryStrToInt(ComboFontSizeGut.Text, I) then
      SynEdit.Gutter.Font.Size := I;

    SynEdit.Gutter.Width := SpinGutWidth.Value;
    SynEdit.RightEdge := SpinEdgePos.Value;

    SynEdit.Gutter.Visible := CheckGutVis.Checked;
    SynEdit.Gutter.ShowLineNumbers := CheckLines.Checked;
    SynEdit.Gutter.LeadingZeros := CheckLeadZeros.Checked;
  end;
end;

procedure TEditorOptionsForm.ApplyGeneral;
var
  SOptions: TSynEditorOptions;
begin
  with MainForm.EditorFrame do
  begin
    SynEdit.MaxUndo := SpinUndo.Value;
    SynEdit.TabWidth := SpinTab.Value;
    SynEdit.InsertCaret := TSynEditCaretType(ComboInsCaret.ItemIndex);
    SynEdit.OverwriteCaret := TSynEditCaretType(ComboOverCaret.ItemIndex);

    SOptions := SynEdit.Options;

    SynEdit.InsertMode := CheckInsert.Checked;
    Options.Editor.DblClickLine := CheckDouble.Checked;
    Options.Editor.FindAtCursor := CheckFind.Checked;
    SynEdit.WordWrap := CheckWrap.Checked;
    Options.Editor.BackSpaceUnindents := CheckBackspace.Checked;

    if CheckGroup.Checked then
      Include(SOptions, eoGroupUndo)
    else
      Exclude(SOptions, eoGroupUndo);
    if CheckIndent.Checked then
      Include(SOptions, eoAutoIndent)
    else
      Exclude(SOptions, eoAutoIndent);
    if CheckEOL.Checked then
      Include(SOptions, eoScrollPastEol)
    else
      Exclude(SOptions, eoScrollPastEol);
    if CheckEOF.Checked then
      Include(SOptions, eoScrollPastEof)
    else
      Exclude(SOptions, eoScrollPastEof);
    if not CheckKeep.Checked then
      Include(SOptions, eoTrimTrailingSpaces)
    else
      Exclude(SOptions, eoTrimTrailingSpaces);
    if not CheckUseTab.Checked then
      Include(SOptions, eoTabsToSpaces)
    else
      Exclude(SOptions, eoTabsToSpaces);
    if CheckSmartTabs.Checked then
    begin
      Include(SOptions, eoSmartTabs);
      Include(SOptions, eoSmartTabDelete);
    end
    else
    begin
      Exclude(SOptions, eoSmartTabs);
      Exclude(SOptions, eoSmartTabDelete);
    end;
    if CheckHome.Checked then
      Include(SOptions, eoEnhanceHomeKey)
    else
      Exclude(SOptions, eoEnhanceHomeKey);
    if CheckHalfPage.Checked then
      Include(SOptions, eoHalfPageScroll)
    else
      Exclude(SOptions, eoHalfPageScroll);
    if CheckScrolls.Checked then
      Include(SOptions, eoHideShowScrollbars)
    else
      Exclude(SOptions, eoHideShowScrollbars);
    if CheckHint.Checked then
      Include(SOptions, eoShowScrollHint)
    else
      Exclude(SOptions, eoShowScrollHint);

    SynEdit.Options := SOptions;
  end;
end;

procedure TEditorOptionsForm.UpdateColors;
var
  I: Long;
begin
  PasSyn.Assign(Options.Editor.SyntaxColors);
  ListElems.Items.Clear;
  for I := 0 to PasSyn.AttrCount - 1 do
    ListElems.Items.Add(PasSyn.Attribute[I].Name);
  ListElems.ItemIndex := 0;
  ListElemsClick(Self);
end;

procedure TEditorOptionsForm.UpdateDisplay;
begin
  with MainForm.EditorFrame do
  begin
    EnumFonts(ComboFontsEd.Items);
    ComboFontsEd.ItemIndex := ComboFontsEd.Items.IndexOf(SynEdit.Font.Name);
    ComboFontsEdChange(Self);
    ComboFontSizeEdChange(Self);

    EnumFonts(ComboFontsGut.Items);
    ComboFontsGut.ItemIndex :=
      ComboFontsGut.Items.IndexOf(SynEdit.Gutter.Font.Name);
    ComboFontsGutChange(Self);
    ComboFontSizeGutChange(Self);

    SpinGutWidth.Value := SynEdit.Gutter.Width;
    SpinEdgePos.Value := SynEdit.RightEdge;

    CheckGutVis.Checked := SynEdit.Gutter.Visible;
    CheckLines.Checked := SynEdit.Gutter.ShowLineNumbers;
    CheckLeadZeros.Checked := SynEdit.Gutter.LeadingZeros;
  end;
end;

procedure TEditorOptionsForm.UpdateGeneral;
begin
  with MainForm.EditorFrame do
  begin
    SpinUndo.Value := SynEdit.MaxUndo;
    SpinTab.Value := SynEdit.TabWidth;
    ComboInsCaret.ItemIndex := Long(SynEdit.InsertCaret);
    ComboOverCaret.ItemIndex := Long(SynEdit.OverwriteCaret);

    CheckInsert.Checked := SynEdit.InsertMode;
    CheckGroup.Checked := eoGroupUndo in SynEdit.Options;
    CheckIndent.Checked := eoAutoIndent in SynEdit.Options;
    CheckEOL.Checked := eoScrollPastEol in SynEdit.Options;
    CheckEOF.Checked := eoScrollPastEof in SynEdit.Options;
    CheckKeep.Checked := not (eoTrimTrailingSpaces in SynEdit.Options);
    CheckUseTab.Checked := not (eoTabsToSpaces in SynEdit.Options);
    CheckSmartTabs.Checked := eoSmartTabs in SynEdit.Options;
    CheckHome.Checked := eoEnhanceHomeKey in SynEdit.Options;
    CheckHalfPage.Checked := eoHalfPageScroll in SynEdit.Options;
    CheckScrolls.Checked := eoHideShowScrollbars in SynEdit.Options;
    CheckHint.Checked := eoShowScrollHint in SynEdit.Options;
    CheckWrap.Checked := SynEdit.WordWrap;

    CheckDouble.Checked := Options.Editor.DblClickLine;
    CheckFind.Checked := Options.Editor.FindAtCursor;
    CheckBackSpace.Checked := Options.Editor.BackSpaceUnindents;
  end;
end;

procedure TEditorOptionsForm.Button1Click(Sender: TObject);
begin
  ApplyGeneral;
  ApplyDisplay;
  ApplyColors;
end;

procedure TEditorOptionsForm.FormShow(Sender: TObject);
begin
  UpdateGeneral;
  UpdateDisplay;
  UpdateColors;
  PagesChange(Self);
end;

function EnumFontFamilyProc(LogFont: PEnumLogFont; var TextMetric:
  PNewTextMetric;
  FontType: Long; LParam: Long): Long; stdcall;
begin
  if ((LogFont.elfLogFont.lfPitchAndFamily and FF_MODERN) = FF_MODERN) or
    ((LogFont.elfLogFont.lfPitchAndFamily and FIXED_PITCH) = FIXED_PITCH) then
    TStrings(LParam).Add(LogFont.elfLogFont.lfFaceName);
  Result := -1;
end;

function EnumFontSizeProc(LogFont: PEnumLogFont; var TextMetric: PNewTextMetric;
  FontType: Long; LParam: Long): Long; stdcall;
var
  Size: string;
begin
  Result := 1;
  if (FontType and RASTER_FONTTYPE) = RASTER_FONTTYPE then
  begin
    Size := IntToStr(LogFont.elfLogFont.lfHeight * 72 div LOGPIXELSY);
    if TStrings(LParam).IndexOf(Size) = -1 then
      TStrings(LParam).Add(Size);
  end;
end;

procedure TEditorOptionsForm.EnumFonts(List: TStrings);
var
  DC: HDC;
begin
  DC := GetDC(0);
  List.Clear;
  EnumFontFamilies(DC, nil, @EnumFontFamilyProc, Long(List));
  ReleaseDC(0, DC);
end;

procedure TEditorOptionsForm.EnumFontSizes(const FontName: string; List:
  TStrings);
var
  DC: HDC;
  I, J: Long;
begin
  DC := GetDC(0);
  List.Clear;
  EnumFontFamilies(DC, PChar(FontName), @EnumFontSizeProc, Long(List));
  if List.Count = 0 then
    for I := 6 to 30 do
      List.Add(IntToStr(I))
  else
    for I := 1 to 3 do
      for J := Pred(List.Count) downto 1 do
        if StrToInt(List[J]) < StrToInt(List[J - 1]) then
          List.Exchange(J, J - 1);
  ReleaseDC(0, DC);
end;

procedure TEditorOptionsForm.ComboFontsGutChange(Sender: TObject);
begin
  EnumFontSizes(ComboFontsGut.Text, ComboFontSizeGut.Items);
  ComboFontSizeGut.Text := IntToStr(SynEdit.Gutter.Font.Size);
  LabSampleGut.Font.Name := ComboFontsGut.Text;
end;

procedure TEditorOptionsForm.ComboFontsEdChange(Sender: TObject);
begin
  EnumFontSizes(ComboFontsEd.Text, ComboFontSizeEd.Items);
  ComboFontSizeEd.Text := IntToStr(SynEdit.Font.Size);
  LabSampleEd.Font.Name := ComboFontsEd.Text;
end;

procedure TEditorOptionsForm.ComboFontSizeEdChange(Sender: TObject);
var
  I: Long;
begin
  if TryStrToInt(ComboFontSizeEd.Text, I) then
    LabSampleEd.Font.Size := I;
end;

procedure TEditorOptionsForm.ComboFontSizeGutChange(Sender: TObject);
var
  I: Long;
begin
  if TryStrToInt(ComboFontSizeGut.Text, I) then
    LabSampleGut.Font.Size := I;
end;

procedure TEditorOptionsForm.ListElemsClick(Sender: TObject);
begin
  if ListElems.ItemIndex >= 0 then
  begin
    ColorFore.Selected := PasSyn.Attribute[ListElems.ItemIndex].Foreground;
    ColorBack.Selected := PasSyn.Attribute[ListElems.ItemIndex].Background;
    CheckBold.Checked := fsBold in PasSyn.Attribute[ListElems.ItemIndex].Style;
    CheckItalic.Checked := fsItalic in
      PasSyn.Attribute[ListElems.ItemIndex].Style;
    CheckUnderline.Checked := fsUnderline in
      PasSyn.Attribute[ListElems.ItemIndex].Style;
  end;
end;

procedure TEditorOptionsForm.CheckBoldClick(Sender: TObject);
begin
  if ListElems.ItemIndex >= 0 then
    with PasSyn.Attribute[ListElems.ItemIndex] do
      if CheckBold.Checked then
        Style := Style + [fsBold]
      else
        Style := Style - [fsBold]
end;

procedure TEditorOptionsForm.CheckItalicClick(Sender: TObject);
begin
  if ListElems.ItemIndex >= 0 then
    with PasSyn.Attribute[ListElems.ItemIndex] do
      if CheckItalic.Checked then
        Style := Style + [fsItalic]
      else
        Style := Style - [fsItalic];
end;

procedure TEditorOptionsForm.CheckUnderlineClick(Sender: TObject);
begin
  if ListElems.ItemIndex >= 0 then
    with PasSyn.Attribute[ListElems.ItemIndex] do
      if CheckUnderline.Checked then
        Style := Style + [fsUnderline]
      else
        Style := Style - [fsUnderline];
end;

procedure TEditorOptionsForm.ColorForeChange(Sender: TObject);
begin
   if ListElems.ItemIndex >= 0 then
    PasSyn.Attribute[ListElems.ItemIndex].Foreground:=ColorFore.Selected;
end;

procedure TEditorOptionsForm.ColorBackChange(Sender: TObject);
begin
   if ListElems.ItemIndex >= 0 then
    PasSyn.Attribute[ListElems.ItemIndex].Background:=ColorBack.Selected;
end;

procedure TEditorOptionsForm.PagesChange(Sender: TObject);
begin
  Wizard.ActivePage:=Pages.ActivePageIndex;
end;

end.

