unit FEnvOptions;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, StdCtrls, ExtCtrls, WizardHeader, CSingletons,
  CBase, CProjects, Kernel, ABase, AConsts, Platforms, AMainActions,
  ActnMan, FData, AOptions, ActnList, ACompilers, Objects, Files, FileCtrl;

type
  TEnvForm = class(TForm)
    Panel1: TPanel;
    Button1: TButton;
    Button2: TButton;
    Label5: TLabel;
    Label6: TLabel;
    Wizard: TWizardHeader;
    Pages: TPageControl;
    TabSheet3: TTabSheet;
    CheckAutoSave: TCheckBox;
    TabSheet4: TTabSheet;
    TabSheet2: TTabSheet;
    Label4: TLabel;
    Label7: TLabel;
    TabSheet5: TTabSheet;
    TabSheet1: TTabSheet;
    Label3: TLabel;
    ListPaths: TListBox;
    Button5: TButton;
    Button6: TButton;
    Group: TGroupBox;
    ListExt: TListBox;
    LabExt: TLabel;
    Button7: TButton;
    BtnDelFType: TButton;
    ComboTypes: TComboBox;
    Label8: TLabel;
    BtnAddExt: TButton;
    BtnDelExt: TButton;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    Panel2: TPanel;
    RadioText: TRadioButton;
    RadioBinary: TRadioButton;
    Panel3: TPanel;
    RadioOpenToEd: TRadioButton;
    RadioOpenAssoc: TRadioButton;
    Label12: TLabel;
    ComboLighter: TComboBox;
    LabName: TLabel;
    BtnRename: TButton;
    ComboStyle: TComboBox;
    ComboColors: TComboBox;
    Label13: TLabel;
    ComboCompilers: TComboBox;
    GroupBox1: TGroupBox;
    Label14: TLabel;
    EdRoot: TEdit;
    Button8: TButton;
    Label15: TLabel;
    CheckRemWindow: TCheckBox;
    CheckRemFiles: TCheckBox;
    CheckProgress: TCheckBox;
    CheckMinimize: TCheckBox;
    Button3: TButton;
    Button4: TButton;
    Button9: TButton;
    Button10: TButton;
    Actions: TActionList;
    CheckPath: TAction;
    EdPath: TEdit;
    CheckIndex: TAction;
    Button11: TButton;
    CheckRoot: TAction;
    Label1: TLabel;
    CheckReplace: TAction;
    CheckAdd: TAction;
    procedure FormShow(Sender: TObject);
    procedure ComboTypesChange(Sender: TObject);
    procedure BtnRenameClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure BtnAddExtClick(Sender: TObject);
    procedure RadioTextClick(Sender: TObject);
    procedure RadioBinaryClick(Sender: TObject);
    procedure RadioOpenToEdClick(Sender: TObject);
    procedure RadioOpenAssocClick(Sender: TObject);
    procedure ComboLighterChange(Sender: TObject);
    procedure BtnDelExtClick(Sender: TObject);
    procedure Button7Click(Sender: TObject);
    procedure BtnDelFTypeClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure PagesChange(Sender: TObject);
    procedure CheckPathUpdate(Sender: TObject);
    procedure CheckIndexUpdate(Sender: TObject);
    procedure ComboCompilersChange(Sender: TObject);
    procedure CheckRootUpdate(Sender: TObject);
    procedure Button8Click(Sender: TObject);
    procedure EdRootChange(Sender: TObject);
    procedure Button10Click(Sender: TObject);
    procedure CheckReplaceUpdate(Sender: TObject);
    procedure CheckAddUpdate(Sender: TObject);
    procedure Button5Click(Sender: TObject);
    procedure Button6Click(Sender: TObject);
    procedure Button11Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure Button9Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure ListPathsMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ListPathsClick(Sender: TObject);
  private
    FTypes: TVideFileTypeList;
    FRoots: TNStringList;
    function CheckName(const Name: string): Boolean;
  public
    procedure UpdateFileTypes;
    procedure ApplyFileTypes;
    procedure UpdateAppearance;
    procedure ApplyAppearance;
    procedure UpdatePaths;
    procedure ApplyPaths;
    procedure UpdateCompilers;
    procedure ApplyCompilers;
    procedure UpdatePrefs;
    procedure ApplyPrefs;
  end;

var
  EnvForm: TEnvForm;

implementation

uses
  FMain;

{$R *.dfm}

procedure TEnvForm.FormShow(Sender: TObject);
begin
  UpdatePrefs;
  UpdateFileTypes;
  UpdateAppearance;
  UpdateCompilers;
  UpdatePaths;
  PagesChange(Self);
end;

procedure TEnvForm.UpdateFileTypes;
var
  I: Long;
  Syn: TTextType;
begin
  FTypes.Clear;
  for I := 0 to FileTypes.Count - 1 do
    FTypes.Add(FileTypes.Types[I].Clone);
  ComboTypes.Items.Clear;
  for I := 0 to FTypes.Count - 1 do
    ComboTypes.Items.AddObject(FTypes.Types[I].Name +
      Iff(FTypes.Types[I].Fixed, ' (Fixed)', ''), FileTypes.Types[I]);

  ComboLighter.Items.Clear;
  for Syn := ttNone to ttINI do
    ComboLighter.Items.AddObject(SyntaxNames[Syn], TObject(Syn));

  ComboTypes.ItemIndex := 0;
  ComboTypesChange(Self);
end;

procedure TEnvForm.ComboTypesChange(Sender: TObject);
var
  FType: TVideFileType;
begin
  FType := FTypes.Types[ComboTypes.ItemIndex];
  if FType <> nil then
  begin
    LabName.Caption := FType.Name;
    RadioText.Checked := FType.Format = ffText;
    RadioBinary.Checked := FType.Format = ffBinary;
    RadioOpenToEd.Checked := FType.CanOpenToEditor;
    RadioOpenAssoc.Checked := not FType.CanOpenToEditor;
    ComboLighter.ItemIndex := Long(FType.TextType);
    ListExt.Items.Text := FType.Extensions.Text;

    BtnRename.Enabled := not FType.Fixed;
    BtnAddExt.Enabled := not FType.Fixed;
    BtnDelExt.Enabled := not FType.Fixed;
    BtnDelFType.Enabled := not FType.Fixed;
    ComboLighter.Enabled := not FType.Fixed;
    RadioText.Enabled := not FType.Fixed;
    RadioBinary.Enabled := not FType.Fixed;
    RadioOpenToEd.Enabled := not FType.Fixed;
    RadioOpenAssoc.Enabled := not FType.Fixed;
  end;
end;

function TEnvForm.CheckName(const Name: string): Boolean;
begin
  Result := not ((FTypes.TypesName[Name] <> nil) and
    (FTypes.TypesName[Name] <> FTypes.Types[ComboTypes.ItemIndex]));
end;

procedure TEnvForm.BtnRenameClick(Sender: TObject);
var
  S: string;
  Idx: Long;
begin
  S := LabName.Caption;
  if Dialogs.InputQuery('Rename File Type',
    'Enter name of the selected file type:', S) then
  begin
    if not CheckName(S) then
      MainActions.ErrorDlg(Format('File type named ''%s'' already exists.',
        [S]), 'Rename File Type')
    else
    begin
      Idx := ComboTypes.ItemIndex;
      if Idx >= 0 then
      begin
        LabName.Caption := S;
        FTypes.Types[Idx].Name := S;
        ComboTypes.Items[Idx] := S;
        ComboTypes.ItemIndex := Idx;
      end;
    end;
  end;
end;

procedure TEnvForm.FormCreate(Sender: TObject);
begin
  FTypes := TVideFileTypeList.Create;
  FRoots := TNStringList.Create;
end;

procedure TEnvForm.FormDestroy(Sender: TObject);
begin
  FTypes.Destroy;
  FRoots.Free;
end;

procedure TEnvForm.BtnAddExtClick(Sender: TObject);
var
  S: string;
begin
  if Dialogs.InputQuery('Add File Extension',
    'Enter name of new extension:', S) then
  begin
    FTypes.Types[ComboTypes.ItemIndex].Extensions.Add(S);
    ListExt.Items.Add(S);
  end;
end;

procedure TEnvForm.RadioTextClick(Sender: TObject);
begin
  FTypes.Types[ComboTypes.ItemIndex].Format := ffText;
end;

procedure TEnvForm.RadioBinaryClick(Sender: TObject);
begin
  FTypes.Types[ComboTypes.ItemIndex].Format := ffBinary;
end;

procedure TEnvForm.RadioOpenToEdClick(Sender: TObject);
begin
  FTypes.Types[ComboTypes.ItemIndex].CanOpenToEditor := True;
end;

procedure TEnvForm.RadioOpenAssocClick(Sender: TObject);
begin
  FTypes.Types[ComboTypes.ItemIndex].CanOpenToEditor := False;
end;

procedure TEnvForm.ComboLighterChange(Sender: TObject);
begin
  FTypes.Types[ComboTypes.ItemIndex].TextType :=
    TTextType(ComboLighter.Items.Objects[ComboLighter.ItemIndex]);
end;

procedure TEnvForm.BtnDelExtClick(Sender: TObject);
begin
  if ListExt.ItemIndex >= 0 then
  begin
    FTypes.Delete(ListExt.ItemIndex);
    ListExt.Items.Delete(ListExt.ItemIndex);
  end;
end;

procedure TEnvForm.Button7Click(Sender: TObject);
var
  S: string;
  FType: TVideFileType;
begin
  S := 'New Type';
  if Dialogs.InputQuery('Add File Type',
    'Enter name of the new file type:', S) then
  begin
    if not CheckName(S) then
      MainActions.ErrorDlg(Format('File type named ''%s'' already exists.',
        [S]), 'Add File Type')
    else
    begin
      FType := TVideFileType.Create;
      FType.Name := S;
      FTypes.Add(FType);
      ComboTypes.Items.AddObject(S, nil);
      ComboTypes.ItemIndex := ComboTypes.Items.Count - 1;
      ComboTypesChange(Self);
    end;
  end;
end;

procedure TEnvForm.BtnDelFTypeClick(Sender: TObject);
var
  Idx: Long;
begin
  Idx := ComboTypes.ItemIndex;
  if Idx >= 0 then
  begin
    FTypes.Delete(Idx);
    ComboTypes.Items.Delete(Idx);
    if Idx = ComboTypes.Items.Count then
      Dec(Idx);
    ComboTypes.ItemIndex := Idx;
    ComboTypesChange(Self);
  end;
end;

procedure TEnvForm.Button1Click(Sender: TObject);
begin
  ApplyPrefs;
  ApplyFileTypes;
  ApplyAppearance;
  ApplyCompilers;
  ApplyPaths;
end;

procedure TEnvForm.ApplyFileTypes;
var
  I: Long;
  Orig: TVideFileType;
begin
  for I := 0 to FTypes.Count - 1 do
  begin
    Orig := TVideFileType(ComboTypes.Items.Objects[I]);
    if Orig <> nil then
      Orig.Assign(FTypes.Types[I])
    else
      FileTypes.Add(FTypes.Types[I].Clone);
  end;
end;

procedure TEnvForm.ApplyAppearance;
var
  Idx: Long;
begin
  Idx := ComboStyle.ItemIndex;
  if Idx >= 0 then
    Options.ActionBarStyle := ActnMan.ActionBarStyles.Style[Idx];

  Idx := ComboColors.ItemIndex;
  if Idx >= 0 then
    Options.ColorMap := ColorMaps[Idx];
end;

procedure TEnvForm.UpdateAppearance;
var
  I: Long;
begin
  ComboStyle.Items.Clear;
  for I := 0 to ActionBarStyles.Count - 1 do
    ComboStyle.Items.AddObject(ActionBarStyles.Style[I].GetStyleName,
      ActionBarStyles.Style[I]);
  ComboStyle.ItemIndex :=
    ActionBarStyles.IndexOfObject(Options.ActionBarStyle);

  ComboColors.Items.Clear;
  for I := 0 to ColorMaps.Count - 1 do
    ComboColors.Items.AddObject(ColorMaps[I].Name, ColorMaps[I].Map);
  ComboColors.ItemIndex := ColorMaps.IndexOf(Options.ColorMap)
end;

procedure TEnvForm.PagesChange(Sender: TObject);
begin
  Wizard.ActivePage := Pages.ActivePageIndex;
end;

procedure TEnvForm.ApplyPaths;
begin
  Options.Paths.Text := ListPaths.Items.Text;
end;

procedure TEnvForm.UpdatePaths;
begin
  ListPaths.Items.Text := Options.Paths.Text;
end;

procedure TEnvForm.ApplyCompilers;
var
  I: Long;
begin
  for I := 0 to Compilers.Count - 1 do
    Compilers[I].Path := FRoots[I];
  Options.Compiler:=Compilers.Find(ComboCompilers.Text);  
end;

procedure TEnvForm.ApplyPrefs;
begin
  Options.AutoSave:=CheckAutoSave.Checked;
  Options.RememberWindow:=CheckRemWindow.Checked;
  Options.RememberFiles:=CheckRemFiles.Checked;
  Options.ShowProgress:=CheckProgress.Checked;
  Options.MinimizeOnRun:=CheckMinimize.Checked;
end;

procedure TEnvForm.UpdateCompilers;
var
  I: Long;
begin
  ComboCompilers.Items.Clear;
  FRoots.Clear;
  for I := 0 to Compilers.Count - 1 do
    ComboCompilers.Items.AddObject(Compilers[I].Name, Compilers[I]);
  for I := 0 to Compilers.Count - 1 do
    FRoots.Add(Compilers[I].Path);

  ComboCompilers.ItemIndex :=
    ComboCompilers.Items.IndexOf(Options.Compiler.Name);
  ComboCompilersChange(Self);
end;

procedure TEnvForm.UpdatePrefs;
begin
  CheckAutoSave.Checked:=Options.AutoSave;
  CheckRemWindow.Checked:=Options.RememberWindow;
  CheckRemFiles.Checked:=Options.RememberFiles;
  CheckProgress.Checked:=Options.ShowProgress;
  CheckMinimize.Checked:=Options.MinimizeOnRun;
end;

procedure TEnvForm.CheckPathUpdate(Sender: TObject);
begin
  if not Vfs.DirectoryExists(EdPath.Text) then
    EdPath.Font.Color := clRed
  else
    EdPath.Font.Color := clWindowText;
end;

procedure TEnvForm.CheckIndexUpdate(Sender: TObject);
begin
  CheckIndex.Enabled := ListPaths.ItemIndex >= 0;
end;

procedure TEnvForm.ComboCompilersChange(Sender: TObject);
begin
  EdRoot.Text := FRoots[ComboCompilers.ItemIndex];
end;

procedure TEnvForm.CheckRootUpdate(Sender: TObject);
begin
  if not Vfs.DirectoryExists(EdRoot.Text) then
    EdRoot.Font.Color := clRed
  else
    EdRoot.Font.Color := clWindowText;
end;

procedure TEnvForm.Button8Click(Sender: TObject);
var
  S: string;
begin
  S := Vfs.GetFullDirName(EdRoot.Text, True);
  if SelectDirectory('Select Directory:', '', S) then
    EdRoot.Text := S;
end;

procedure TEnvForm.EdRootChange(Sender: TObject);
var
  Idx: Long;
begin
  Idx := ComboCompilers.ItemIndex;
  if Idx >= 0 then
    FRoots[Idx] := EdRoot.Text;
end;

procedure TEnvForm.Button10Click(Sender: TObject);
var
  S: string;
begin
  S := Vfs.GetFullDirName(EdPath.Text, True);
  if SelectDirectory('Select Directory:', '', S) then
    EdPath.Text := S;
end;

procedure TEnvForm.CheckReplaceUpdate(Sender: TObject);
begin
  CheckReplace.Enabled := (ListPaths.ItemIndex >= 0) and
    (ListPaths.Items[ListPaths.ItemIndex] <> EdPath.Text) and
    Vfs.DirectoryExists(EdPath.Text);
end;

procedure TEnvForm.CheckAddUpdate(Sender: TObject);
begin
  CheckAdd.Enabled := (ListPaths.Items.IndexOf(EdPath.Text) = -1)
    and (EdPath.Text <> '') and Vfs.DirectoryExists(EdPath.Text);
end;

procedure TEnvForm.Button5Click(Sender: TObject);
begin
  ListPaths.Items.Add(EdPath.Text);
end;

procedure TEnvForm.Button6Click(Sender: TObject);
begin
  ListPaths.DeleteSelected;
  EdPath.Text := '';
end;

procedure TEnvForm.Button11Click(Sender: TObject);
var
  Idx: Long;
begin
  Idx := ListPaths.ItemIndex;
  if Idx >= 0 then
  begin
    ListPaths.Items[Idx] := EdPath.Text;
    ListPaths.ItemIndex := Idx;
  end;
end;

procedure TEnvForm.Button4Click(Sender: TObject);
var
  Idx: Long;
begin
  Idx := ListPaths.ItemIndex;
  if (Idx > 0) and (ListPaths.Items.Count > 1) then
  begin
    ListPaths.Items.Exchange(Idx - 1, Idx);
    ListPaths.ItemIndex := Idx - 1;
  end;
end;

procedure TEnvForm.Button9Click(Sender: TObject);
var
  Idx: Long;
begin
  Idx := ListPaths.ItemIndex;
  if (Idx < ListPaths.Items.Count - 1) and (ListPaths.Items.Count > 1) then
  begin
    ListPaths.Items.Exchange(Idx + 1, Idx);
    ListPaths.ItemIndex := Idx + 1;
  end;
end;

procedure TEnvForm.Button3Click(Sender: TObject);
var
  I: Long;
begin
  for I := 0 to ListPaths.Items.Count - 1 do
  begin
    if I = ListPaths.Count then
      Break;
    if not Vfs.DirectoryExists(ListPaths.Items[I]) then
      ListPaths.Items.Delete(I);
  end;
end;

procedure TEnvForm.ListPathsMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  Idx: Long;
begin
  Idx := ListPaths.ItemIndex;
  if Idx >= 0 then
    EdPath.Text := ListPaths.Items[Idx];
end;

procedure TEnvForm.ListPathsClick(Sender: TObject);
var
  Idx: Long;
begin
  Idx := ListPaths.ItemIndex;
  if Idx >= 0 then
    EdPath.Text := ListPaths.Items[Idx];
end;

end.

