unit FProjectOptions;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, ComCtrls, WizardHeader, CProjects, Kernel,
  CBase, AMainActions, ACompilers, AOptions, Files, FileCtrl, ActnList;

type
  TProjectForm = class(TForm)
    l: TPanel;
    Button1: TButton;
    Button2: TButton;
    Pages: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    Label1: TLabel;
    ComboCompilers: TComboBox;
    GroupBox1: TGroupBox;
    CheckEval: TCheckBox;
    CheckExtSyn: TCheckBox;
    CheckAnsi: TCheckBox;
    CheckTyped: TCheckBox;
    CheckHint: TCheckBox;
    CheckWarn: TCheckBox;
    CheckDebug: TCheckBox;
    CheckAssert: TCheckBox;
    Label2: TLabel;
    Label3: TLabel;
    EdName: TEdit;
    TabSheet3: TTabSheet;
    Label4: TLabel;
    ListPaths: TListBox;
    Wizard: TWizardHeader;
    EdCmdLine: TEdit;
    CheckCppExt: TCheckBox;
    GroupMode: TRadioGroup;
    GroupCPU: TRadioGroup;
    CheckOver: TCheckBox;
    CheckIO: TCheckBox;
    CheckRange: TCheckBox;
    EdOutputDir: TLabeledEdit;
    Button5: TButton;
    Label6: TLabel;
    Label7: TLabel;
    ComboEnum: TComboBox;
    ComboRecord: TComboBox;
    MemoDesc: TMemo;
    Label8: TLabel;
    GroupType: TRadioGroup;
    Button3: TButton;
    Button6: TButton;
    EdPath: TEdit;
    Button4: TButton;
    Button7: TButton;
    Button9: TButton;
    Button10: TButton;
    Button11: TButton;
    ActionList1: TActionList;
    CheckAdd: TAction;
    CheckReplace: TAction;
    CheckIndex: TAction;
    procedure Button1Click(Sender: TObject);
    procedure PagesChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure EdOutputDirChange(Sender: TObject);
    procedure Button5Click(Sender: TObject);
    procedure CheckAddUpdate(Sender: TObject);
    procedure CheckReplaceUpdate(Sender: TObject);
    procedure CheckIndexUpdate(Sender: TObject);
    procedure ListPathsClick(Sender: TObject);
    procedure ListPathsMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Button11Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button6Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure Button7Click(Sender: TObject);
    procedure Button9Click(Sender: TObject);
    procedure Button10Click(Sender: TObject);
  private
    FProject: TVideProject;
  public
    procedure ShowOptions(Project: TVideProject);
    function ApplyGeneral:Boolean;
    procedure ApplyCompiler;
    procedure ApplyDirs;
    procedure UpdateGeneral;
    procedure UpdateCompiler;
    procedure UpdateDirs;
  end;

var
  ProjectForm: TProjectForm;

implementation

{$R *.dfm}

procedure TProjectForm.Button1Click(Sender: TObject);
begin
  if ApplyGeneral then
  begin
  ApplyCompiler;
  ApplyDirs;
  ModalResult:=mrOK;
  Close;
  end;
end;

procedure TProjectForm.PagesChange(Sender: TObject);
begin
  Wizard.ActivePage:=Pages.ActivePageIndex;
end;

procedure TProjectForm.FormShow(Sender: TObject);
begin
  UpdateGeneral;
  UpdateCompiler;
  UpdateDirs;
  PagesChange(Self);
end;

procedure TProjectForm.ApplyCompiler;
begin
with FProject.CompilerSettings do
begin
  Compiler:=Compilers.Find(ComboCompilers.Text);

  CmdLine:=EdCmdLine.Text;

  Mode:=TCompilerMode(GroupMode.ItemIndex);
  TargetCPU:=TTargetCPU(GroupCPU.ItemIndex);
  MinEnumSize:=StrToInt(ComboEnum.Text);
  RecordAlign:=StrToInt(ComboRecord.Text);

  BooleanEval:=CheckEval.Checked;
  TypedPointers:=CheckTyped.Checked;
  ExtSyntax:=CheckExtSyn.Checked;
  AnsiStrings:=CheckAnsi.Checked;
  CppExtensions:=CheckCppExt.Checked;
  DebugInfo:=CheckDebug.Checked;
  Assertions:=CheckAssert.Checked;
  Warnings:=CheckWarn.Checked;
  Hints:=CheckHint.Checked;
  RangeCheck:=CheckRange.Checked;
  IOCheck:=CheckIO.Checked;
  OverflowCheck:=CheckOver.Checked;
 end;
end;

procedure TProjectForm.ApplyDirs;
begin
  FProject.OutputDir:=EdOutputDir.Text;
  FProject.Paths.Text:=ListPaths.Items.Text;
end;

function TProjectForm.ApplyGeneral:Boolean;
begin
  Result:=False;
  if EdName.Text <> FProject.Name then
    if not MainActions.ProjectRename(FProject, EdName.Text) then
      Exit;
  FProject.Description:=MemoDesc.Lines.Text;
  FProject.ProjectType:=TProjectType(GroupType.ItemIndex);
  Result:=True;
end;

procedure TProjectForm.ShowOptions(Project: TVideProject);
begin
  FProject:=Project;
  ShowModal;
end;

procedure TProjectForm.UpdateCompiler;
var
  I:Long;
begin
  ComboCompilers.Items.Clear;
  for I:=0 to Compilers.Count-1 do
    ComboCompilers.Items.AddObject(Compilers[I].Name, Compilers[I]);
  ComboCompilers.ItemIndex:=Compilers.IndexOf(FProject.CompilerSettings.Compiler);

  EdCmdLine.Text:=FProject.CompilerSettings.CmdLine;

  GroupMode.ItemIndex:=Long(FProject.CompilerSettings.Mode);
  GroupCPU.ItemIndex:=Long(FProject.CompilerSettings.TargetCPU);
  ComboEnum.ItemIndex:=ComboEnum.Items.IndexOf(IntToStr(FProject.CompilerSettings.MinEnumSize));
  ComboRecord.ItemIndex:=ComboRecord.Items.IndexOf(IntToStr(FProject.CompilerSettings.RecordAlign));

  CheckEval.Checked:=FProject.CompilerSettings.BooleanEval;
  CheckTyped.Checked:=FProject.CompilerSettings.TypedPointers;
  CheckExtSyn.Checked:=FProject.CompilerSettings.ExtSyntax;
  CheckAnsi.Checked:=FProject.CompilerSettings.AnsiStrings;
  CheckCppExt.Checked:=FProject.CompilerSettings.CppExtensions;
  CheckDebug.Checked:=FProject.CompilerSettings.DebugInfo;
  CheckAssert.Checked:=FProject.CompilerSettings.Assertions;
  CheckWarn.Checked:=FProject.CompilerSettings.Warnings;
  CheckHint.Checked:=FProject.CompilerSettings.Hints;
  CheckRange.Checked:=FProject.CompilerSettings.RangeCheck;
  CheckIO.Checked:=FProject.CompilerSettings.IOCheck;
  CheckOver.Checked:=FProject.CompilerSettings.OverflowCheck;
end;

procedure TProjectForm.UpdateDirs;
begin
  EdOutputDir.Text:=FProject.OutputDir;
  ListPaths.Items.Text:=FProject.Paths.Text;
end;

procedure TProjectForm.UpdateGeneral;
begin
  EdName.Text:=FProject.Name;
  MemoDesc.Lines.Text:=FProject.Description;
  GroupType.ItemIndex:=Long(FProject.ProjectType);
end;

procedure TProjectForm.EdOutputDirChange(Sender: TObject);
begin
   if not Vfs.DirectoryExists(EdOutputDir.Text) then
    EdOutputDir.Font.Color := clRed
  else
    EdOutputDir.Font.Color := clWindowText;
end;

procedure TProjectForm.Button5Click(Sender: TObject);
var
  S: string;
begin
  S := Vfs.GetFullDirName(EdOutputDir.Text, True);
  if SelectDirectory('Select Directory:', '', S) then
    EdOutputDir.Text := S;
end;

procedure TProjectForm.CheckAddUpdate(Sender: TObject);
begin
 CheckAdd.Enabled := (ListPaths.Items.IndexOf(EdPath.Text) = -1)
    and (EdPath.Text <> '') and Vfs.DirectoryExists(EdPath.Text);
end;

procedure TProjectForm.CheckReplaceUpdate(Sender: TObject);
begin
 CheckReplace.Enabled := (ListPaths.ItemIndex >= 0) and
    (ListPaths.Items[ListPaths.ItemIndex] <> EdPath.Text) and
    Vfs.DirectoryExists(EdPath.Text);
end;

procedure TProjectForm.CheckIndexUpdate(Sender: TObject);
begin
  CheckIndex.Enabled := ListPaths.ItemIndex >= 0;
end;

procedure TProjectForm.ListPathsClick(Sender: TObject);
var
  Idx: Long;
begin
  Idx := ListPaths.ItemIndex;
  if Idx >= 0 then
    EdPath.Text := ListPaths.Items[Idx];
end;

procedure TProjectForm.ListPathsMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  Idx: Long;
begin
  Idx := ListPaths.ItemIndex;
  if Idx >= 0 then
    EdPath.Text := ListPaths.Items[Idx];
end;

procedure TProjectForm.Button11Click(Sender: TObject);
var
  Idx: Long;
begin
  Idx := ListPaths.ItemIndex;
  if Idx >= 0 then
  begin
    ListPaths.Items[Idx] := EdPath.Text;
    ListPaths.ItemIndex := Idx;
  end;
end;

procedure TProjectForm.Button3Click(Sender: TObject);
begin
  ListPaths.Items.Add(EdPath.Text);
end;

procedure TProjectForm.Button6Click(Sender: TObject);
begin
 ListPaths.DeleteSelected;
  EdPath.Text := '';
end;

procedure TProjectForm.Button4Click(Sender: TObject);
var
  I: Long;
begin
  for I := 0 to ListPaths.Items.Count - 1 do
  begin
    if I = ListPaths.Count then
      Break;
    if not Vfs.DirectoryExists(ListPaths.Items[I]) then
      ListPaths.Items.Delete(I);
  end;
end;

procedure TProjectForm.Button7Click(Sender: TObject);
var
  Idx: Long;
begin
  Idx := ListPaths.ItemIndex;
  if (Idx > 0) and (ListPaths.Items.Count > 1) then
  begin
    ListPaths.Items.Exchange(Idx - 1, Idx);
    ListPaths.ItemIndex := Idx - 1;
  end;
end;

procedure TProjectForm.Button9Click(Sender: TObject);
var
  Idx: Long;
begin
  Idx := ListPaths.ItemIndex;
  if (Idx < ListPaths.Items.Count - 1) and (ListPaths.Items.Count > 1) then
  begin
    ListPaths.Items.Exchange(Idx + 1, Idx);
    ListPaths.ItemIndex := Idx + 1;
  end;
end;

procedure TProjectForm.Button10Click(Sender: TObject);
var
  S: string;
begin
  S := Vfs.GetFullDirName(EdPath.Text, True);
  if SelectDirectory('Select Directory:', '', S) then
    EdPath.Text := S;
end;

end.

