unit FSaveChanges;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, ExtCtrls, WizardHeader, CheckLst, Kernel,
  AConsts, ABase, CSingletons, Lists, CProjects;

type
  TSaveForm = class(TForm)
    Panel1: TPanel;
    Panel3: TPanel;
    BtnAll: TButton;
    Button2: TButton;
    Button3: TButton;
    Button4: TButton;
    WizardHeader1: TWizardHeader;
    Panel2: TPanel;
    Panel4: TPanel;
    Label1: TLabel;
    ListSave: TCheckListBox;
    procedure ListSaveMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Button2Click(Sender: TObject);
    procedure BtnAllClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ListSaveClickCheck(Sender: TObject);
  private
    FAllowedItems: TSaveItems;
    FCheckWidth: Long;
    procedure FillList;
    procedure CheckSubItems(Idx:Long);
  public
    Items: TNObjectList;

    function SaveModifiedItems(OnlyModified: Boolean; AllowedItems: TSaveItems):
      Boolean;
  end;

var
  SaveForm: TSaveForm;

implementation

{$R *.dfm}

procedure TSaveForm.ListSaveMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  Idx: Long;
  R: TRect;
begin
  Idx := ListSave.ItemAtPos(Classes.Point(X, Y), True);
  R := ListSave.ItemRect(Idx);
  Inc(R.Left, FCheckWidth);
  if (Idx >= 0) and PtInRect(R, Classes.Point(X, Y)) then
  begin
    ListSave.Checked[Idx] := not ListSave.Checked[Idx];
    CheckSubItems(Idx);
  end;
end;

procedure TSaveForm.Button2Click(Sender: TObject);
var
  I: Long;
begin
  for I := 0 to ListSave.Items.Count - 1 do
    ListSave.Checked[I] := False;
end;

procedure TSaveForm.BtnAllClick(Sender: TObject);
var
  I: Long;
begin
  for I := 0 to ListSave.Items.Count - 1 do
    ListSave.Checked[I] := True;
end;

function TSaveForm.SaveModifiedItems(OnlyModified: Boolean;
  AllowedItems: TSaveItems): Boolean;
var
  Indent, I, Count: Long;
begin
  FAllowedItems := AllowedItems;
  Indent := 0;
  ListSave.Items.Clear;

  if (scSolutions in AllowedItems) and ((not OnlyModified) or
    (Solution.Modified or Solution.Dirty)) then
  begin
    ListSave.Items.AddObject(ExtractFileName(Solution.FileName), Solution);
    Inc(Indent, SaveFormIndent);
  end;

  if (scProjects in AllowedItems) then
  begin
    for I := 0 to Solution.Count - 1 do
      if (not OnlyModified) or (Solution.Projects[I].Modified or
        Solution.Projects[I].Dirty) then
        ListSave.Items.AddObject(StringOfChar(#32, Indent) +
          ExtractFileName(Solution.Projects[I].FileName), Solution.Projects[I]);
  end;

  if (scActiveFile in AllowedItems) and (not (scEditorFiles in AllowedItems))
    and (Editor.ActiveFile <> nil) and ((not OnlyModified) or
    (Editor.ActiveFile.Modified or Editor.ActiveFile.Dirty)) then
  begin
    ListSave.Items.AddObject(ExtractFileName(Editor.ActiveFile.FileName),
      Editor.ActiveFile);
  end;

  if (scEditorFiles in AllowedItems) and (Editor.Count > 0) then
  begin
    Count := 0;
    for I := 0 to Editor.Count - 1 do
      if Editor.Files[I].Modified or Editor.Files[I].Dirty then
        Inc(Count);

    if (Count > 0) or (not OnlyModified) then
    begin
      Indent := 0;
      ListSave.Items.Add(sEditorFiles);
      Inc(Indent, SaveFormIndent);

      for I := 0 to Editor.Count - 1 do
        if (not OnlyModified) or (Editor.Files[I].Modified or
          Editor.Files[I].Dirty) then
          ListSave.Items.AddObject(StringOfChar(#32, Indent) +
            ExtractFileName(Editor.Files[I].FileName), Editor.Files[I]);
    end;
  end;
  Result := ListSave.Count < 1;
  if not Result then
  begin
    Result := ShowModal = mrOK;
    if Result then
      FillList;
  end;
end;

procedure TSaveForm.FormCreate(Sender: TObject);
begin
  FCheckWidth := 0;
  with TBitmap.Create do
  try
    Handle := LoadBitmap(0, PChar(OBM_CHECKBOXES));
    FCheckWidth := Width div 4 + 2;
  finally
    Free;
  end;
  Items := TNObjectList.Create;
end;

procedure TSaveForm.FormShow(Sender: TObject);
begin
  BtnAll.Click;
end;

procedure TSaveForm.FormDestroy(Sender: TObject);
begin
  Items.Free;
end;

procedure TSaveForm.FillList;
var
  I: Long;
begin
  Items.Clear;
  for I := 0 to ListSave.Count - 1 do
    if (ListSave.Items.Objects[I] <> nil) and ListSave.Checked[I] then
      Items.Add(ListSave.Items.Objects[I]);
end;

procedure TSaveForm.ListSaveClickCheck(Sender: TObject);
begin
  CheckSubItems(ListSave.ItemIndex);
end;

procedure TSaveForm.CheckSubItems(Idx: Long);
var
  I: Long;
begin
  if (ListSave.Items[Idx] = sEditorFiles) and
    ListSave.Checked[Idx] then
    for I := Idx + 1 to ListSave.Count - 1 do
      ListSave.Checked[I] := True;

  if ListSave.Checked[Idx] and (ListSave.Items.Objects[Idx] <> nil) then
    if ListSave.Items.Objects[Idx].ClassNameIs(TVideSolution.ClassName) then
    begin
      I := Idx + 1;
      while (I < ListSave.Count) do
        if ListSave.Items[I] <> sEditorFiles then
        begin
          ListSave.Checked[I] := True;
          Inc(I);
        end
        else
          Break;
    end;
end;

end.

